# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate More advanced descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Explore... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
resp = DataVector('Response variable: ')
pred = DataVector('Categorical Predictor: ')
'''

    spssscript = '''
<% import numpy %>

EXAMINE VARIABLES=${resp} BY ${pred}
  /PLOT=NONE
  /COMPARE GROUPS
  /STATISTICS DESCRIPTIVES
  /CINTERVAL 95
  /MISSING LISTWISE
  /NOTOTAL.

* Chart Builder.
GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${pred} ${resp} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${pred}=col(source(s), name("${pred}"), unit.category())
  DATA: ${resp}=col(source(s), name("${resp}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: axis(dim(1), label("${datafile.variables[pred]['varlabel']}"))
  GUIDE: axis(dim(2), label("${datafile.variables[resp]['varlabel']}"))
  SCALE: cat(dim(1), include(${', '.join('\"' + str(v) + '\"' for v in numpy.unique(datafile.variables[pred]['data'].compressed()))}))
  SCALE: linear(dim(2), include(0))
  ELEMENT: schema(position(bin.quantile.letter(${pred}*${resp})), color.interior(color."D3CE97"), label(id))
END GPL.


GRAPH
  /ERRORBAR(CI 95)=${resp} BY ${pred}.

UNIANOVA ${resp} BY ${pred}
  /METHOD=SSTYPE(3)
  /INTERCEPT=INCLUDE
  /POSTHOC=${pred}(TUKEY)
  /POSTHOC=${pred}(LSD)
  /POSTHOC=${pred}(BONFERRONI)
  /PLOT=PROFILE(${pred})
  /PRINT=HOMOGENEITY DESCRIPTIVE
  /CRITERIA=ALPHA(.05)
  /DESIGN=${pred}.

'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

import random

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

out = etree.parse(BytesIO(outputs['Table_Between Subjects Factors.xml'].content))
print(etree.tostring(out, pretty_print=True))

caseout = etree.parse(BytesIO(outputs['Table_Case Processing Summary.xml'].content))
print(etree.tostring(caseout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/@label)"
caserowtext = caseout.xpath(path, namespaces=ns)
print(caserowtext)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/@label)"
casecoltext = caseout.xpath(path, namespaces=ns)
print(casecoltext)

path = "//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/@label"
casecats = caseout.xpath(path, namespaces=ns)
if len(casecats) == 0:
     path = "//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/@text"
     casecats = caseout.xpath(path, namespaces=ns)
print(casecats)

path = "//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Valid']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number"
Nvalid = [int(x) for x in caseout.xpath(path, namespaces=ns)]
print(Nvalid)

path = "//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Missing']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number"
Nmissing = [int(x) for x in caseout.xpath(path, namespaces=ns)]
print(Nmissing)

path = "//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number"
Ntotal = [int(x) for x in caseout.xpath(path, namespaces=ns)]
print(Ntotal)

out = etree.parse(BytesIO(outputs['Table_Descriptive Statistics.xml'].content))
print(etree.tostring(out, pretty_print=True))

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
resplab = descsout.xpath(path, namespaces=ns)
print(resplab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/@label)"
predlab = descsout.xpath(path, namespaces=ns)
print(predlab)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
mean_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(mean_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
mean_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(mean_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
ses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(ses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
ses_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(ses_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
lci_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(lci_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
lci_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(lci_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
uci_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(uci_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
uci_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(uci_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
tms_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(tms_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
tms_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(tms_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
meds_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(meds_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
meds_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(meds_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
varis_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(varis_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
varis_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(varis_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
sds_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(sds_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
sds_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(sds_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
mins_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(mins_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
mins_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(mins_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
maxs_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(maxs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
maxs_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(maxs_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
ranges_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(ranges_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
ranges_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(ranges_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
iqrs_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(iqrs_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
iqrs_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(iqrs_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
skews_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(skews_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
skews_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(skews_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
skewses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(skewses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
skewses_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(skewses_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number"
kurts_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(kurts_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text"
kurts_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(kurts_str)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number"
kurtses_num = [str2float(x) for x in descsout.xpath(path, namespaces=ns)]
print(kurtses_num)

path = "//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text"
kurtses_str = [x for x in descsout.xpath(path, namespaces=ns)]
print(kurtses_str)

homoout = etree.parse(BytesIO(outputs['Table_Homogeneous Subsets.xml'].content))
print(etree.tostring(homoout, pretty_print=True))

path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:group[@text='Group']/x:category/@number"
subsetgroupnums = [int(x) for x in homoout.xpath(path, namespaces=ns)]
print(subsetgroupnums)

path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:group[@text='Group']/x:category/@text"
subsetgroupcats = [str(x) for x in homoout.xpath(path, namespaces=ns)]
print(subsetgroupcats)

path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:group[@text='Group']/x:category/x:dimension[@axis='column']/x:category[@text='N']/x:cell/@number"
subsetN = [int(x) for x in homoout.xpath(path, namespaces=ns)]
print(subsetN)

subsetcats = []
for sub in subsetgroupnums:
    path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:group[@text='Group']/x:category[@number='" + str(sub) +  "']/x:dimension[@axis='column']/x:group[@text='Subset']/x:category/@number"
    subsetcats.append([int(x) for x in homoout.xpath(path, namespaces=ns)])
print(subsetcats)

subsetvals = []
for sub in subsetgroupnums:
    path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:group[@text='Group']/x:category[@number='" + str(sub) +  "']/x:dimension[@axis='column']/x:group[@text='Subset']/x:category/x:cell/@number"
    subsetvals.append([str2float(x) for x in homoout.xpath(path, namespaces=ns)])
print(subsetvals)

path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:category[@text='Sig.']/x:dimension[@axis='column']/x:group[@text='Subset']/x:category/x:cell/@number"
sigs_num = [str2float(x) for x in homoout.xpath(path, namespaces=ns)]
print(sigs_num)

path = "//x:dimension[@text='Test']/x:category/x:dimension[@axis='row']/x:category[@text='Sig.']/x:dimension[@axis='column']/x:group[@text='Subset']/x:category/x:cell/@text"
sigs_str = [x for x in homoout.xpath(path, namespaces=ns)]
print(sigs_str)

levout = etree.parse(BytesIO(outputs['Table_Levene Test.xml'].content))
print(etree.tostring(levout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@number)"
levf_num = str2float(levout.xpath(path, namespaces=ns))
print(levf_num)

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@text)"
levf_str = levout.xpath(path, namespaces=ns)
print(levf_str)

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='df1']/x:cell/@number)"
levdf1 = int(levout.xpath(path, namespaces=ns))
print(levdf1)

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='df2']/x:cell/@number)"
levdf2 = int(levout.xpath(path, namespaces=ns))
print(levdf2)

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
levsig_num = str2float(levout.xpath(path, namespaces=ns))
print(levsig_num)

path = "string(//x:dimension[@text='Dependent Variable']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
levsig_str = levout.xpath(path, namespaces=ns)
print(levsig_str)

out = etree.parse(BytesIO(outputs['Table_Notes.xml'].content))
print(etree.tostring(out, pretty_print=True))

out = etree.parse(BytesIO(outputs['Table_Notes_1.xml'].content))
print(etree.tostring(out, pretty_print=True))

out = etree.parse(BytesIO(outputs['Table_Notes_2.xml'].content))
print(etree.tostring(out, pretty_print=True))

hsdout = etree.parse(BytesIO(outputs['Table_POSTHOC Multiple Comparisons.xml'].content))
print(etree.tostring(hsdout, pretty_print=True))

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdmeandiff_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdmeandiff_num[i, j] = tmp[k]
            k += 1
print(hsdmeandiff_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdmeandiff_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdmeandiff_str[i, j] = tmp[k]
            k += 1
print(hsdmeandiff_str)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdstderr_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdstderr_num[i, j] = tmp[k]
            k += 1
print(hsdstderr_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdstderr_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdstderr_str[i, j] = tmp[k]
            k += 1
print(hsdstderr_str)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdsig_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdsig_num[i, j] = tmp[k]
            k += 1
print(hsdsig_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdsig_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdsig_str[i, j] = tmp[k]
            k += 1
print(hsdsig_str)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdlb_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdlb_num[i, j] = tmp[k]
            k += 1
print(hsdlb_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdlb_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdlb_str[i, j] = tmp[k]
            k += 1
print(hsdlb_str)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdlb_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdlb_num[i, j] = tmp[k]
            k += 1
print(hsdlb_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdlb_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdlb_str[i, j] = tmp[k]
            k += 1
print(hsdlb_str)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@number"
tmp = [str2float(x) for x in hsdout.xpath(path, namespaces=ns)]
hsdub_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdub_num[i, j] = tmp[k]
            k += 1
print(hsdub_num)

path = "//x:category[@text='Tukey HSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@text"
tmp = hsdout.xpath(path, namespaces=ns)
hsdub_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            hsdub_str[i, j] = tmp[k]
            k += 1
print(hsdub_str)

lsdout = etree.parse(BytesIO(outputs['Table_POSTHOC Multiple Comparisons_1.xml'].content))
print(etree.tostring(lsdout, pretty_print=True))

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@number"
tmp = [str2float(x) for x in lsdout.xpath(path, namespaces=ns)]
lsdmeandiff_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdmeandiff_num[i, j] = tmp[k]
            k += 1
print(lsdmeandiff_num)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@text"
tmp = lsdout.xpath(path, namespaces=ns)
lsdmeandiff_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdmeandiff_str[i, j] = tmp[k]
            k += 1
print(lsdmeandiff_str)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@number"
tmp = [str2float(x) for x in lsdout.xpath(path, namespaces=ns)]
lsdstderr_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdstderr_num[i, j] = tmp[k]
            k += 1
print(lsdstderr_num)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@text"
tmp = lsdout.xpath(path, namespaces=ns)
lsdstderr_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdstderr_str[i, j] = tmp[k]
            k += 1
print(lsdstderr_str)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number"
tmp = [str2float(x) for x in lsdout.xpath(path, namespaces=ns)]
lsdsig_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdsig_num[i, j] = tmp[k]
            k += 1
print(lsdsig_num)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text"
tmp = lsdout.xpath(path, namespaces=ns)
lsdsig_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdsig_str[i, j] = tmp[k]
            k += 1
print(lsdsig_str)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@number"
tmp = [str2float(x) for x in lsdout.xpath(path, namespaces=ns)]
lsdlb_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdlb_num[i, j] = tmp[k]
            k += 1
print(lsdlb_num)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@text"
tmp = lsdout.xpath(path, namespaces=ns)
lsdlb_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdlb_str[i, j] = tmp[k]
            k += 1
print(lsdlb_str)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@number"
tmp = [str2float(x) for x in lsdout.xpath(path, namespaces=ns)]
lsdub_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdub_num[i, j] = tmp[k]
            k += 1
print(lsdub_num)

path = "//x:category[@text='LSD']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@text"
tmp = lsdout.xpath(path, namespaces=ns)
lsdub_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            lsdub_str[i, j] = tmp[k]
            k += 1
print(lsdub_str)

bonout = etree.parse(BytesIO(outputs['Table_POSTHOC Multiple Comparisons_2.xml'].content))
print(etree.tostring(bonout, pretty_print=True))

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@number"
tmp = [str2float(x) for x in bonout.xpath(path, namespaces=ns)]
bonmeandiff_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonmeandiff_num[i, j] = tmp[k]
            k += 1
print(bonmeandiff_num)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean Difference (I-J)']/x:cell/@text"
tmp = bonout.xpath(path, namespaces=ns)
bonmeandiff_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonmeandiff_str[i, j] = tmp[k]
            k += 1
print(bonmeandiff_str)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@number"
tmp = [str2float(x) for x in bonout.xpath(path, namespaces=ns)]
bonstderr_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonstderr_num[i, j] = tmp[k]
            k += 1
print(bonstderr_num)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Error']/x:cell/@text"
tmp = bonout.xpath(path, namespaces=ns)
bonstderr_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonstderr_str[i, j] = tmp[k]
            k += 1
print(bonstderr_str)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number"
tmp = [str2float(x) for x in bonout.xpath(path, namespaces=ns)]
bonsig_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonsig_num[i, j] = tmp[k]
            k += 1
print(bonsig_num)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text"
tmp = bonout.xpath(path, namespaces=ns)
bonsig_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonsig_str[i, j] = tmp[k]
            k += 1
print(bonsig_str)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@number"
tmp = [str2float(x) for x in bonout.xpath(path, namespaces=ns)]
bonlb_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonlb_num[i, j] = tmp[k]
            k += 1
print(bonlb_num)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Lower Bound']/x:cell/@text"
tmp = bonout.xpath(path, namespaces=ns)
bonlb_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonlb_str[i, j] = tmp[k]
            k += 1
print(bonlb_str)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@number"
tmp = [str2float(x) for x in bonout.xpath(path, namespaces=ns)]
bonub_num = numpy.zeros((len(casecats), len(casecats)))
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonub_num[i, j] = tmp[k]
            k += 1
print(bonub_num)

path = "//x:category[@text='Bonferroni']/x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval']/x:category[@text='Upper Bound']/x:cell/@text"
tmp = bonout.xpath(path, namespaces=ns)
bonub_str = numpy.zeros((len(casecats), len(casecats)), dtype=object)
k = 0
for i in range(0, len(casecats)):
    for j in range(0, len(casecats)):
        if i != j:
            bonub_str[i, j] = tmp[k]
            k += 1
print(bonub_str)

bsout = etree.parse(BytesIO(outputs['Table_Test of Between Subjects Fixed Eff.xml'].content))
print(etree.tostring(bsout, pretty_print=True))

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
cmss_num = str2float(bsout.xpath(path, namespaces=ns))
print(cmss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
cmss_str = bsout.xpath(path, namespaces=ns)
print(cmss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
cmdf = int(bsout.xpath(path, namespaces=ns))
print(cmdf)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@number)"
cmms_num = str2float(bsout.xpath(path, namespaces=ns))
print(cmms_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@text)"
cmms_str = bsout.xpath(path, namespaces=ns)
print(cmms_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@number)"
cmf_num = str2float(bsout.xpath(path, namespaces=ns))
print(cmf_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@text)"
cmf_str = bsout.xpath(path, namespaces=ns)
print(cmf_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
cmsig_num = str2float(bsout.xpath(path, namespaces=ns))
print(cmsig_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Model']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
cmsig_str = bsout.xpath(path, namespaces=ns)
print(cmsig_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
intss_num = str2float(bsout.xpath(path, namespaces=ns))
print(intss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
intss_str = bsout.xpath(path, namespaces=ns)
print(intss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
intdf = int(bsout.xpath(path, namespaces=ns))
print(intdf)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@number)"
intms_num = str2float(bsout.xpath(path, namespaces=ns))
print(intms_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@text)"
intms_str = bsout.xpath(path, namespaces=ns)
print(intms_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@number)"
intf_num = str2float(bsout.xpath(path, namespaces=ns))
print(intf_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@text)"
intf_str = bsout.xpath(path, namespaces=ns)
print(intf_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
intsig_num = str2float(bsout.xpath(path, namespaces=ns))
print(intsig_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Intercept']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
intsig_str = bsout.xpath(path, namespaces=ns)
print(intsig_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
predss_num = str2float(bsout.xpath(path, namespaces=ns))
print(predss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
predss_str = bsout.xpath(path, namespaces=ns)
print(predss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
preddf = int(bsout.xpath(path, namespaces=ns))
print(preddf)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@number)"
predms_num = str2float(bsout.xpath(path, namespaces=ns))
print(predms_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@text)"
predms_str = bsout.xpath(path, namespaces=ns)
print(predms_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@number)"
predf_num = str2float(bsout.xpath(path, namespaces=ns))
print(predf_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='F']/x:cell/@text)"
predf_str = bsout.xpath(path, namespaces=ns)
print(predf_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
predsig_num = str2float(bsout.xpath(path, namespaces=ns))
print(predsig_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='" + str(pred) + "']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
predsig_str = bsout.xpath(path, namespaces=ns)
print(predsig_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Error']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
errss_num = str2float(bsout.xpath(path, namespaces=ns))
print(errss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Error']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
errss_str = bsout.xpath(path, namespaces=ns)
print(errss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Error']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
errdf = int(bsout.xpath(path, namespaces=ns))
print(errdf)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Error']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@number)"
errms_num = str2float(bsout.xpath(path, namespaces=ns))
print(errms_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Error']/x:dimension[@text='Statistics']/x:category[@text='Mean Square']/x:cell/@text)"
errms_str = bsout.xpath(path, namespaces=ns)
print(errms_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
totss_num = str2float(bsout.xpath(path, namespaces=ns))
print(totss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
totss_str = bsout.xpath(path, namespaces=ns)
print(totss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
totdf = int(bsout.xpath(path, namespaces=ns))
print(totdf)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Total']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@number)"
ctotss_num = str2float(bsout.xpath(path, namespaces=ns))
print(ctotss_num)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Total']/x:dimension[@text='Statistics']/x:category[@text='Type III Sum of Squares']/x:cell/@text)"
ctotss_str = bsout.xpath(path, namespaces=ns)
print(ctotss_str)

path = "string(//x:dimension[@text='Source']/x:group/x:category[@text='Corrected Total']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
ctotdf = int(bsout.xpath(path, namespaces=ns))
print(ctotdf)

# Outputs:
# Chart_000.png : Boxplot (EXAMINE)
# Chart_1_000.png : Error chart (GRAPH)
# Chart_2_000.png : Profile plot (UNIANOVA)

# Table_Between Subjects Factors.htm : (UNIANOVA)
# Table_Case Processing Summary.htm : valid/missing/total cases (EXAMINE)
# Table_Descriptive Statistics.htm : Mean, sd and N (UNIANOVA)
# Table_Descriptives.htm : Statistics (EXAMINE)
# Table_Homogeneous Subsets.htm : (UNIANOVA)
# Table_Levene Test.htm : (UNIANOVA)
# Table_Notes.htm : Notes (EXAMINE)
# Table_Notes_1.htm : Notes (GRAPH)
# Table_Notes_2.htm : Notes (UNIANOVA)
# Table_POSTHOC Multiple Comparisons.htm : Tukey HSD (UNIANOVA)
# Table_POSTHOC Multiple Comparisons_1.htm : LSD (UNIANOVA)
# Table_POSTHOC Multiple Comparisons_2.htm : Bonferroni (UNIANOVA)
# Table_Test of Between Subjects Fixed Eff.htm : (UNIANOVA)


varname1short = str(resp)
varname1long = str(resp)
if resplab != '':
    varname1long = resplab + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(pred)
varname2long = str(pred)
if predlab != '':
    varname2long = predlab + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

randcat1 = random.randint(0, len(casecats) - 1)
randcat2 = random.randint(0, len(casecats) - 1)
randcat3 = random.randint(0, len(casecats) - 1)
randcat4 = random.randint(0, len(casecats) - 1)

introhtml = '<h2>Analysis of Variance practical</h2>'
introhtml += '<p>'
introhtml += 'In this practical we will investigate how we model the influence of a categorical predictor on a continuous response. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

boxplotprachtml = '<p>'
boxplotprachtml += 'In this case we are interested in the effect of ' + varname2short + ' on our response variable ' + varname1short + '. '
boxplotprachtml += varname2short + ' has ' + str(len(casecats)) + ' categories: ' + ', '.join([str(cat) for cat in casecats]) + '. '
boxplotprachtml += 'We will begin to look at this relationship graphically and look at the distribution of ' + varname1short + ' separately for each category and a good way to do this is via a box plot. '
boxplotprachtml += 'To do this in SPSS:'
boxplotprachtml += '</p>'

boxplotinstrhtml = '<ul class = "instructions">'
boxplotinstrhtml += '<li>Select <strong>Boxplot</strong> from the <strong>Legacy Dialogs</strong> submenu of the <strong>Graphs</strong> menu.</li>'
boxplotinstrhtml += '<li>Select <strong>Simple</strong>  and <strong>Summaries for groups of cases</strong> and click on the <strong>Define</strong> button.</li>'
boxplotinstrhtml += '<li>Transfer the ' + varname1long + ' variable to the <strong>Variable</strong> box.</li>'
boxplotinstrhtml += '<li>Transfer the ' + varname2long + ' variable to the <strong>Category Axis</strong> box.</li>'
boxplotinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
boxplotinstrhtml += '</ul>'

boxplotprachtml += boxplotinstrhtml

boxplotprachtml += '<p>'
boxplotprachtml += 'This will produce a table detailing the numbers of valid observations which we do not show here and then a plot with a box for each category as shown below:'
boxplotprachtml += '</p>'

boxplotprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

boxplotprachtml += '<p>'
boxplotprachtml += 'A boxplot uses the 1st and 3rd quantiles of the data to form the box with the median presented as a line in the middle of the box. '
boxplotprachtml += 'In this case the medians are for each category as follows: ' + ', '.join([casecats[i] + ' = ' + meds_str[i] for i in range(0, len(casecats)-1)]) + ' and for ' + casecats[len(casecats)-1] + ' = ' + meds_str[len(casecats)-1]   + ' and we see that category ' + casecats[meds_num.index(max(meds_num))] + ' has the highest median whilst category ' + casecats[meds_num.index(min(meds_num))] + ' has the lowest median. '
boxplotprachtml += '</p>'
outputs['boxplot.html'] = HTMLOutput(boxplotprachtml, "Boxplot explanation")

boxplotquesthtml = '<p>'
boxplotquesthtml += 'Firstly use SPSS to create a boxplot of ' + varname1long + ' for each category of ' + varname2long + ' and answer the following: '
boxplotquesthtml += '</p> '
boxplotquesthtml += '<ul>'
boxplotquesthtml += '<li>Question: Which category has the highest median?</li>'
boxplotquesthtml += '<li>Question: Which category has the lowest median?</li>'
boxplotquesthtml += '<li>Question: What is the median for category ' + casecats[randcat1] + '?</li>'
boxplotquesthtml += '</ul>'

outputs['boxplotquestions.html'] = HTMLOutput(boxplotquesthtml, "Boxplot explanation questions")

boxplotsolhtml = '<p>'
boxplotsolhtml += 'The SPSS instructions are as follows: '
boxplotsolhtml += '</p>'
boxplotsolhtml += boxplotinstrhtml
boxplotsolhtml += '<ul>'
boxplotsolhtml += '<li>Question: Which category has the highest median?</li>'
boxplotsolhtml += '<li>Question: Which category has the lowest median?</li>'
boxplotsolhtml += '<li>Question: What is the median for category ' + casecats[randcat1] + '?</li>'
boxplotsolhtml += '</ul>'

boxplotsolhtml += '<p>'
boxplotsolhtml += 'Solutions: The output from SPSS is as follows:'
boxplotsolhtml += '</p>'
boxplotsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
boxplotsolhtml += '<p>'
boxplotsolhtml += 'The answers are as follows:'
boxplotsolhtml += '</p>'
boxplotsolhtml += '<ul>'
boxplotsolhtml += '<li>Category ' + casecats[meds_num.index(max(meds_num))] + ' has the highest median</li>'
boxplotsolhtml += '<li>Category ' + casecats[meds_num.index(min(meds_num))] + ' has the lowest median</li>'
boxplotsolhtml += '<li>The median for category ' + casecats[randcat1] + ' is ' + meds_str[randcat1] + '</li>'
boxplotsolhtml += '</ul>'

outputs['boxplotanswers.html'] = HTMLOutput(boxplotsolhtml, "Boxplot explanation answers")

errplotprachtml = '<p>'
errplotprachtml += 'In fact we will be fitting an analysis of variance (ANOVA) model and this is based on group means rather than medians and so an alternative error bar plot is useful. '
errplotprachtml += 'To get this in SPSS do the following:'
errplotprachtml += '</p>'

errplotinstrhtml = '<ul class = "instructions">'
errplotinstrhtml += '<li>Select <strong>Error Bar</strong> from the <strong>Legacy Dialogs</strong> submenu of the <strong>Graphs</strong> menu.</li>'
errplotinstrhtml += '<li>Select <strong>Simple</strong> and <strong>Summaries for groups of cases</strong> as for the boxplot and click on the <strong>Define</strong> button.</li>'
errplotinstrhtml += '<li>Transfer the ' + varname1long + ' variable to the <strong>Variable</strong> box.</li>'
errplotinstrhtml += '<li>Transfer the ' + varname2long + ' variable to the <strong>Category Axis</strong> box.</li>'
errplotinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
errplotinstrhtml += '</ul>'

errplotprachtml += errplotinstrhtml

errplotprachtml += '<p>'
errplotprachtml += 'The graph will appear as shown below:'
errplotprachtml += '</p>'

errplotprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

errplotprachtml += '<p>'
errplotprachtml += 'Here we are now plotting the means rather than the medians and the error bars represent 95% confidence intervals for the different groups. '
errplotprachtml += 'The highest mean is ' + mean_str[mean_num.index(max(mean_num))] + ' for category ' + casecats[mean_num.index(max(mean_num))] + ' whilst the lowest mean is ' + mean_str[mean_num.index(min(mean_num))] + ' for category ' + casecats[mean_num.index(min(mean_num))] + '. '

nonovercats = []
nonoverlaps = 0
order = numpy.argsort(lci_num)
for i in range(1, len(order)):
    if not (uci_num[order[i-1]] > lci_num[order[i]] or lci_num[order[i-1]] < uci_num[order[i]]):
        nonovercats.append((casecats[order[i-1]], casecats[order[i]]))
        nonoverlaps += 1

if nonoverlaps == 0:
    errplotprachtml += 'Here we see that all pairs of categories overlap and so we might not expect an effect of ' + varname2short + ' on ' + varname1short + '. '

if nonoverlaps == 1:
    errplotprachtml += 'The confidence interval for the pair of categories ' + nonovercats[0][0] + ' and ' + nonovercats[0][1] + ' does not overlap so any effect for the predictor ' + varname2short + ' is likely to be due to this pairing. '

if nonoverlaps > 1:
    errplotprachtml += 'Several of the confidence intervals (' + ', '.join([cat1 + ' vs ' + cat2 for cat1, cat2 in nonovercats]) + ') do not overlap so we anticipate that ' + varname2short + ' will have a significant effect on ' + varname1short + '. '

errplotprachtml += '</p>'
outputs['errorplot.html'] = HTMLOutput(errplotprachtml, "Error plot explanation")

errplotquesthtml = '<p>'
errplotquesthtml += 'Next use SPSS to create an error bar plot of ' + varname1long + ' for each category of ' + varname2long + ' and answer the following: '
errplotquesthtml += '</p>'
errplotquesthtml += '<ul>'
errplotquesthtml += '<li>Question: Which category has the highest mean and what is it?</li>'
errplotquesthtml += '<li>Question: What category has the lowest mean and what is it?</li>'
errplotquesthtml += '<li>Question: Which categories confidence intervals do not overlap?</li>'
errplotquesthtml += '<li>Question: Do we expect significant effects?</li>'
errplotquesthtml += '</ul>'

outputs['errorplotquestions.html'] = HTMLOutput(errplotquesthtml, "Error plot explanation questions")

errplotsolhtml = '<p>'
errplotsolhtml += 'The SPSS instructions are as follows: '
errplotsolhtml += '</p>'
errplotsolhtml += errplotinstrhtml
errplotsolhtml += '<ul>'
errplotsolhtml += '<li>Question: Which category has the highest mean and what is it?</li>'
errplotsolhtml += '<li>Question: What category has the lowest mean and what is it?</li>'
errplotsolhtml += '<li>Question: Which categories confidence intervals do not overlap?</li>'
errplotsolhtml += '<li>Question: Do we expect significant effects?</li>'
errplotsolhtml += '</ul>'

errplotsolhtml += '<p>'
errplotsolhtml += 'Solutions: The output from SPSS is as follows:'
errplotsolhtml += '</p>'
errplotsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'
errplotsolhtml += '<p>'
errplotsolhtml += 'The answers are as follows:'
errplotsolhtml += '</p>'
errplotsolhtml += '<ul>'
errplotsolhtml += '<li>The highest mean is ' + mean_str[mean_num.index(max(mean_num))] + ' for category ' + casecats[mean_num.index(max(mean_num))] + '.</li>'
errplotsolhtml += '<li>The lowest mean is ' + mean_str[mean_num.index(min(mean_num))] + ' for category ' + casecats[mean_num.index(min(mean_num))] + '.</li>'
if nonoverlaps == 0:
    errplotsolhtml += '<li>All pairs of categories overlap.</li>'
    errplotsolhtml += '<li>We might not expect an effect of ' + varname2short + ' on ' + varname1short + '.</li>'
if nonoverlaps == 1:
    errplotsolhtml += '<li>The confidence interval for the pair of categories ' + nonovercats[0][0] + ' and ' + nonovercats[0][1] + ' does not overlap.</li>'    
    errplotsolhtml += '<li>Any effect for the predictor ' + varname2short + ' is likely to be due to the pairing of ' + nonovercats[0][0] + ' and ' + nonovercats[0][1] + '.</li>'
if nonoverlaps > 1:
    errplotsolhtml += '<li>Several of the confidence intervals (' + ', '.join([cat1 + ' vs ' + cat2 for cat1, cat2 in nonovercats]) + ') do not overlap.</li>'
    errplotsolhtml += '<li>We anticipate that ' + varname2short + ' will have a significant effect on ' + varname1short + '.</li>'
errplotsolhtml += '</ul>'

outputs['errorplotanswers.html'] = HTMLOutput(errplotsolhtml, "Error plot explanation answers")

summstatsprachtml = '</p>'
summstatsprachtml += 'We have mentioned summary statistics here (medians and means) and we can access these via the <strong>Explore</strong> option as follows:'
summstatsprachtml += '</p>'

summstatsinstrhtml = '<ul class = "instructions">'
summstatsinstrhtml += '<li>Choose <strong>Explore</strong> from the <strong>Descriptives</strong> submeny within the <strong>Analyse</strong> menu.</li>'
summstatsinstrhtml += '<li>Add ' + varname1long + ' to the <strong>Dependent list</strong>.</li>'
summstatsinstrhtml += '<li>Add ' + varname2long + ' to the <strong>Factor list<strong>.</li>'
summstatsinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
summstatsinstrhtml += '</ul>'

summstatsprachtml += summstatsinstrhtml

summstatsprachtml += '<p>'
summstatsprachtml += 'The summary statistics will appear as below. '
summstatsprachtml += '</p>'

summstatsprachtml += outputs['Table_Descriptives.htm'].content

summstatsprachtml += '<p>'
summstatsprachtml += 'If we look at category ' + casecats[0] + ' we can see the median = ' + meds_str[0] + ' which is the middle line in the earlier boxplot and the IQR = ' + iqrs_str[0] + ' which is the width of the box in the plot. '
summstatsprachtml += 'For the error bars plot for category ' + casecats[0] + ' we see that the mean = ' + mean_str[0] + ' which is the x in the middle of the bars whilst the values (' + lci_str[0] + ', '  + uci_str[0] + ') quoted under lower and upper bound for the 95% Confidence interval for the mean represent the ends of the bars in the plot. '
summstatsprachtml += 'We can find similar numbers here for the other categories.'
summstatsprachtml += '</p>'
outputs['summarystats.html'] = HTMLOutput(summstatsprachtml, "Summary statistics explanation")

summstatsquesthtml = '<p>'
summstatsquesthtml += 'Next use the Explore window in SPSS to look at summary statistics of ' + varname1long + ' for each category of ' + varname2long + ' and answer the following: '
summstatsquesthtml += '</p>'
summstatsquesthtml += '<ul>'
summstatsquesthtml += '<li>Question: What is the median of category ' + casecats[randcat1] + '?</li>'
summstatsquesthtml += '<li>Question: What is the IQR of category ' + casecats[randcat3] + '?</li>'
summstatsquesthtml += '<li>Question: What is the mean of category ' + casecats[randcat2] + '?</li>'
summstatsquesthtml += '<li>Question: What is the confidence interval for category ' + casecats[randcat4] + '?</li>'
summstatsquesthtml += '</ul>'

outputs['summarystatsquestions.html'] = HTMLOutput(summstatsquesthtml, "Summary statistics explanation questions")

summstatssolhtml = '<p>'
summstatssolhtml += 'The SPSS instructions are as follows: '
summstatssolhtml += '</p>'
summstatssolhtml += summstatsinstrhtml
summstatssolhtml += '<ul>'
summstatssolhtml += '<li>Question: What is the median of category ' + casecats[randcat1] + '?</li>'
summstatssolhtml += '<li>Question: What is the IQR of category ' + casecats[randcat3] + '?</li>'
summstatssolhtml += '<li>Question: What is the mean of category ' + casecats[randcat2] + '?</li>'
summstatssolhtml += '<li>Question: What is the confidence interval for category ' + casecats[randcat4] + '?</li>'
summstatssolhtml += '</ul>'
summstatssolhtml += '<p>'
summstatssolhtml += 'Solutions: The output from SPSS is as follows:'
summstatssolhtml += '</p>'
summstatssolhtml += outputs['Table_Descriptives.htm'].content
summstatssolhtml += '<p>'
summstatssolhtml += 'The answers are as follows:'
summstatssolhtml += '</p>'
summstatssolhtml += '<ul>'
summstatssolhtml += '<li>The median of category ' + casecats[randcat1] + ' is ' + meds_str[randcat1] + '.</li>'
summstatssolhtml += '<li>The IQR of category ' + casecats[randcat3] + ' is ' + iqrs_str[randcat3] + '.</li>'
summstatssolhtml += '<li>The mean of category ' + casecats[randcat2] + ' is ' + mean_str[randcat2] + '.</li>'
summstatssolhtml += '<li>The confidence interval for category ' + casecats[randcat4] + ' is ' + lci_str[randcat4] + ' to ' + uci_str[randcat4]  + '.</li>'
summstatssolhtml += '</ul>'

outputs['summarystatsanswers.html'] = HTMLOutput(summstatssolhtml, "Summary statistics explanation answers")

anovaprachtml = '<p>'
anovaprachtml += 'We will next go on to perform an ANOVA to test for significant effects of the predictor. '
anovaprachtml += 'This is done as follows:'
anovaprachtml += '</p>'

anovainstrhtml = '<ul class = "instructions">'
anovainstrhtml += '<li>Choose <strong>Univariate</strong> from the <strong>General Linear Model</strong> -> <strong>Analyse</strong> menu.</li>'
anovainstrhtml += '<li>Choose ' + varname1long + ' as the <strong>Dependent Variable</strong>.</li>'
anovainstrhtml += '<li>Choose ' + varname2long + ' as a <strong>Fixed Factor</strong>.</li>'
anovainstrhtml += '<li>Click <strong>Options</strong> and in the window tick <strong>Descriptive statistics</strong> and <strong>Homegeneity tests</strong>.</li>'
anovainstrhtml += '<li>Click on <strong>Continue</strong> to return to the main window.</li>'
anovainstrhtml += '<li>Click on <strong>Plots...</strong> and transfer ' + varname2long + ' to the <strong>Horizontal Axis</strong> box.</li>'
anovainstrhtml += '<li>Click <strong>Add</strong> and then <strong>Continue</strong>.</li>'
anovainstrhtml += '<li>Click on <strong>Post hoc...</strong>.</li>'
anovainstrhtml += '<li>Transfer ' + varname2long + ' to the <strong>post hoc test for</strong> box and select <strong>LSD</strong>, <strong>Bonferroni</strong> and <strong>Tukey</strong>.</li>'
anovainstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
anovainstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
anovainstrhtml += '</ul>'

anovaprachtml += anovainstrhtml

anovaprachtml += '<p>'
anovaprachtml += 'This will produce lots of outputs so we will talk about these in turn. '
anovaprachtml += 'First Levene&apos;s test: '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_Levene Test.htm'].content

anovaprachtml += '<p>'
anovaprachtml += 'The Levene&apos;s test is used to test one of the underlying assumptions of the ANOVA which is the homogeneity of variances i.e. that the residual variances are equal in each group. '
anovaprachtml += 'This test requires a test statistic that has value here ' + levf_str + ' and under the hypothesis of equal variances this statistic follows an F distribution with ' + str(levdf1) + ' and ' + str(levdf2) + ' degrees of freedom where ' + str(levdf1) + ' is the number of categories - 1 and ' + str(levdf2) + ' is the number of observations - the number of categories. '
anovaprachtml += 'Here we see the p value is ' + levsig_str + ' '
if levsig_num < 0.05:
    anovaprachtml += 'which is less than 0.05 and therefore significant which means we can reject the null hypothesis and so the assumption of equal variances is not true and we should adjust the variable in some way (maybe via a transformation) if we wish to use an ANOVA. '
else:
    anovaprachtml += 'which is greater than 0.05 and therefore we cannot reject the Null hypothesis and so we are able to assume equal variances and proceed with the ANOVA. '
anovaprachtml += 'The ANOVA itself is described by the ANOVA table given below: '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_Test of Between Subjects Fixed Eff.htm'].content

anovaprachtml += '<p>'
anovaprachtml += 'The above table gives all the information required for us to decide if ' + varname2short + ' is a significant predictor of ' + varname1short + '. '
anovaprachtml += 'We will here go through the table column by column to explain how the ANOVA works. '
anovaprachtml += 'SPSS gives rather a lot of rows in its ANOVA tables largely because it also allows one to test the intercept term which we are less interested in here. '
anovaprachtml += 'So we will begin with the Type III Sum of Squares (SS) column and for the row Corrected total we see the value ' + ctotss_str + '. '
anovaprachtml += 'This value is calculated by for each observation taking the value for ' + varname1short + ' and subtracting the overall mean of ' + varname1short + '. '
anovaprachtml += 'These values are then squared and their sum is the value ' + ctotss_str + ' we see in the table. '
anovaprachtml += '(Note the value ' + totss_str + ' in the Total row is calculated similarly but without subtracting the mean from each observation). '
anovaprachtml += 'Next the value in row ' + varname2short + ' is calculated by working out the mean of ' + varname1short + ' for each category in ' + varname2short + '. '
anovaprachtml += 'We take these category means and subtract the overall mean from them and again square and sum them to give the value ' + predss_str + '. '
anovaprachtml += 'The Corrected Model row you will see in the one way ANOVA simply repeats the ' + varname2short + ' row. '
anovaprachtml += 'Finally the Error row sum of squares is calculated by subtracting the ' + varname2short + ' SS value from the Corrected total SS value i.e. ' + errss_str + ' = ' + ctotss_str + ' - ' + predss_str + ', which is effectively the sums of squares not explained by ' + varname2short + '. '
anovaprachtml += 'For ' + varname2short + ' to be a significant predictor of ' + varname1short + ' we hope its SS value is large relative to the Error SS but these numbers are based on different sample sizes and so we first need to adjust them to reflect this so the next column is the degrees of freedom (df) column. '
anovaprachtml += 'Here we see we have ' + str(totdf) + ' total degrees of freedom which represents the number of observations but ' + str(ctotdf) + ' corrected total df as we lose one by estimating the mean. '
anovaprachtml += 'For ' + varname2short + ' we have '+ str(preddf) + ' df which is the number of categories - 1 again losing one as if we knew the mean and all bar one of the category means we could calculated the last one. '
anovaprachtml += 'Finally the Error df is ' + str(errdf) + ' which again is calculated by subtraction i.e. ' + str(errdf) + ' = ' + str(ctotdf) + ' - ' + str(preddf) + '. '
anovaprachtml += 'We next use the df to adjust the SS into Mean Squares (MS) and so MS for ' + varname2short + ' is SS for ' + varname2short + ' divided by df for ' + varname2short + ' which means ' + predms_str + ' = ' + predss_str + ' / ' + str(preddf) + '. '
anovaprachtml += 'Similarly for the Error column we have an MS of ' + errms_str + '. '
anovaprachtml += 'These two mean squares are now on the same scale and so we can look at their relative sizes by taking their ratio so F = ' + predf_str + ' = ' + predms_str + ' / ' + errms_str + '. '
anovaprachtml += 'This test statistic follows an F distribution with ' + str(preddf) + ' and ' + str(errdf) + ' degrees of freedom and equates to a p value of ' + predsig_str + ' given in the Sig. column. '
if predsig_num > 0.05:
    anovaprachtml += 'This p value is greater than 0.05 so we cannot reject the null hypothesis of no group effects. '
else:
    anovaprachtml += 'This p value is less than 0.05 and so we can reject the null hypothesis and we find that ' + varname2short + ' is a significant predictor of ' + varname1short + '. '
anovaprachtml += '</p>'

anovaprachtml += '<p>'
anovaprachtml += 'Our next step after establishing whether or not there are significant differences in ' + varname1short + ' for the different categories of ' + varname2short + ' is to do a post-hoc multiple comparisons test. '
anovaprachtml += 'We selected three different tests and we will look at these in turn. '
anovaprachtml += 'The first we will look at is Tukey&apos;s HSD (honest significant difference) shown below: '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_POSTHOC Multiple Comparisons.htm'].content

anovaprachtml += '<p>'
anovaprachtml += 'Here we see a table with one row for each pair of categories (in fact there are two for each pairing as the reverse ordering is also included) and the first two columns identify which pair of categories are being compared so for example the first pairing is ' + casecats[0] + ' and ' + casecats[1] +'. '
anovaprachtml += 'The next column gives the difference in means between this pair of categories and so we for example that the mean of ' + casecats[0] + ' minus the mean of ' + casecats[1] + ' has value ' + hsdmeandiff_str[0, 1] + ' and in the following column the associated standard error ' + hsdstderr_str[0, 1] + ' which will be based on the number of observations in each group and the overall variability. '
anovaprachtml += 'These two columns will be the same regardless of which posthoc test you choose whilst the last three columns differ by test. '
anovaprachtml += 'The Sig. column gives a p value for a difference in mean between the two categories and the final 2 columns give an associated confidence interval. '
anovaprachtml += 'For example the difference between ' + casecats[0] + ' and ' + casecats[1] + ' has a p value ' + hsdsig_str[0, 1] + '. '
if hsdsig_num[1, 0] > 0.05:
    anovaprachtml += 'which is greater than 0.05 and therefore not significant and the confidence interval (' + hsdlb_str[0, 1] + ', ' + hsdub_str[0, 1] + ') contains 0 which agrees with this. '
else:
    anovaprachtml += 'which is less than 0.05 and therefore a significant difference and the confidence interval is (' + hsdlb_str[0, 1] + ', ' + hsdub_str[0, 1] + ') which doesn&apos;t contains 0 which agrees with this. '
anovaprachtml += 'The Tukey HSD procedure balances good power with a reasonably tight control of the Type I error rate. '
# Work out a list of pairs which are significantly different
hsdsigpairs = []
nrow, ncol = hsdsig_num.shape
for i in range(0, nrow):
    for j in range(0, i):
        if hsdsig_num[i, j] < 0.05:
            hsdsigpairs.append((casecats[i], casecats[j]))
if len(hsdsigpairs) == 0:
    anovaprachtml += 'We see that none of the pairs of categories are significantly different. '
elif len(hsdsigpairs) == (nrow * (nrow-1)) / 2 :
    anovaprachtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovaprachtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in hsdsigpairs]) + ') are significantly different. '
anovaprachtml += 'We can compare these pairings with our initial error bars. '
anovaprachtml += '</p>'

anovaprachtml += '<p>'
anovaprachtml += 'Next we will look at the LSD procedure with the results shown below: '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_POSTHOC Multiple Comparisons_1.htm'].content

anovaprachtml += '<p>'
anovaprachtml += 'The LSD procedure is less conservative and we should therefore get at least as many pairs that are statistically significantly different. '
# Work out a list of pairs which are significantly different
lsdsigpairs = []
nrow, ncol = lsdsig_num.shape
for i in range(0, nrow):
    for j in range(0, i):
        if lsdsig_num[i, j] < 0.05:
            lsdsigpairs.append((casecats[i], casecats[j]))
if len(lsdsigpairs) == 0:
    anovaprachtml += 'We see that none of the pairs of categories are significantly different. '
elif len(lsdsigpairs) == (nrow * (nrow-1)) / 2 :
    anovaprachtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovaprachtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in lsdsigpairs]) + ') are significantly different. '
anovaprachtml += '</p>'

anovaprachtml += '<p>'
anovaprachtml += 'Finally we look at the Bonferroni procedure with the results as shown below: '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_POSTHOC Multiple Comparisons_2.htm'].content

anovaprachtml += '<p>'
anovaprachtml += 'The Bonferroni procedure is very conservative and of the three selected we will generally get the least pairs that are statistically significantly different. '
# Work out a list of pairs which are significantly different
bonsigpairs = []
nrow, ncol = bonsig_num.shape
for i in range(0, nrow):
    for j in range(0, i):
        if bonsig_num[i, j] < 0.05:
            bonsigpairs.append((casecats[i], casecats[j]))
if len(bonsigpairs) == 0:
    anovaprachtml += 'We see that none of the pairs of categories are significantly different. '
elif len(bonsigpairs) == (nrow * (nrow-1)) / 2 :
    anovaprachtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovaprachtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in bonsigpairs]) + ') are significantly different. '
anovaprachtml += '</p>'

anovaprachtml += '<p>'
anovaprachtml += 'The options we have chosen also give us a marginal means plot as shown below: '
anovaprachtml += '</p>'

anovaprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'

anovaprachtml += '<p>'
anovaprachtml += 'In general a marginal means plot will show the means for each category adjusting for any other variables in the model but here we are simply fitting a one way ANOVA and so we will simply see the means for each group plotted together so for example we see the mean for category ' + str(casecats[0]) + ' is ' + mean_str[0] + ' and for category ' + str(casecats[1]) + ' is ' + mean_str[1] + '. '
anovaprachtml += '</p>'

anovaprachtml += '<p>'
anovaprachtml += 'Finally we have asked SPSS to find homogeneous subsets within our categories. '
anovaprachtml += 'These are groups of categories that cannot be separated (using in this case Tukey&apos;s HSD) as statistically different within the category. '
anovaprachtml += '</p>'

anovaprachtml += outputs['Table_Homogeneous Subsets.htm'].content

subsets = set()
for s in subsetcats:
    subsets.update(s)

anovaprachtml += '<p>'
if len(subsets) == 1:
    anovaprachtml += 'In this case we see there is one identified subset containing all categories. '
else: 
    anovaprachtml += 'In this case we see there are ' + str(len(subsets)) + ' identified subsets with categories '
    for s in subsets:
        cats = []
        for j in range(0, len(subsetgroupnums)):
            if s in subsetcats[j]:
                cats.append(subsetgroupcats[j])
        anovaprachtml += ', '.join(cats) + ' in subset ' + str(s) + '; '
anovaprachtml += '</p>'
outputs['anova.html'] = HTMLOutput(anovaprachtml, "ANVOA explanation")

anovaquesthtml = '<p>'
anovaquesthtml += 'Finally use the Univariate option under General Linear Model in SPSS to fit an ANOVA and answer the following: '
anovaquesthtml += '</p>'
anovaquesthtml += '<ul>'
anovaquesthtml += '<li>Question: Looking at the Levene&apos;s test does this mean we can assume equal variances between categories?</li>'
anovaquesthtml += '<li>Question: Looking at the Between Subjects Effects table do we observe significant effects of ' + varname2short + ' on ' + varname1short + '?</li>'
anovaquesthtml += '<li>Question: Looking at the Tukey&apos;s HSD pairwise comparisons which pairings does it suggest are significantly different?</li>'
anovaquesthtml += '<li>Question: Looking at the LSD pairwise comparisons which pairings does it suggest are significantly different?</li>'
anovaquesthtml += '<li>Question: Looking at the Bonferroni pairwise comparisons which pairings does it suggest are significantly different?</li>'
anovaquesthtml += '<li>Question: Looking at the Marginal Means plot which category has the largest and which the smallest means?</li>'
anovaquesthtml += '<li>Question: Looking at the table for homogeneous subsets how many subsets are identified and which categories are in each?</li>'
anovaquesthtml += '</ul>'
outputs['anovaquestions.html'] = HTMLOutput(anovaquesthtml, "ANVOA explanation questions")

anovasolhtml = '<p>'
anovasolhtml += 'The SPSS instructions are as follows: '
anovasolhtml += '</p>'
anovasolhtml += anovainstrhtml
anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the Levene&apos;s test does this mean we can assume equal variances between categories?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_Levene Test.htm'].content
anovasolhtml += '<p>'
if levsig_num < 0.05:
    anovasolhtml += 'Here we see the p value is ' + levsig_str + ' which is less than 0.05 and therefore significant which means we can reject the null hypothesis and so the assumption of equal variances is not true and we should adjust the variable in some way (maybe via a transformation) if we wish to use an ANOVA. '
else:
    anovasolhtml += 'Here we see the p value is ' + levsig_str + ' which is greater than 0.05 and therefore we cannot reject the Null hypothesis and so we are able to assume equal variances and proceed with the ANOVA. '
anovasolhtml +='</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the Between Subjects Effects table do we observe significant effects of ' + varname2short + ' on ' + varname1short + '?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_Test of Between Subjects Fixed Eff.htm'].content
anovasolhtml += '<p>'
if predsig_num > 0.05:
    anovasolhtml += 'The p value is greater than 0.05 so we cannot reject the null hypothesis of no group effects. '
else:
    anovasolhtml += 'The p value is less than 0.05 and so we can reject the null hypothesis and we find that ' + varname2short + ' is a significant predictor of ' + varname1short + '. '
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the Tukey&apos;s HSD pairwise comparisons which pairings does it suggest are significant different?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_POSTHOC Multiple Comparisons.htm'].content
anovasolhtml += '<p>'
if len(hsdsigpairs) == 0:
    anovasolhtml += 'We see that none of the pairs of categories are significantly different. '
elif len(hsdsigpairs) == (nrow * (nrow-1)) / 2 :
    anovasolhtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovasolhtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in hsdsigpairs]) + ') are significantly different. '
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the LSD pairwise comparisons which pairings does it suggest are significantly different?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_POSTHOC Multiple Comparisons_1.htm'].content
anovasolhtml += '<p>'
if len(lsdsigpairs) == 0:
    anovasolhtml += 'We see that none of the pairs of categories are significantly different. '
elif len(lsdsigpairs) == (nrow * (nrow-1)) / 2 :
    anovasolhtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovasolhtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in lsdsigpairs]) + ') are significantly different. '
anovasolhtml += '</p>'

anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the Bonferroni pairwise comparisons which pairings does it suggest are significantly different?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_POSTHOC Multiple Comparisons_2.htm'].content
anovasolhtml += '<p>'
if len(bonsigpairs) == 0:
    anovasolhtml += 'We see that none of the pairs of categories are significantly different. '
elif len(bonsigpairs) == (nrow * (nrow-1)) / 2 :
    anovasolhtml += 'We see that all of the pairs of categories are significantly different. '
else:
    anovasolhtml += 'We see that the following pairs (' + ','.join([cat1 + ':' + cat2 for cat1, cat2 in bonsigpairs]) + ') are significantly different. '
anovasolhtml += '</p>'

anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the Marginal Means plot which category has the largest and which the smallest means?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'
anovasolhtml += '<p>'
anovasolhtml += 'Category ' + casecats[mean_num.index(max(mean_num))] + ' has the largest mean and ' + casecats[mean_num.index(min(mean_num))] + ' has the smallest mean. '
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Question: Looking at the table for homogeneous subsets how many subsets are identified and which categories are in each?'
anovasolhtml += '</p>'
anovasolhtml += '<p>'
anovasolhtml += 'Solution: The output from SPSS is as follows:'
anovasolhtml += '</p>'
anovasolhtml += outputs['Table_Homogeneous Subsets.htm'].content
anovasolhtml += '<p>'
if len(subsets) == 1:
    anovasolhtml += 'There is one identified subset containing all categories. '
else: 
    anovasolhtml += 'There are ' + str(len(subsets)) + ' identified subsets with categories '
    for s in subsets:
        cats = []
        for j in range(0, len(subsetgroupnums)):
            if s in subsetcats[j]:
                cats.append(subsetgroupcats[j])
        anovasolhtml += ', '.join(cats) + ' in subset ' + str(s) + '; '
anovasolhtml += '</p>'

outputs['anovaanswers.html'] = HTMLOutput(anovasolhtml, "ANVOA explanation answers")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['boxplot.html'].content
combinedprachtml += outputs['errorplot.html'].content
combinedprachtml += outputs['summarystats.html'].content
combinedprachtml += outputs['anova.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['boxplotquestions.html'].content
combinedquesthtml += outputs['errorplotquestions.html'].content
combinedquesthtml += outputs['summarystatsquestions.html'].content
combinedquesthtml += outputs['anovaquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['boxplotanswers.html'].content
combinedsolhtml += outputs['errorplotanswers.html'].content
combinedsolhtml += outputs['summarystatsanswers.html'].content
combinedsolhtml += outputs['anovaanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")
'''