# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate More advanced descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Explore... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var1 = DataVector('First continuous variable to summarize: ')
var2 = DataVector('Second continuous variable to summarize: ')
'''

    spssscript = '''
FREQUENCIES VARIABLE=${var1}
 /PERCENTILES=25 50 75
 /STATISTICS=MEAN MEDIAN MODE STDDEV VARIANCE RANGE MINIMUM MAXIMUM
 /HISTOGRAM NORMAL.

FREQUENCIES VARIABLE=${var2}
 /PERCENTILES=25 50 75
 /STATISTICS=MEAN MEDIAN MODE STDDEV VARIANCE RANGE MINIMUM MAXIMUM
 /HISTOGRAM NORMAL.

GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${var1} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${var1}=col(source(s), name("${var1}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: text.footnote(label("${datafile.variables[var1]['varlabel']}"))
  ELEMENT: schema(position(bin.quantile.letter(1*${var1})), color.interior(color."D3CE97"), label(id))
END GPL.

GGRAPH
  /GRAPHDATASET NAME="graphdataset" VARIABLES=${var2} MISSING=LISTWISE REPORTMISSING=NO
  /GRAPHSPEC SOURCE=INLINE.
BEGIN GPL
  SOURCE: s=userSource(id("graphdataset"))
  DATA: ${var2}=col(source(s), name("${var2}"))
  DATA: id=col(source(s), name("$CASENUM"), unit.category())
  GUIDE: text.footnote(label("${datafile.variables[var2]['varlabel']}"))
  ELEMENT: schema(position(bin.quantile.letter(1*${var2})), color.interior(color."D3CE97"), label(id))
END GPL.

'''


    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

import random

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

statsout = etree.parse(BytesIO(outputs['Table_Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category/@varName)"
varnames1 = statsout.xpath(path, namespaces=ns)
print(varnames1)

path = "string(//x:dimension[@text='Variables']/x:category/@label)"
varlabs1 = statsout.xpath(path, namespaces=ns)
print(varlabs1)

path = "string(//x:group[@text='N']/x:category[@text='Valid']/x:cell/@number)"
Nvalid1 = int(statsout.xpath(path, namespaces=ns))
print(Nvalid1)

path = "string(//x:group[@text='N']/x:category[@text='Missing']/x:cell/@number)"
Nmiss1 = int(statsout.xpath(path, namespaces=ns))
print(Nmiss1)

path = "string(//x:category[@text='Mean']/x:cell/@number)"
means1_num = str2float(statsout.xpath(path, namespaces=ns))
print(means1_num)

path = "string(//x:category[@text='Mean']/x:cell/@text)"
means1_str = statsout.xpath(path, namespaces=ns)
print(means1_str)

path = "string(//x:category[@text='Median']/x:cell/@number)"
medians1_num = str2float(statsout.xpath(path, namespaces=ns))
print(medians1_num)

path = "string(//x:category[@text='Median']/x:cell/@text)"
medians1_str = statsout.xpath(path, namespaces=ns)
print(medians1_str)

path = "string(//x:category[@text='Mode']/x:cell/@number)"
modes1_num = str2float(statsout.xpath(path, namespaces=ns))
print(modes1_num)

path = "string(//x:category[@text='Mode']/x:cell/@text)"
modes1_str = statsout.xpath(path, namespaces=ns)
print(modes1_str)

path = "string(//x:category[@text='Std. Deviation']/x:cell/@number)"
sds1_num = str2float(statsout.xpath(path, namespaces=ns))
print(sds1_num)

path = "string(//x:category[@text='Std. Deviation']/x:cell/@text)"
sds1_str = statsout.xpath(path, namespaces=ns)
print(sds1_str)

path = "string(//x:category[@text='Variance']/x:cell/@number)"
variances1_num = str2float(statsout.xpath(path, namespaces=ns))
print(variances1_num)

path = "string(//x:category[@text='Variance']/x:cell/@text)"
variances1_str = statsout.xpath(path, namespaces=ns)
print(variances1_str)

path = "string(//x:category[@text='Range']/x:cell/@number)"
ranges1_num = str2float(statsout.xpath(path, namespaces=ns))
print(ranges1_num)

path = "string(//x:category[@text='Range']/x:cell/@text)"
ranges1_str = statsout.xpath(path, namespaces=ns)
print(ranges1_str)

path = "string(//x:category[@text='Minimum']/x:cell/@number)"
mins1_num = str2float(statsout.xpath(path, namespaces=ns))
print(mins1_num)

path = "string(//x:category[@text='Minimum']/x:cell/@text)"
mins1_str = statsout.xpath(path, namespaces=ns)
print(mins1_str)

path = "string(//x:category[@text='Maximum']/x:cell/@number)"
maxs1_num = str2float(statsout.xpath(path, namespaces=ns))
print(maxs1_num)

path = "string(//x:category[@text='Maximum']/x:cell/@text)"
maxs1_str = statsout.xpath(path, namespaces=ns)
print(maxs1_str)

path = "//x:group[@text='Percentiles']/x:category/x:cell/@number"
centiles1_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(centiles1_num)

path = "//x:group[@text='Percentiles']/x:category/x:cell/@text"
centiles1_str = statsout.xpath(path, namespaces=ns)
print(centiles1_str)

statsout = etree.parse(BytesIO(outputs['Table_Statistics_1.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category/@varName)"
varnames2 = statsout.xpath(path, namespaces=ns)
print(varnames2)

path = "string(//x:dimension[@text='Variables']/x:category/@label)"
varlabs2 = statsout.xpath(path, namespaces=ns)
print(varlabs2)

path = "string(//x:group[@text='N']/x:category[@text='Valid']/x:cell/@number)"
Nvalid2 = int(statsout.xpath(path, namespaces=ns))
print(Nvalid2)

path = "string(//x:group[@text='N']/x:category[@text='Missing']/x:cell/@number)"
Nmiss2 = int(statsout.xpath(path, namespaces=ns))
print(Nmiss2)

path = "string(//x:category[@text='Mean']/x:cell/@number)"
means2_num = str2float(statsout.xpath(path, namespaces=ns))
print(means2_num)

path = "string(//x:category[@text='Mean']/x:cell/@text)"
means2_str = statsout.xpath(path, namespaces=ns)
print(means2_str)

path = "string(//x:category[@text='Median']/x:cell/@number)"
medians2_num = str2float(statsout.xpath(path, namespaces=ns))
print(medians2_num)

path = "string(//x:category[@text='Median']/x:cell/@text)"
medians2_str = statsout.xpath(path, namespaces=ns)
print(medians2_str)

path = "string(//x:category[@text='Mode']/x:cell/@number)"
modes2_num = str2float(statsout.xpath(path, namespaces=ns))
print(modes2_num)

path = "string(//x:category[@text='Mode']/x:cell/@text)"
modes2_str = statsout.xpath(path, namespaces=ns)
print(modes2_str)

path = "string(//x:category[@text='Std. Deviation']/x:cell/@number)"
sds2_num = str2float(statsout.xpath(path, namespaces=ns))
print(sds2_num)

path = "string(//x:category[@text='Std. Deviation']/x:cell/@text)"
sds2_str = statsout.xpath(path, namespaces=ns)
print(sds2_str)

path = "string(//x:category[@text='Variance']/x:cell/@number)"
variances2_num = str2float(statsout.xpath(path, namespaces=ns))
print(variances2_num)

path = "string(//x:category[@text='Variance']/x:cell/@text)"
variances2_str = statsout.xpath(path, namespaces=ns)
print(variances2_str)

path = "string(//x:category[@text='Range']/x:cell/@number)"
ranges2_num = str2float(statsout.xpath(path, namespaces=ns))
print(ranges2_num)

path = "string(//x:category[@text='Range']/x:cell/@text)"
ranges2_str = statsout.xpath(path, namespaces=ns)
print(ranges2_str)

path = "string(//x:category[@text='Minimum']/x:cell/@number)"
mins2_num = str2float(statsout.xpath(path, namespaces=ns))
print(mins2_num)

path = "string(//x:category[@text='Minimum']/x:cell/@text)"
mins2_str = statsout.xpath(path, namespaces=ns)
print(mins2_str)

path = "string(//x:category[@text='Maximum']/x:cell/@number)"
maxs2_num = str2float(statsout.xpath(path, namespaces=ns))
print(maxs2_num)

path = "string(//x:category[@text='Maximum']/x:cell/@text)"
maxs2_str = statsout.xpath(path, namespaces=ns)
print(maxs2_str)

path = "//x:group[@text='Percentiles']/x:category/x:cell/@number"
centiles2_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(centiles2_num)

path = "//x:group[@text='Percentiles']/x:category/x:cell/@text"
centiles2_str = statsout.xpath(path, namespaces=ns)
print(centiles2_str)

statsout = etree.parse(BytesIO(outputs['Table_Frequencies.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/@text"
vals1 = statsout.xpath(path, namespaces=ns)
print(vals1)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number"
freqs1 = [int(x) for x in statsout.xpath(path, namespaces=ns)]
print(freqs1)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number"
allcents1_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(allcents1_num)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text"
allcents1_str = statsout.xpath(path, namespaces=ns)
print(allcents1_str)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number"
validcents1_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(validcents1_num)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text"
validcents1_str = statsout.xpath(path, namespaces=ns)
print(validcents1_str)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@number"
cumcents1_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(cumcents1_num)

path = "//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@text"
cumcents1_str = statsout.xpath(path, namespaces=ns)
print(cumcents1_str)

path = "string(//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totfreq1 = int(statsout.xpath(path, namespaces=ns))
print(totfreq1)

path = "string(//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totpercent1_num = str2float(statsout.xpath(path, namespaces=ns))
print(totpercent1_num)

path = "string(//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totpercent1_str = statsout.xpath(path, namespaces=ns)
print(totpercent1_str)

path = "string(//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number)"
totvalidpercent1_num = str2float(statsout.xpath(path, namespaces=ns))
print(totvalidpercent1_num)

path = "string(//x:dimension[@varName='" + varnames1 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text)"
totvalidpercent1_str = statsout.xpath(path, namespaces=ns)
print(totvalidpercent1_str)

statsout = etree.parse(BytesIO(outputs['Table_Frequencies_1.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/@text"
vals2 = [str(x) for x in statsout.xpath(path, namespaces=ns)]
print(vals2)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number"
freqs2 = [int(x) for x in statsout.xpath(path, namespaces=ns)]
print(freqs2)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number"
allcents2_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(allcents2_num)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text"
allcents2_str = statsout.xpath(path, namespaces=ns)
print(allcents2_str)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number"
validcents2_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(validcents2_num)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text"
validcents2_str = statsout.xpath(path, namespaces=ns)
print(validcents2_str)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@number"
cumcents2_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(cumcents2_num)

path = "//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:group/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Cumulative Percent']/x:cell/@text"
cumcents2_str = statsout.xpath(path, namespaces=ns)
print(cumcents2_str)

path = "string(//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Frequency']/x:cell/@number)"
totfreq2 = int(statsout.xpath(path, namespaces=ns))
print(totfreq2)

path = "string(//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@number)"
totpercent2_num = str2float(statsout.xpath(path, namespaces=ns))
print(totpercent2_num)

path = "string(//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Percent']/x:cell/@text)"
totpercent2_str = statsout.xpath(path, namespaces=ns)
print(totpercent2_str)

path = "string(//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@number)"
totvalidpercent2_num = str2float(statsout.xpath(path, namespaces=ns))
print(totvalidpercent2_num)

path = "string(//x:dimension[@varName='" + varnames2 + "']/x:group[@text='Valid']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='Valid Percent']/x:cell/@text)"
totvalidpercent2_str = statsout.xpath(path, namespaces=ns)
print(totvalidpercent2_str)


# Outputs:
# Chart_000.png : Histogram of first variable
# Chart_1_000.png .. Chart_{nvar - 1}_000.png : Histogram of next variables
# Chart_{nvar}_000.png .. Chart_{(2*nvar) - 1}_000.png : box plot of variables
# Table_Frequencies.htm : Frequencies of first variable
# Table_Frequencies_{1..}.htm : Frequencies of second.. variable
# Table_Notes.htm : notes regarding frequencies command
# Table_Notes_1.htm : notes regarding simple bar chart
# Table_Notes_2.htm : notes regarding grouped bar chart
# Table_Statistics.htm : N valid/missing for each variable

varname1short = varnames1
varname1long = varnames1
if varlabs1 != '':
    varname1long = varlabs1 + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = varnames2
varname2long = varnames2
if varlabs2 != '':
    varname2long = varlabs2 + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'


introhtml = '<h2>Descriptive statistics for all variables practical</h2>'
introhtml += '<p>'
introhtml += 'In this descriptive statistics practical we will investigate summarising continuous variables. '
introhtml += 'We will look at how in SPSS we can obtain some summary statistics that describe the distribution of variables both in terms of measures of location and spread. '
introhtml += 'We will also look at how we might summarise these variables graphically. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

var1prachtml = '<p>'
var1prachtml += 'We will begin by looking at how to use SPSS to get summary statistics for our first variable, ' + varname1short + '. '
var1prachtml += '</p>'

var1instructhtml = '<ul class = "instructions">'
var1instructhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var1instructhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var1instructhtml += '<li>Click on the <strong>Statistics</strong> button to go to the statistics screen.</li>'
var1instructhtml += '<li>Here we need to select ALL the summary statistics that we are interested in looking at.</li>'
var1instructhtml += '<li>Select <strong>Mean, Median</strong> and <strong>Mode</strong> from under <strong>Central Tendency</strong>.</li>'
var1instructhtml += '<li>Select <strong>Std. deviation, Variance, Range, Minimum</strong> and <strong>Maximum</strong> from under <strong>Dispersion</strong>.</li>'
var1instructhtml += '<li>Finally Select <strong>Quartiles</strong> from under <strong>Percentile Values.</strong></li>'
var1instructhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
var1instructhtml += '<li>Click on the <strong>OK</strong> button to produce the tables as shown below.</li>'
var1instructhtml += '</ul>'

var1prachtml += var1instructhtml

var1prachtml += '<p>'
var1prachtml += 'The first table contains all the summary statistics that we requested for the variable as shown below:'
var1prachtml += '</p>'

var1prachtml += outputs['Table_Statistics.htm'].content

var1prachtml += '<p>'
var1prachtml += 'We can see here that we have ' + str(Nvalid1) + ' valid values for the variable ' + varname1short + ', '
if Nmiss1 == 0:
    var1prachtml += 'and no missing data. '
else:
    var1prachtml += 'and ' + str(Nmiss1) + ' values are missing. '
var1prachtml += '</p>'

var1prachtml += '<p>'
var1prachtml += 'The statistics begins with three measures of the centre of the distribution, the mean, median and the mode. '
var1prachtml += 'For the variable ' + varname1short + ' we find that the arithmetic mean (or average) value is ' + means1_str + '. '
var1prachtml += 'The median or middle value is ' + medians1_str + '. '
if medians1_num > means1_num:
    var1prachtml += 'This is larger than the mean so if there is any skew to the distribution it will likely be negative. '
elif medians1_num < means1_num:
    var1prachtml += 'This is smaller than the mean so if there is any skew to the distribution it will likely be positive. '
var1prachtml += 'The third measure is the mode or most frequent value which takes value ' + modes1_str + '. '
var1prachtml += 'SPSS calculates this by looking at the frequencies of each possible value so the mode is probably more useful for categorical data. '
var1prachtml += 'You can check this by looking at the second table produced by the command '
if len(freqs1) < 16:
    var1prachtml += '(shown after this explanation) '
var1prachtml += 'which shows the frequencies of each value. '
var1prachtml += '</p>' 

var1prachtml += '<p>'
var1prachtml += 'We next look at measures of the spread of values for the variable ' + varname1short + '. '
var1prachtml += 'These begin with the standard deviation which takes value ' + sds1_str + ' and its squared value, the variance which takes value '+ variances1_str + '. '
lowlim = means1_num-2*sds1_num
hilim = means1_num+2*sds1_num
var1prachtml += 'Typically, if the data are normally distributed, appoximately 95% of observations will lie within 2 standard deviations of the mean i.e. between ' + str(numpy.round(lowlim, 3)) + ' and ' + str(numpy.round(hilim, 3)) + '. '
var1prachtml += 'The smallest value observed is ' + mins1_str + ' and the largest value is ' + maxs1_str + ' giving an overall range of length ' + ranges1_str + '. '
var1prachtml += 'We can finally see the quartiles of the distribution under the more general percentiles heading and so we see the lower (25%) quartile takes value ' + centiles1_str[0] + ' meaning that 25% of observations are below this value. '
var1prachtml += 'Conversely 25% of observations are above the upper (75%) quartile which takes value ' + centiles1_str[2] + '. '
var1prachtml += 'The 50% quantile is the median which we covered earlier. '
var1prachtml += '</p>'

if len(freqs1) < 16:
    var1prachtml += '<p>'
    var1prachtml += 'The SPSS command also produces a second table shown below of all the observed values for ' + varname1short + ' and their frequencies. '
    var1prachtml += '</p>'
    var1prachtml += outputs['Table_Frequencies.htm'].content

    var1prachtml += '<p>'
    var1prachtml += 'We do not have much to add about this table aside to mention that the mode, ' + modes1_str + ' can be found by finding the value with the largest frequency. '
    var1prachtml += 'The Cumulative Percent column can also be used to confirm the percentiles and you should find that if you scan down this column that the values pass 25% at the value quoted as the lower quartile and similarly passing 50% and 75% for the median and upper quartile respectively. '
    var1prachtml += '</p>'
else:
    var1prachtml += '<p>'
    var1prachtml += 'The SPSS command also produces a second table  of all the observed values for ' + varname1short + ' and their frequencies although this is not very useful when the data is not categorical. '
    var1prachtml += '</p>'
outputs['variable1.html'] = HTMLOutput(var1prachtml, "Variable one text")

var1questhtml = '<p>'
var1questhtml += 'Use the Frequencies window in SPSS to answer the following questions about ' + varname1short + ':'
var1questhtml += '</p>'
var1questhtml += '<ul>'
var1questhtml += '<li>Question: What is the mean for ' + varname1short + '?</li>'
var1questhtml += '<li>Question: What is the median for ' + varname1short + ' and is it bigger than the mean?</li>'
var1questhtml += '<li>Question: What is the mode for ' + varname1short + ' and how is it calculated?</li>'
var1questhtml += '<li>Question: What is the standard deviation for ' + varname1short + ' and what does it tell us?</li>'
var1questhtml += '<li>Question: What is the smallest value for ' + varname1short + '?</li>'
var1questhtml += '<li>Question: What is the largest value for ' + varname1short + '?</li>'
var1questhtml += '<li>Question: What are the lower and upper limits of the interquartile range for ' + varname1short + ' and how wide is it?</li>'
var1questhtml += '</ul>'

outputs['variable1questions.html'] = HTMLOutput(var1questhtml, "Questions for variable one text")

var1solhtml = '<p>'
var1solhtml += 'The SPSS instructions for the first table of descriptive statistics are as follows: '
var1solhtml += '</p>'
var1solhtml += '<ul class = "instructions">'
var1solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var1solhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
var1solhtml += '<li>Click on the <strong>Statistics</strong> button to go to the statistics screen.</li>'
var1solhtml += '<li>Here we need to select ALL the summary statistics that we are interested in looking at.</li>'
var1solhtml += '<li>Select <strong>Mean, Median</strong> and <strong>Mode</strong> from under <strong>Central Tendency</strong>.</li>'
var1solhtml += '<li>Select <strong>Std. deviation, Variance, Range, Minimum</strong> and <strong>Maximum</strong> from under <strong>Dispersion</strong>.</li>'
var1solhtml += '<li>Finally Select <strong>Quartiles</strong> from under <strong>Percentile Values.</strong></li>'
var1solhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
var1solhtml += '<li>Click on the <strong>OK</strong> button to produce the tables required.</li>'
var1solhtml += '</ul>'
var1solhtml += '<ul>'
var1solhtml += '<li>Question: What is the mean for ' + varname1short + '?</li>'
var1solhtml += '<li>Question: What is the median for ' + varname1short + ' and is it bigger than the mean?</li>'
var1solhtml += '<li>Question: What is the mode for ' + varname1short + ' and how is it calculated?</li>'
var1solhtml += '<li>Question: What is the standard deviation for ' + varname1short + ' and what does it tell us?</li>'
var1solhtml += '<li>Question: What is the smallest value for ' + varname1short + '?</li>'
var1solhtml += '<li>Question: What is the largest value for ' + varname1short + '?</li>'
var1solhtml += '<li>Question: What are the lower and upper limits of the interquartile range for ' + varname1short + ' and how wide is it?</li>'
var1solhtml += '</ul>'
var1solhtml += '<p>'
var1solhtml += 'Solutions: The first tabular output from SPSS is as follows:'
var1solhtml += '</p>'
var1solhtml += outputs['Table_Statistics.htm'].content
var1solhtml += '<p>'
var1solhtml += 'The answers are as follows:'
var1solhtml += '</p>'
var1solhtml += '<ul>'
var1solhtml += '<li>For the variable ' + varname1short + ' the arithmetic mean (or average) value is ' + means1_str + '.</li>'
var1solhtml += '<li>'
var1solhtml += 'The median or middle value is ' + medians1_str + '. '
if medians1_num > means1_num:
    var1solhtml += 'This is larger than the mean so if there is any skew to the distribution it will likely be negative. '
elif medians1_num < means1_num:
    var1solhtml += 'This is smaller than the mean so if there is any skew to the distribution it will likely be positive. '
var1solhtml += '</li>'
var1solhtml += '<li>'
var1solhtml += 'The mode or most frequent value which takes value ' + modes1_str + '. '
var1solhtml += 'SPSS calculates this by looking at the frequencies of each possible value so the mode is probably more useful for categorical data. '
var1solhtml += '</li>'
var1solhtml += '<li>'
var1solhtml += 'The standard deviation of ' + varname1short + ' takes value ' + sds1_str + '. '
lowlim = means1_num-2*sds1_num
hilim = means1_num+2*sds1_num
var1solhtml += 'Typically 95% of observations will lie within 2 standard deviations of the mean i.e. between ' + str(numpy.round(lowlim, 3)) + ' and ' + str(numpy.round(hilim, 3)) + '. '
var1solhtml += '</li>'
var1solhtml += '<li>The smallest value observed is ' + mins1_str+ '.</li>'
var1solhtml += '<li>The largest value observed is ' + maxs1_str+ '.</li>'
var1solhtml += '<li>'
var1solhtml += 'The inter-quartile range runs from the lower quartile to the upper quartile. '
var1solhtml += 'The lower (25%) quartile takes value ' + centiles1_str[0] + ' meaning that 25% of observations are below this value. '
var1solhtml += 'Conversely 25% of observations are above the upper (75%) quartile which takes value ' + centiles1_str[2] + '. '
var1solhtml += 'Therefore the interquartile range is of length ' + centiles1_str[2] + ' - ' + centiles1_str[0] + ' = ' + str(centiles1_num[2] - centiles1_num[0]) + '. '
var1solhtml += '</li>'
var1solhtml += '</ul>'

outputs['variable1answers.html'] = HTMLOutput(var1solhtml, "Answers for variable one text")


var2prachtml = '<p>'
var2prachtml += 'We will now move on to looking at a second variable, ' + varname2short + '. '
var2prachtml += '</p>'

var2prachtml += '<p>'
var2prachtml += 'This can be done in SPSS as follows:'
var2prachtml += '</p>'

var2instructhtml = '<ul class = "instructions">'
var2instructhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var2instructhtml += '<li>Remove the ' + varname1long + ' variable from the <strong> Variable(s)</strong> box.</li>'
var2instructhtml += '<li>Copy the ' + varname2long + ' variable into the <strong> Variable(s)</strong> box.</li>'
var2instructhtml += '<li>The <strong>Statistics</strong> options will be remembered so do not need adding again.</li>'
var2instructhtml += '<li>Click on the <strong>OK</strong> button to produce the tables as shown below.</li>'
var2instructhtml += '</ul>'

var2prachtml += var2instructhtml

var2prachtml += '<p>'
var2prachtml += 'Once again the first table contains all the summary statistics that we requested for the variable as shown below:'
var2prachtml += '</p>'

var2prachtml += outputs['Table_Statistics_1.htm'].content

var2prachtml += '<p>'
var2prachtml += 'This time we can see that we have ' + str(Nvalid2) + ' valid values for the variable ' + varname2short + ', '
if Nmiss2 == 0:
    var2prachtml += 'and no missing data. '
else:
    var2prachtml += 'and ' + str(Nmiss2) + ' values are missing. '
var2prachtml += '</p>'

var2prachtml += '<p>'
var2prachtml += 'The three measures of the centre of the distribution, the mean, median and the mode appear next. '
var2prachtml += 'For the variable ' + varname2short + ' we find that the arithmetic mean (or average) value is ' + means2_str + '. '
var2prachtml += 'The median or middle value is ' + medians2_str + '. '
if medians2_num > means2_num:
    var2prachtml += 'This is larger than the mean so if there is any skew to the distribution it will likely be negative. '
elif medians2_num < means2_num:
    var2prachtml += 'This is smaller than the mean so if there is any skew to the distribution it will likely be positive. '
var2prachtml += 'The third measure is the mode or most frequent value which takes value ' + modes2_str + '. '
var2prachtml += 'You can again check this is the most frequent value by looking at the second table produced by the command'
if len(freqs2) < 16:
    var2prachtml += ' (shown after this explanation)'
var2prachtml += '. '
var2prachtml += '</p>' 

var2prachtml += '<p>'
var2prachtml += 'We next look at measures of the spread of values for the variable ' + varname2short + '. '
var2prachtml += 'These begin with the standard deviation which takes value ' + sds2_str + ' and its squared value, the variance which takes value ' + variances2_str + '. '
lowlim = means2_num-2*sds2_num
hilim = means2_num+2*sds2_num
var2prachtml += 'As noted, typically 95% of observations will lie within 2 standard deviations of the mean i.e. between ' + str(numpy.round(lowlim, 3)) + ' and ' + str(numpy.round(hilim, 3)) + '. '
var2prachtml += 'The smallest value observed is ' + mins2_str + ' and the largest value is ' + maxs2_str + ' giving an overall range of length ' + ranges2_str + '. '
var2prachtml += 'We can finally see the quartiles of the distribution under the more general percentiles heading and so we see the lower (25%) quartile takes value ' + centiles2_str[0] + ' meaning that 25% of observations are below this value. '
var2prachtml += 'Conversely 25% of observations are above the upper (75%) quartile which takes value ' + centiles2_str[2] + '. '
var2prachtml += 'The 50% quantile is the median which we covered earlier. '
var2prachtml += '</p>'

if len(freqs2) < 16:
    var2prachtml += '<p>'
    var2prachtml += 'As before, the SPSS command also produces a second table shown below of all the observed values for ' + varname2short + ' and their frequencies. '
    var2prachtml += '</p>'
    var2prachtml += outputs['Table_Frequencies_1.htm'].content

    var2prachtml += '<p>'
    var2prachtml += 'We do not have much to add about this table aside to mention that the mode, ' + modes2_str + ' can be found by finding the value with the largest frequency. '
    var2prachtml += 'The Cumulative Percent column can also be used to confirm the percentiles and you should find that if you scan down this column that the values pass 25% at the value quoted as the lower quartile and similarly passing 50% and 75% for the median and upper quartile respectively. '
    var2prachtml += '</p>'
else:
    var2prachtml += '<p>'
    var2prachtml += 'As before, the SPSS command also produces a second table  of all the observed values for ' + varname2short + ' and their frequencies although this is not very useful when the data is not categorical. '
    var2prachtml += '</p>'

outputs['variable2.html'] = HTMLOutput(var2prachtml, "Variable two text")

var2questhtml = '<p>'
var2questhtml += 'Use the Frequencies window in SPSS to answer the following questions about ' + varname2short + ':'
var2questhtml += '</p>'
var2questhtml += '<ul>'
var2questhtml += '<li>Question: What is the mean for ' + varname2short + '?</li>'
var2questhtml += '<li>Question: What is the median for ' + varname2short + ' and is it bigger than the mean?</li>'
var2questhtml += '<li>Question: What is the mode for ' + varname2short + ' and how is it calculated?</li>'
var2questhtml += '<li>Question: What is the standard deviation for ' + varname2short + ' and what does it tell us?</li>'
var2questhtml += '<li>Question: What is the smallest value for ' + varname2short + '?</li>'
var2questhtml += '<li>Question: What is the largest value for ' + varname2short + '?</li>'
var2questhtml += '<li>Question: What are the lower and upper limits of the interquartile range for ' + varname2short + ' and how wide is it?</li>'
var2questhtml += '</ul>'

outputs['variable2questions.html'] = HTMLOutput(var2questhtml, "Questions for variable two text")

var2solhtml = '<p>'
var2solhtml += 'The SPSS instructions for the second table of descriptive statistics are as follows: '
var2solhtml += '</p>'
var2solhtml += '<ul class = "instructions">'
var2solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
var2solhtml += '<li>Remove the ' + varname1long + ' variable from the <strong> Variable(s)</strong> box.</li>'
var2solhtml += '<li>Copy the ' + varname2long + ' variable into the <strong> Variable(s)</strong> box.</li>'
var2solhtml += '<li>The <strong>Statistics</strong> options will be remembered so do not need adding again.</li>'
var2solhtml += '<li>Click on the <strong>OK</strong> button to produce the tables required.</li>'
var2solhtml += '</ul>'
var2solhtml += '<ul>'
var2solhtml += '<li>Question: What is the mean for ' + varname2short + '?</li>'
var2solhtml += '<li>Question: What is the median for ' + varname2short + ' and is it bigger than the mean?</li>'
var2solhtml += '<li>Question: What is the mode for ' + varname2short + ' and how is it calculated?</li>'
var2solhtml += '<li>Question: What is the standard deviation for ' + varname2short + ' and what does it tell us?</li>'
var2solhtml += '<li>Question: What is the smallest value for ' + varname2short + '?</li>'
var2solhtml += '<li>Question: What is the largest value for ' + varname2short + '?</li>'
var2solhtml += '<li>Question: What are the lower and upper limits of the interquartile range for ' + varname2short + ' and how wide is it?</li>'
var2solhtml += '</ul>'
var2solhtml += '<p>'
var2solhtml += 'Solutions: The first tabular output from SPSS is as follows:'
var2solhtml += '</p>'
var2solhtml += outputs['Table_Statistics_1.htm'].content
var2solhtml += '<p>'
var2solhtml += 'The answers are as follows:'
var2solhtml += '</p>'
var2solhtml += '<ul>'
var2solhtml += '<li>For the variable ' + varname2short + ' the arithmetic mean (or average) value is ' + means2_str + '.</li>'
var2solhtml += '<li>'
var2solhtml += 'The median or middle value is ' + medians2_str + '. '
if medians2_num > means2_num:
    var2solhtml += 'This is larger than the mean so if there is any skew to the distribution it will likely be negative. '
elif medians2_num < means2_num:
    var2solhtml += 'This is smaller than the mean so if there is any skew to the distribution it will likely be positive. '
var2solhtml += '</li>'
var2solhtml += '<li>'
var2solhtml += 'The mode or most frequent value which takes value ' + modes2_str + '. '
var2solhtml += 'SPSS calculates this by looking at the frequencies of each possible value so the mode is probably more useful for categorical data. '
var2solhtml += '</li>'
var2solhtml += '<li>'
var2solhtml += 'The standard deviation of ' + varname2short + ' takes value ' + sds2_str + '. '
lowlim = means2_num-2*sds2_num
hilim = means2_num+2*sds2_num
var2solhtml += 'Typically 95% of observations will lie within 2 standard deviations of the mean i.e. between ' + str(numpy.round(lowlim, 3)) + ' and ' + str(numpy.round(hilim, 3)) + '. '
var2solhtml += '</li>'
var2solhtml += '<li>The smallest value observed is ' + mins2_str+ '.</li>'
var2solhtml += '<li>The largest value observed is ' + maxs2_str+ '.</li>'
var2solhtml += '<li>'
var2solhtml += 'The inter-quartile range runs from the lower quartile to the upper quartile. '
var2solhtml += 'The lower (25%) quartile takes value ' + centiles2_str[0] + ' meaning that 25% of observations are below this value. '
var2solhtml += 'Conversely 25% of observations are above the upper (75%) quartile which takes value ' + centiles2_str[2] + '. '
var2solhtml += 'Therefore the interquartile range is of length ' + centiles2_str[2] + ' - ' + centiles2_str[0] + ' = ' + str(centiles2_num[2] - centiles2_num[0]) + '. '
var2solhtml += '</li>'
var2solhtml += '</ul>'

outputs['variable2answers.html'] = HTMLOutput(var2solhtml, "Answers for variable two text")

hist1prachtml = '<p>'
hist1prachtml += 'We will next look at the data graphically but again using options from the Frequencies screen in SPSS for our first variable, ' + varname1short + ' as follows:'
hist1prachtml += '</p>'

hist1instrhtml = '<ul class = "instructions">'
hist1instrhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
hist1instrhtml += '<li>Remove the ' + varname2long + ' variable from the <strong>Variable(s)</strong> box.</li>'
hist1instrhtml += '<li>Return the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
hist1instrhtml += '<li>Click on the <strong>Charts...</strong> button to bring up the chart options.</li>'
hist1instrhtml += '<li>Click on the <strong>Histogram</strong> Chart type and also the <strong>Show normal curve on histogram</strong> tick box.</li>'
hist1instrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
hist1instrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
hist1instrhtml += '</ul>'

hist1prachtml += hist1instrhtml

hist1prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

hist1prachtml += '<p>'
hist1prachtml += 'The graph produced is a histogram and basically is somewhat similar to a bar graph but each bar in the histogram represents a range of values and as a result there are no gaps between bars. '
hist1prachtml += 'SPSS chooses the limits for each bar and actually plots frequencies of observations that lie between the limits. To check this you might compare the frequencies in the table with the bars in the graph. ' 
hist1prachtml += 'We have asked for a normal curve to be superimposed on the plot and this curve is a plot of the normal distribution that has the same mean and standard deviation as the data. '
hist1prachtml += 'Some statistical tests rely on the variable approximately following a normal distribution and if this is true then the histogram should roughly follow the curve. '
hist1prachtml += 'If the data is skewed, for example, then this will not be the case. '
hist1prachtml += '</p>'

outputs['histogram1.html'] = HTMLOutput(hist1prachtml, "Histogram one text")

hist1questhtml = '<p>'
hist1questhtml += 'Use the Frequencies window  to construct a histogram in SPSS to answer the following questions about ' + varname1short + ':'
hist1questhtml += '</p>'
hist1questhtml += '<ul>'
hist1questhtml += '<li>Question: What does the histogram show?</li>'
hist1questhtml += '</ul>'

outputs['histogram1questions.html'] = HTMLOutput(hist1questhtml, "Histogram one question text")

hist1solhtml = '<p>'
hist1solhtml += 'The SPSS instructions for the first histogram are as follows: '
hist1solhtml += '</p>'
hist1solhtml += '<ul class = "instructions">'
hist1solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
hist1solhtml += '<li>Remove the ' + varname2long + ' variable from the <strong>Variable(s)</strong> box.</li>'
hist1solhtml += '<li>Return the ' + varname1long + ' variable into the <strong>Variable(s)</strong> box.</li>'
hist1solhtml += '<li>Click on the <strong>Charts...</strong> button to bring up the chart options.</li>'
hist1solhtml += '<li>Click on the <strong>Histogram</strong> Chart type and also the <strong>Show normal curve on histogram</strong> tick box.</li>'
hist1solhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
hist1solhtml += '<li>Click on the <strong>OK</strong> button to produce the graph required.</li>'
hist1solhtml += '</ul>'
hist1solhtml += '<ul>'
hist1solhtml += '<li>Question: What does the histogram show?</li>'
hist1solhtml += '</ul>'
hist1solhtml += '<p>'
hist1solhtml += 'Solution: The output from SPSS is as follows:'
hist1solhtml += '</p>'
hist1solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
hist1solhtml += '<p>'
hist1solhtml += 'The histogram is somewhat similar to a bar graph but each bar in the histogram represents a range of values and as a result there are no gaps between bars. '
hist1solhtml += 'SPSS chooses the limits for each bar and actually plots frequencies of observations that lie between the limits. ' 
hist1solhtml += 'We have asked for a normal curve to be superimposed on the plot and this curve is a plot of the normal distribution that has the same mean and standard deviation as the data. '
hist1solhtml += 'If the variable approximately follows a normal distribution then the histogram should roughly follow the curve. '
hist1solhtml += 'If the data is skewed then this will not be the case. '
hist1solhtml += '</p>'

outputs['histogram1answers.html'] = HTMLOutput(hist1solhtml, "Histogram one question text")


hist2prachtml = '<p>'
hist2prachtml += 'We will next look at plotting a similar histogram for the second variable, ' + varname2short + ' as follows:'
hist2prachtml += '</p>'

hist2instrhtml = '<ul class = "instructions">'
hist2instrhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
hist2instrhtml += '<li>Remove the ' + varname1long + ' variable from the <strong>Variable(s)</strong> box.</li>'
hist2instrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Variable(s)</strong> box.</li>'
hist2instrhtml += '<li>The <strong>Charts</strong> options will be remembered so do not need adding again.</li>'
hist2instrhtml += '<li>Click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
hist2instrhtml += '</ul>'

hist2prachtml += hist2instrhtml

hist2prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'

hist2prachtml += '<p>'
hist2prachtml += 'Again we can look at the shape of the histogram and check for unusual observations and compare the graph with the plot of the normal distribution. '
hist2prachtml += '</p>'

outputs['histogram2.html'] = HTMLOutput(hist2prachtml, "Histogram two text")

hist2questhtml = '<p>'
hist2questhtml += 'Use the Frequencies window  to construct a histogram in SPSS to answer the following questions about ' + varname2short + ':'
hist2questhtml += '</p>'
hist2questhtml += '<ul>'
hist2questhtml += '<li>Question: What should we look for in this histogram?</li>'
hist2questhtml += '</ul>'

outputs['histogram2questions.html'] = HTMLOutput(hist2questhtml, "Histogram two question text")

hist2solhtml = '<p>'
hist2solhtml += 'The SPSS instructions for the second histogram are as follows: '
hist2solhtml += '</p>'
hist2solhtml += '<ul class = "instructions">'
hist2solhtml += '<li>Select <strong>Frequencies</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
hist2solhtml += '<li>Remove the ' + varname1long + ' variable from the <strong>Variable(s)</strong> box.</li>'
hist2solhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Variable(s)</strong> box.</li>'
hist2solhtml += '<li>The <strong>Charts</strong> options will be remembered so do not need adding again.</li>'
hist2solhtml += '<li>Click on the <strong>OK</strong> button to produce the graph required.</li>'
hist2solhtml += '</ul>'
hist2solhtml += '<ul>'
hist2solhtml += '<li>Question: What should we look for in this histogram?</li>'
hist2solhtml += '</ul>'
hist2solhtml += '<p>'
hist2solhtml += 'Solution: The output from SPSS is as follows:'
hist2solhtml += '</p>'
hist2solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_1_000.png'].content)  + '">'
hist2solhtml += '<p>'
hist2solhtml += 'We can look at the shape of the histogram and check for unusual observations and compare the graph with the plot of the normal distribution. '
hist2solhtml += '</p>'
outputs['histogram2answers.html'] = HTMLOutput(hist2solhtml, "Histogram two question text")

box1prachtml = '<p>'
box1prachtml += 'We will finish up this practical by looking at a second plot called a boxplot. '
box1prachtml += 'We will do this first for variable, ' + varname1short + '. '
box1prachtml += 'The boxplot is not available from the Frequencies option so instead we need to do the following in SPSS:'
box1prachtml += '</p>'

box1instrhtml = '<ul class = "instructions">'
box1instrhtml += '<li>Select <strong>Boxplot</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
box1instrhtml += '<li>We want to choose <strong>Simple</strong> and <strong>Summaries of separate variables</strong> from the options here.</li>'
box1instrhtml += '<li>Next click on <strong>Define</strong> to set up the box plot.</li>'
box1instrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box1instrhtml += '<li>Ignore the rest of the options and click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
box1instrhtml += '</ul>'

box1prachtml += box1instrhtml

box1prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'

box1prachtml += '<p>'
box1prachtml += 'Here we see the boxplot for the ' + varname1short + ' variable. '
box1prachtml += 'The central box covers the interquartile range and so it has a value at the bottom of ' + centiles1_str[0] + ' and at the top of ' + centiles1_str[2] + '. '
box1prachtml += 'The median which takes value ' + centiles1_str[1] + ' is represented by a vertical line in the middle of the box. '
box1prachtml += 'The lines stretching out of the box to form T shapes are known as whiskers and will show the range unless there are any outliers (defined here as points 1.5 times the height of the box away from the box). '
box1prachtml += 'If there are outliers the whiskers will end at 1.5 times the height of the box away from the box and the outliers will be marked as circles with a number representing which observation number they are. '
box1prachtml += '</p>'
outputs['boxplot1.html'] = HTMLOutput(box1prachtml, "Boxplot one text")

box1questhtml = '<p>'
box1questhtml += 'Use SPSS to construct a boxplot and answer the following questions about ' + varname1short + ':'
box1questhtml += '</p>'
box1questhtml += '<ul>'
box1questhtml += '<li>Question: From the boxplot what is the median (approximately) for ' + varname1short + '?</li>'
box1questhtml += '<li>Question: What is the lower quartile (approximately) for ' + varname1short + '?</li>'
box1questhtml += '<li>Question: What is the upper quartile (approximately) for ' + varname1short + '?</li>'
box1questhtml += '</ul>'

outputs['boxplot1questions.html'] = HTMLOutput(box1questhtml, "Boxplot one question text")

box1solhtml = '<p>'
box1solhtml += 'The SPSS instructions for the first boxplot are as follows: '
box1solhtml += '</p>'
box1solhtml += '<ul class = "instructions">'
box1solhtml += '<li>Select <strong>Boxplot</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
box1solhtml += '<li>We want to choose <strong>Simple</strong> and <strong>Summaries of separate variables</strong> from the options here.</li>'
box1solhtml += '<li>Next click on <strong>Define</strong> to set up the box plot.</li>'
box1solhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box1solhtml += '<li>Ignore the rest of the options and click on the <strong>OK</strong> button to produce the graph required.</li>'
box1solhtml += '</ul>'
box1solhtml += '<ul>'
box1solhtml += '<li>Question: From the boxplot what is the median (approximately) for ' + varname1short + '?</li>'
box1solhtml += '<li>Question: What is the lower quartile (approximately) for ' + varname1short + '?</li>'
box1solhtml += '<li>Question: What is the upper quartile (approximately) for ' + varname1short + '?</li>'
box1solhtml += '</ul>'
box1solhtml += '<p>'
box1solhtml += 'Solution: The output from SPSS is as follows:'
box1solhtml += '</p>'
box1solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'
box1solhtml += '<p>'
box1solhtml += 'The answers are as follows:'
box1solhtml += '</p>'
box1solhtml += '<ul>'
box1solhtml += '<li>The median which takes value ' + centiles1_str[1] + ' is represented by a vertical line in the middle of the box.</li> '
box1solhtml += '<li>The lower quartile which takes value ' + centiles1_str[0] + ' is represented by the bottom of the box.</li> '
box1solhtml += '<li>The upper quartile which takes value ' + centiles1_str[2] + ' is represented by the top of the box.</li> '
box1solhtml += '</ul>'

outputs['boxplot1answers.html'] = HTMLOutput(box1solhtml, "Boxplot one question text")

box2prachtml = '<p>'
box2prachtml += 'We can also look at a boxplot for variable, ' + varname2short + ' as follows:'
box2prachtml += '</p>'

box2instrhtml = '<ul class = "instructions">'
box2instrhtml += '<li>Select <strong>Boxplot</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
box2instrhtml += '<li>Keep the choices of <strong>Simple</strong> and <strong>Summaries of separate variables </strong> and click on <strong> Define </strong> to set up the box plot.</li>'
box2instrhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box2instrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box2instrhtml += '<li>Ignore the rest of the options and click on the <strong>OK</strong> button to produce the graph as shown below.</li>'
box2instrhtml += '</ul>'

box2prachtml += box2instrhtml

box2prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'

box2prachtml += '<p>'
box2prachtml += 'Here we see the boxplot for the ' + varname2short + ' variable. '
box2prachtml += 'This time the central box has a value at the bottom of ' + centiles2_str[0] + ' and at the top of ' + centiles2_str[2] + '. '
box2prachtml += 'The median which takes value ' + centiles2_str[1] + ' is represented by a vertical line in the middle of the box. '
box2prachtml += '</p>'
outputs['boxplot2.html'] = HTMLOutput(box2prachtml, "Boxplot two text")

box2questhtml = '<p>'
box2questhtml += 'Use SPSS to construct a boxplot and answer the following questions about ' + varname2short + ':'
box2questhtml += '</p>'
box2questhtml += '<ul>'
box2questhtml += '<li>Question: From the boxplot what is the median (approximately) for ' + varname2short + '?</li>'
box2questhtml += '<li>Question: What is the lower quartile (approximately) for ' + varname2short + '?</li>'
box2questhtml += '<li>Question: What is the upper quartile (approximately) for ' + varname2short + '?</li>'
box2questhtml += '</ul>'

outputs['boxplot2questions.html'] = HTMLOutput(box2questhtml, "Boxplot one question text")

box2solhtml = '<p>'
box2solhtml += 'The SPSS instructions for the second boxplot are as follows: '
box2solhtml += '</p>'
box2solhtml += '<ul class = "instructions">'
box2solhtml += '<li>Select <strong>Boxplot</strong> from the <strong>Legacy Dialogs</strong> submenu available from the <strong>Graphs</strong> menu.</li>'
box2solhtml += '<li>Keep the choices of <strong>Simple</strong> and <strong>Summaries of separate variables </strong> and click on <strong> Define </strong> to set up the box plot.</li>'
box2solhtml += '<li>Remove the ' + varname1long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box2solhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Boxes Represent:</strong> box.</li>'
box2solhtml += '<li>Ignore the rest of the options and click on the <strong>OK</strong> button to produce the graph required.</li>'
box2solhtml += '</ul>'
box2solhtml += '<ul>'
box2solhtml += '<li>Question: From the boxplot what is the median (approximately) for ' + varname2short + '?</li>'
box2solhtml += '<li>Question: What is the lower quartile (approximately) for ' + varname2short + '?</li>'
box2solhtml += '<li>Question: What is the upper quartile (approximately) for ' + varname2short + '?</li>'
box2solhtml += '</ul>'
box2solhtml += '<p>'
box2solhtml += 'The output from SPSS is as follows:'
box2solhtml += '</p>'
box2solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'
box2solhtml += '<p>'
box2solhtml += 'The answers are as follows:'
box2solhtml += '</p>'
box2solhtml += '<ul>'
box2solhtml += '<li>The median which takes value ' + centiles2_str[1] + ' is represented by a vertical line in the middle of the box.</li> '
box2solhtml += '<li>The lower quartile which takes value ' + centiles2_str[0] + ' is represented by the bottom of the box.</li> '
box2solhtml += '<li>The upper quartile which takes value ' + centiles2_str[2] + ' is represented by the top of the box.</li> '
box2solhtml += '</ul>'

outputs['boxplot2answers.html'] = HTMLOutput(box2solhtml, "Boxplot one question text")

conchtml = '<p>'
conchtml += 'This graph is the end of the practical. '
conchtml += '</p>'
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['variable1.html'].content
combinedprachtml += outputs['variable2.html'].content
combinedprachtml += outputs['histogram1.html'].content
combinedprachtml += outputs['histogram2.html'].content
combinedprachtml += outputs['boxplot1.html'].content
combinedprachtml += outputs['boxplot2.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['variable1questions.html'].content
combinedquesthtml += outputs['variable2questions.html'].content
combinedquesthtml += outputs['histogram1questions.html'].content
combinedquesthtml += outputs['histogram2questions.html'].content
combinedquesthtml += outputs['boxplot1questions.html'].content
combinedquesthtml += outputs['boxplot2questions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['variable1answers.html'].content
combinedsolhtml += outputs['variable2answers.html'].content
combinedsolhtml += outputs['histogram1answers.html'].content
combinedsolhtml += outputs['histogram2answers.html'].content
combinedsolhtml += outputs['boxplot1answers.html'].content
combinedsolhtml += outputs['boxplot2answers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''