# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Calculate Descriptive Statistics in SPSS (equivalent to Analyze->Descriptive Statistics->Descriptives... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
y = DataVector('Response variable: ')
pred1 = DataVector('First predictor variable: ')
pred2 = DataVector('Second predictor variable: ')
'''

    spssscript = '''

REGRESSION
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN 
  /DEPENDENT ${y}
  /METHOD=ENTER ${pred1}.

REGRESSION
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN 
  /DEPENDENT ${y}
  /METHOD=ENTER ${pred2}.

REGRESSION
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN 
  /DEPENDENT ${y}
  /METHOD=ENTER ${pred2} ${pred1}.

REGRESSION
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN 
  /DEPENDENT ${y}
  /METHOD=ENTER ${pred1}
  /METHOD=ENTER ${pred2}
  /SCATTERPLOT=(*ZRESID, ${pred1})
  /SCATTERPLOT=(*ZRESID, ${pred2})
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID).

CORRELATIONS
  /VARIABLES=${y} ${pred1} ${pred2} 
  /PRINT=TWOTAIL NOSIG
  /STATISTICS DESCRIPTIVES
  /MISSING=PAIRWISE.

'''

    processoutputs = '''
import EStat
from EStat.Templating import *
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

anovaout = etree.parse(BytesIO(outputs['Table_ANOVA.xml'].content))
print(etree.tostring(anovaout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@number"
aovss1_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovss1_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@text"
aovss1_str = anovaout.xpath(path, namespaces=ns)
print(aovss1_str)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@number"
aovms1_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovms1_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@text"
aovms1_str = anovaout.xpath(path, namespaces=ns)
print(aovms1_str)

path = "//x:dimension[@axis='column']/x:category[@text='df']/x:cell/@number"
aovdf1 = [int(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovdf1)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@number"
aovf1_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovf1_num)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@text"
aovf1_str = anovaout.xpath(path, namespaces=ns)
print(aovf1_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
aovsig1_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovsig1_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
aovsig1_str = anovaout.xpath(path, namespaces=ns)
print(aovsig1_str)

anovaout = etree.parse(BytesIO(outputs['Table_ANOVA_1.xml'].content))
print(etree.tostring(anovaout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@number"
aovss2_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovss2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@text"
aovss2_str = anovaout.xpath(path, namespaces=ns)
print(aovss2_str)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@number"
aovms2_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovms2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@text"
aovms2_str = anovaout.xpath(path, namespaces=ns)
print(aovms2_str)

path = "//x:dimension[@axis='column']/x:category[@text='df']/x:cell/@number"
aovdf2 = [int(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovdf2)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@number"
aovf2_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovf2_num)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@text"
aovf2_str = anovaout.xpath(path, namespaces=ns)
print(aovf2_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
aovsig2_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovsig2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
aovsig2_str = anovaout.xpath(path, namespaces=ns)
print(aovsig2_str)

anovaout = etree.parse(BytesIO(outputs['Table_ANOVA_2.xml'].content))
print(etree.tostring(anovaout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@number"
aovss3_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovss3_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@text"
aovss3_str = anovaout.xpath(path, namespaces=ns)
print(aovss3_str)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@number"
aovms3_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovms3_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@text"
aovms3_str = anovaout.xpath(path, namespaces=ns)
print(aovms3_str)

path = "//x:dimension[@axis='column']/x:category[@text='df']/x:cell/@number"
aovdf3 = [int(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovdf3)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@number"
aovf3_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovf3_num)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@text"
aovf3_str = anovaout.xpath(path, namespaces=ns)
print(aovf3_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
aovsig3_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovsig3_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
aovsig3_str = anovaout.xpath(path, namespaces=ns)
print(aovsig3_str)

anovaout = etree.parse(BytesIO(outputs['Table_ANOVA_3.xml'].content))
print(etree.tostring(anovaout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@number"
aovss123_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovss123_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sum of Squares']/x:cell/@text"
aovss123_str = anovaout.xpath(path, namespaces=ns)
print(aovss123_str)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@number"
aovms123_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovms123_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean Square']/x:cell/@text"
aovms123_str = anovaout.xpath(path, namespaces=ns)
print(aovms123_str)

path = "//x:dimension[@axis='column']/x:category[@text='df']/x:cell/@number"
aovdf123 = [int(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovdf123)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@number"
aovf123_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovf123_num)

path = "//x:dimension[@axis='column']/x:category[@text='F']/x:cell/@text"
aovf123_str = anovaout.xpath(path, namespaces=ns)
print(aovf123_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
aovsig123_num = [str2float(x) for x in anovaout.xpath(path, namespaces=ns)]
print(aovsig123_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
aovsig123_str = anovaout.xpath(path, namespaces=ns)
print(aovsig123_str)

coefout = etree.parse(BytesIO(outputs['Table_Coefficients.xml'].content))
print(etree.tostring(coefout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@number"
coefb1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefb1_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@text"
coefb1_str = coefout.xpath(path, namespaces=ns)
print(coefb1_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@number"
coefse1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefse1_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@text"
coefse1_str = coefout.xpath(path, namespaces=ns)
print(coefse1_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@number"
coefbeta1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefbeta1_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@text"
coefbeta1_str = coefout.xpath(path, namespaces=ns)
print(coefbeta1_str)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@number"
coeft1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeft1_num)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@text"
coeft1_str = coefout.xpath(path, namespaces=ns)
print(coeft1_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
coefsig1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefsig1_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
coefsig1_str = coefout.xpath(path, namespaces=ns)
print(coefsig1_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@number"
coeflb1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeflb1_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@text"
coeflb1_str = coefout.xpath(path, namespaces=ns)
print(coeflb1_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@number"
coefub1_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefub1_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@text"
coefub1_str = coefout.xpath(path, namespaces=ns)
print(coefub1_str)

coefout = etree.parse(BytesIO(outputs['Table_Coefficients_1.xml'].content))
print(etree.tostring(coefout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@number"
coefb2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefb2_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@text"
coefb2_str = coefout.xpath(path, namespaces=ns)
print(coefb2_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@number"
coefse2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefse2_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@text"
coefse2_str = coefout.xpath(path, namespaces=ns)
print(coefse2_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@number"
coefbeta2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefbeta2_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@text"
coefbeta2_str = coefout.xpath(path, namespaces=ns)
print(coefbeta2_str)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@number"
coeft2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeft2_num)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@text"
coeft2_str = coefout.xpath(path, namespaces=ns)
print(coeft2_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
coefsig2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefsig2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
coefsig2_str = coefout.xpath(path, namespaces=ns)
print(coefsig2_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@number"
coeflb2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeflb2_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@text"
coeflb2_str = coefout.xpath(path, namespaces=ns)
print(coeflb2_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@number"
coefub2_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefub2_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@text"
coefub2_str = coefout.xpath(path, namespaces=ns)
print(coefub2_str)

coefout = etree.parse(BytesIO(outputs['Table_Coefficients_2.xml'].content))
print(etree.tostring(coefout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@number"
coefb3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefb3_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@text"
coefb3_str = coefout.xpath(path, namespaces=ns)
print(coefb3_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@number"
coefse3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefse3_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@text"
coefse3_str = coefout.xpath(path, namespaces=ns)
print(coefse3_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@number"
coefbeta3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefbeta3_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@text"
coefbeta3_str = coefout.xpath(path, namespaces=ns)
print(coefbeta3_str)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@number"
coeft3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeft3_num)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@text"
coeft3_str = coefout.xpath(path, namespaces=ns)
print(coeft3_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
coefsig3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefsig3_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
coefsig3_str = coefout.xpath(path, namespaces=ns)
print(coefsig3_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@number"
coeflb3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeflb3_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@text"
coeflb3_str = coefout.xpath(path, namespaces=ns)
print(coeflb3_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@number"
coefub3_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefub3_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@text"
coefub3_str = coefout.xpath(path, namespaces=ns)
print(coefub3_str)

coefout = etree.parse(BytesIO(outputs['Table_Coefficients_3.xml'].content))
print(etree.tostring(coefout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@number"
coefb123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefb123_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='B']/x:cell/@text"
coefb123_str = coefout.xpath(path, namespaces=ns)
print(coefb123_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@number"
coefse123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefse123_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Std. Error']/x:cell/@text"
coefse123_str = coefout.xpath(path, namespaces=ns)
print(coefse123_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@number"
coefbeta123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefbeta123_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Beta']/x:cell/@text"
coefbeta123_str = coefout.xpath(path, namespaces=ns)
print(coefbeta123_str)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@number"
coeft123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeft123_num)

path = "//x:dimension[@axis='column']/x:category[@text='t']/x:cell/@text"
coeft123_str = coefout.xpath(path, namespaces=ns)
print(coeft123_str)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@number"
coefsig123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefsig123_num)

path = "//x:dimension[@axis='column']/x:category[@text='Sig.']/x:cell/@text"
coefsig123_str = coefout.xpath(path, namespaces=ns)
print(coefsig123_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@number"
coeflb123_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coeflb123_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Lower Bound']/x:cell/@text"
coeflb123_str = coefout.xpath(path, namespaces=ns)
print(coeflb123_str)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@number"
coefub_num = [str2float(x) for x in coefout.xpath(path, namespaces=ns)]
print(coefub_num)

path = "//x:dimension[@axis='column']/x:group/x:category[@text='Upper Bound']/x:cell/@text"
coefub_str = coefout.xpath(path, namespaces=ns)
print(coefub_str)

modelout = etree.parse(BytesIO(outputs['Table_Model Summary.xml'].content))
print(etree.tostring(modelout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@number"
r1_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r1_num)

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@text"
r1_str = modelout.xpath(path, namespaces=ns)
print(r1_str)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@number"
rsq1_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(rsq1_num)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@text"
rsq1_str = modelout.xpath(path, namespaces=ns)
print(rsq1_str)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@number"
adjrsq1_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(adjrsq1_num)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@text"
adjrsq1_str = modelout.xpath(path, namespaces=ns)
print(adjrsq1_str)

modelout = etree.parse(BytesIO(outputs['Table_Model Summary_1.xml'].content))
print(etree.tostring(modelout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@number"
r2_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r2_num)

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@text"
r2_str = modelout.xpath(path, namespaces=ns)
print(r2_str)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@number"
rsq2_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(rsq2_num)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@text"
rsq2_str = modelout.xpath(path, namespaces=ns)
print(rsq2_str)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@number"
adjrsq2_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(adjrsq2_num)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@text"
adjrsq2_str = modelout.xpath(path, namespaces=ns)
print(adjrsq2_str)

modelout = etree.parse(BytesIO(outputs['Table_Model Summary_2.xml'].content))
print(etree.tostring(modelout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@number"
r3_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r3_num)

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@text"
r3_str = modelout.xpath(path, namespaces=ns)
print(r3_str)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@number"
rsq3_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(rsq3_num)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@text"
rsq3_str = modelout.xpath(path, namespaces=ns)
print(rsq3_str)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@number"
adjrsq3_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(adjrsq3_num)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@text"
adjrsq3_str = modelout.xpath(path, namespaces=ns)
print(adjrsq3_str)

modelout = etree.parse(BytesIO(outputs['Table_Model Summary_3.xml'].content))
print(etree.tostring(modelout, pretty_print=True))

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@number"
r123_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(r123_num)

path = "//x:dimension[@axis='column']/x:category[@text='R']/x:cell/@text"
r123_str = modelout.xpath(path, namespaces=ns)
print(r123_str)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@number"
rsq123_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(rsq123_num)

path = "//x:dimension[@axis='column']/x:category[@text='R Square']/x:cell/@text"
rsq123_str = modelout.xpath(path, namespaces=ns)
print(rsq123_str)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@number"
adjrsq123_num = [str2float(x) for x in modelout.xpath(path, namespaces=ns)]
print(adjrsq123_num)

path = "//x:dimension[@axis='column']/x:category[@text='Adjusted R Square']/x:cell/@text"
adjrsq123_str = modelout.xpath(path, namespaces=ns)
print(adjrsq123_str)

resiout = etree.parse(BytesIO(outputs['Table_Residuals Statistics.xml'].content))
print(etree.tostring(resiout, pretty_print=True))

explout = etree.parse(BytesIO(outputs['Table_Variables Entered Removed.xml'].content))
print(etree.tostring(explout, pretty_print=True))

explout = etree.parse(BytesIO(outputs['Table_Variables Entered Removed_1.xml'].content))
print(etree.tostring(explout, pretty_print=True))

explout = etree.parse(BytesIO(outputs['Table_Variables Entered Removed_2.xml'].content))
print(etree.tostring(explout, pretty_print=True))

explout = etree.parse(BytesIO(outputs['Table_Variables Entered Removed_3.xml'].content))
print(etree.tostring(explout, pretty_print=True))


corrout = etree.parse(BytesIO(outputs['Table_Correlations.xml'].content))
print(etree.tostring(corrout, pretty_print=True))

statsout = etree.parse(BytesIO(outputs['Table_Descriptive Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category/@label)"
ylab = statsout.xpath(path, namespaces=ns)
print(ylab)

path = "string(//x:dimension[@text='Variables']/x:category[@varName='" + str(pred1) + "']/@label)"
xlab1 = statsout.xpath(path, namespaces=ns)
print(xlab1)

path = "string(//x:dimension[@text='Variables']/x:category[@varName='" + str(pred2) + "']/@label)"
xlab2 = statsout.xpath(path, namespaces=ns)
print(xlab2)

path = "//x:dimension[@axis='column']/x:category[@text='Mean']/x:cell/@number"
means_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(means_num)

path = "//x:dimension[@axis='column']/x:category[@text='Mean']/x:cell/@text"
means_str = statsout.xpath(path, namespaces=ns)
print(means_str)

path = "//x:dimension[@axis='column']/x:category[@text='Std. Deviation']/x:cell/@number"
sds_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(sds_num)

path = "//x:dimension[@axis='column']/x:category[@text='Std. Deviation']/x:cell/@text"
sds_str = statsout.xpath(path, namespaces=ns)
print(sds_str)

path = "//x:dimension[@axis='column']/x:category[@text='Minimum']/x:cell/@number"
mins_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(mins_num)

path = "//x:dimension[@axis='column']/x:category[@text='Minimum']/x:cell/@text"
mins_str = statsout.xpath(path, namespaces=ns)
print(mins_str)

path = "//x:dimension[@axis='column']/x:category[@text='Maximum']/x:cell/@number"
maxs_num = [str2float(x) for x in statsout.xpath(path, namespaces=ns)]
print(maxs_num)

path = "//x:dimension[@axis='column']/x:category[@text='Maximum']/x:cell/@text"
maxs_str = statsout.xpath(path, namespaces=ns)
print(maxs_str)

# Outputs:
# Chart_000.png : histogram (standardised residual)
# Chart_1_000.png : Normal P-P (standardised residual)
# Chart_2_000.png : scatterplot (standardised residual vs pred1)
# Chart_3_000.png : scatterplot (standardised residual vs pred2)
# Table_ANOVA.htm : Anova table (pred1)
# Table_ANOVA_1.htm : Anova table (pred2)
# Table_ANOVA_2.htm : Anova table (pred1,2)
# Table_ANOVA_3.htm : Anova table (pred1, pred2)
# Table_Coefficients.htm : Regression coefficients (pred1)
# Table_Coefficients_1.htm : Regression coefficients (pred2)
# Table_Coefficients_2.htm : Regression coefficients (pred1,2)
# Table_Coefficients_3.htm : Regression coefficients (pred1, pred2))
# Table_Model Summary.htm : Fit statistics (pred1)
# Table_Model Summary_1.htm : Fit statistics (pred2)
# Table_Model Summary_2.htm : Fit statistics (pred1,2)
# Table_Model Summary_3.htm : Fit statistics (pred1, pred2)
# Table_Notes.htm : Notes for regression command (pred1)
# Table_Notes_1.htm : Notes for regression command (pred2)
# Table_Notes_2.htm : Notes for regression command (pred1,2)
# Table_Notes_3.htm : Notes for regression command (pred1, pred2)
# Table_Residuals Statistics.htm : summary statistics for prediction and residual (pred1, pred2)
# Table_Variables Entered Removed.htm : details of dependent variables aded or removed (pred1)
# Table_Variables Entered Removed_1.htm : details of dependent variables aded or removed (pred2)
# Table_Variables Entered Removed_2.htm : details of dependent variables aded or removed (pred1,2)
# Table_Variables Entered Removed_3.htm : details of dependent variables aded or removed (pred1, pred2)


respnameshort = str(y)
respnamelong = str(y)
if ylab != '':
    respnamelong = ylab + '[' + respnamelong + ']'

respnameshort = '<strong>' + respnameshort + '</strong>'
respnamelong = '<strong>' + respnamelong + '</strong>'

varname1short = str(pred1)
varname1long = str(pred1)
if xlab1 != '':
    varname1long = xlab1 + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(pred2)
varname2long = str(pred2)
if xlab2 != '':
    varname2long = xlab2 + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'


introhtml = '<h2>Multiple Regression practical</h2>'
introhtml += '<p>'
introhtml += 'In this practical we will look at regressing two different predictor variables individually on a response, followed by a model containing both of them. '
introhtml += 'We will also look at a second approach to doing this. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introductory text")

regress1prachtml = '<p>'
regress1prachtml += 'We start by running the first linear regression to look at if there is a significant (linear) effect of ' + varname1short + ' on ' + respnameshort + '. '
regress1prachtml += 'This is done in SPSS as follows:'
regress1prachtml += '</p>'

regress1instrhtml = '<ul class="instructions">'
regress1instrhtml += '<li>Select <strong>Linear</strong> from the <strong>Regression</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
regress1instrhtml += '<li>Copy the ' + respnamelong + ' variable into the <strong>Dependent</strong> box.</li>'
regress1instrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Independent(s)</strong> box.</li>'
regress1instrhtml += '<li>Click on the <strong>Statistics</strong> button.</li>'
regress1instrhtml += '<li>On the screen appears add the tick for <strong>Confidence Interval</strong> to those for <strong>Estimates</strong> and <strong>Model fit</strong>.</li>'
regress1instrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
regress1instrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
regress1instrhtml += '</ul>'

regress1prachtml += regress1instrhtml

regress1prachtml += '<p>'
regress1prachtml += 'SPSS will produce several tabular outputs but here we will focus on only the model summary and coefficients tables that can be seen below:'
regress1prachtml += '</p>'

regress1prachtml += outputs['Table_Model Summary.htm'].content

regress1prachtml += '<p>'
regress1prachtml += 'Here we see some fit statistics for the overall model. '
regress1prachtml += 'The statistic R here takes the value ' + r1_str[0] + ' and is equivalent to the Pearson correlation coefficient for a simple linear regression, that is a regression with only one predictor variable. '
regress1prachtml += 'R squared (' + rsq1_str[0] + ') is simply the value of R squared (R multiplied by itself) and represents the proportion of variance in the response variable, ' + respnameshort  + ' explained by ' + varname1short + '. ' 
regress1prachtml += 'The table also includes an adjusted R square measure which here takes value ' + adjrsq1_str[0] + ' and is a version of R squared that is adjusted to take account of the number of predictors (one in the case of this simple linear regression) that are in the model. '
regress1prachtml += 'We next look at the coefficients table which is shown below:'
regress1prachtml += '</p>'

regress1prachtml += outputs['Table_Coefficients.htm'].content

regress1prachtml += '<p>'
regress1prachtml += 'This table often gives the most interesting information about the regression model. '
regress1prachtml += 'We begin with the coefficients that form the regression equation. '
regress1prachtml += 'The regression intercept (labelled Constant in SPSS) takes the value ' + coefb1_str[0] + ' and is the predicted value of ' + respnameshort + ' when ' + varname1short + ' takes value 0. '
regress1prachtml += 'The regression slope, or unstandardised coefficient, (B in SPSS) takes value ' + coefb1_str[1] + ' and is the amount by which we predict that ' + respnameshort + ' changes for an increase of 1 unit in ' + varname1short + '. '
regress1prachtml += '</p>'

regress1prachtml += '<p>'
regress1prachtml += 'Both coefficients have associated standard errors that can be used to assess their significance. '
regress1prachtml += 'SPSS also reports a standardised coefficient (the Beta) that can be interpreted as a &quot;unit-free&quot; measure of effect size, one that can be used to compare the magnitude of effects of predictors measured in different units. '
regress1prachtml += 'Here Beta takes the value ' + coefbeta1_str[0] + ' which represents the predicted change in the number of standard deviations of ' + respnameshort + ' for an increase of 1 standard deviation in ' + varname1short + '. '
regress1prachtml += '</p>'

regress1prachtml += '<p>'
regress1prachtml += 'To test for the significance of the coefficients we need to form test statistics which are reported under the t column and these are simply B / Std.Error. '
regress1prachtml += 'For the slope on ' + varname1short + ' the t statistic is ' + coeft1_str[1] + ' and this value can be compared with a t distribution to test the null hypothesis that the slope is 0. '
regress1prachtml += 'We can see the resulting p value for the test under the Sig. column. '
if coefsig1_num[1] > 0.05:
    regress1prachtml += 'The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope coefficient on ' + varname1short + '  is zero. '
else:
    if coefsig1_num[1] < 0.001:
        regress1prachtml += 'The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress1prachtml += 'The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' which is less than 0.05. '
    regress1prachtml += 'We therefore have significant evidence to reject the null hypothesis that the slope coefficient on ' + varname1short + ' is zero. '
regress1prachtml += '</p>'

regress1prachtml += '<p>'
regress1prachtml += 'We can also check if the intercept is different from zero though this is often of less interest. '
regress1prachtml += 'For the intercept here the t statistic is ' + coeft1_str[0] + ' and ' + 'the p value (quoted under Sig.) is ' + coefsig1_str[0]
if coefsig1_num[0] > 0.05:
    regress1prachtml += ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the intercept is zero. '
else:
    if coefsig1_num[0] < 0.001:
        regress1prachtml += ' (reported as p &lt; .001)'
    regress1prachtml += ' which is less than 0.05. '
    regress1prachtml += 'We therefore have significant evidence to reject the null hypothesis that the intercept is zero. '
regress1prachtml += '</p>'

regress1prachtml += '<p>'
regress1prachtml += 'The final two columns give confidence intervals for the coefficients and so a 95 percent confident interval for the intercept takes values between ' + coeflb1_str[0] + ' and ' + coefub1_str[0] + '. '
regress1prachtml += '</p>'

regress1prachtml += '<p>'
regress1prachtml += 'Similarly a 95 percent confidence interval for the slope for  ' + varname1short + ' takes value between ' + str(coeflb1_str[1]) + ' and ' + str(coefub1_str[1]) + '. '
if coefsig1_num[1] > 0.05:
    regress1prachtml += 'Here we see the confidence interval contains 0 which corresponds to the fact we could not reject the null hypothesis that the slope was 0. '
else:
    regress1prachtml += 'Here we see the confidence interval does not contain 0 which corresponds to the fact we could reject the null hypothesis that the slope was 0. '
regress1prachtml += '</p>'
outputs['regression1.html'] = HTMLOutput(regress1prachtml, "Regression 1 text")

regress1questhtml = '<p>'
regress1questhtml += 'Use SPSS to fit a regression of ' + respnamelong + ' on the single predictor ' + varname1long + ' and answer the following questions: '
regress1questhtml += '</p>'
regress1questhtml += '<ul>'
regress1questhtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?</li>'
regress1questhtml += '<li>Question: Looking at the Coefficients table what is the predicted value of ' + respnameshort + ' when ' + varname1short + ' = 0?</li>'
regress1questhtml += '<li>Question: Looking at the Coefficients table how much is a one unit increase of ' + varname1short + ' predicted to change ' + respnameshort + '?</li>'
regress1questhtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regress1questhtml += '<li>Question: Is the slope significant?</li>'
regress1questhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress1questhtml += '<li>Question: What is the 95% CI for the slope?</li>'
regress1questhtml += '</ul>'

outputs['regression1questions.html'] = HTMLOutput(regress1questhtml, "Regression 1 text questions")

regress1solhtml = '<p>'
regress1solhtml += 'The SPSS instructions for the first simple regression are as follows: '
regress1solhtml += '</p>'
regress1solhtml += regress1instrhtml
regress1solhtml += '<p>'
regress1solhtml += 'Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?'
regress1solhtml += '</p>'
regress1solhtml += '<p>'
regress1solhtml += 'Solution: The output from SPSS is as follows:'
regress1solhtml += '</p>'
regress1solhtml += outputs['Table_Model Summary.htm'].content
regress1solhtml += '<p>The R squared provides the proportion of the variance explained by the regression and has the value ' + rsq1_str[0] + '.</p>'
regress1solhtml += '<ul>'
regress1solhtml += '<li>Question: Looking at the Coefficients table what is the predicted value of ' + respnameshort + ' when ' + varname1short + ' = 0?</li>'
regress1solhtml += '<li>Question: Looking at the Coefficients table how much is a one unit increase of ' + varname1short + ' predicted to change ' + respnameshort + '?</li>'
regress1solhtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regress1solhtml += '<li>Question: Is the slope significant?</li>'
regress1solhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress1solhtml += '<li>Question: What is the 95% CI for the slope?</li>'
regress1solhtml += '</ul>'
regress1solhtml += '<p>'
regress1solhtml += 'Solutions: The output from SPSS is as follows:'
regress1solhtml += '</p>'
regress1solhtml += outputs['Table_Coefficients.htm'].content
regress1solhtml += '<p>'
regress1solhtml += 'The answers are as follows:'
regress1solhtml += '</p>'
regress1solhtml += '<ul>'
regress1solhtml += '<li>The predicted value of ' + respnameshort + ' when ' + varname1short + ' = 0 is ' + coefb1_str[0] + '.</li>'
regress1solhtml += '<li>A one unit increase of ' + varname1short + ' is predicted to change ' + respnameshort + ' by ' + coefb1_str[1] + '.</li>'
regress1solhtml += '<li>The standardised slope coefficient takes value ' + coefbeta1_str[0] + ' which represents the predicted change in ' + respnameshort + ' for an increase of 1 standard deviation in ' + varname1short + '.</li>'
if coefsig1_num[1] > 0.05:
    regress1solhtml += '<li>The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope is zero.</li>'
else:
    regress1solhtml += '<li>'
    if coefsig1_num[1] < 0.001:
        regress1solhtml += 'The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress1solhtml += 'The p value (quoted under Sig.) is ' + coefsig1_str[1] + ' which is less than 0.05. '
    regress1solhtml += 'We therefore have significant evidence to reject the null hypothesis that the slope is zero. '
    regress1solhtml += '</li>'
regress1solhtml += '<li>The intercept confidence interval is ' + coeflb1_str[0] + ' to ' + coefub1_str[0] + '.</li>'
regress1solhtml += '<li>The ' + varname1short + ' slope confidence interval is ' + coeflb1_str[1] + ' to ' + coefub1_str[1] + '.</li>'
regress1solhtml += '</ul>'

outputs['regression1answers.html'] = HTMLOutput(regress1solhtml, "Regression 1 text answers")

regress2prachtml = '<p>'
regress2prachtml += 'We will next run the second linear regression to look at if there is a significant (linear) effect of ' + varname2short + ' on ' + respnameshort + '. '
regress2prachtml += 'This is done in SPSS as follows:'
regress2prachtml += '</p>'

regress2instrhtml = '<ul class="instructions">'
regress2instrhtml += '<li>Select <strong>Linear</strong> from the <strong>Regression</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
regress2instrhtml += '<li>Remove the ' + varname1long + ' variable from the <strong>Independent(s)</strong> box.</li>'
regress2instrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Independent(s)</strong> box.</li>'
regress2instrhtml += '<li>The other options will be remembered from last time.</li>'
regress2instrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
regress2instrhtml += '</ul>'

regress2prachtml += regress2instrhtml

regress2prachtml += '<p>'
regress2prachtml += 'The model and coefficients tables for this second model can be seen below:'
regress2prachtml += '</p>'

regress2prachtml += outputs['Table_Model Summary_1.htm'].content

regress2prachtml += '<p>'
regress2prachtml += 'This time we see some fit statistics for the regression with ' + varname2short + '. '
regress2prachtml += 'The statistic R here takes the value ' + str(r2_str[0]) + '. '
regress2prachtml += 'R squared (' + rsq2_str[0] + ') represents the proportion of variance in the response variable, ' + respnameshort  + ' explained by ' + varname2short + '. ' 
regress2prachtml += 'This time the adjusted R square measure takes value ' + adjrsq2_str[0] + '. '
regress2prachtml += 'We next look at the coefficients table which is shown below:'
regress2prachtml += '</p>'

regress2prachtml += outputs['Table_Coefficients_1.htm'].content

regress2prachtml += '<p>'
regress2prachtml += 'This time the coefficients that form the regression equation are as follows: '
regress2prachtml += 'The regression intercept takes value ' + coefb2_str[0] + ' while the regression slope takes value ' + coefb2_str[1] + ' and is the amount by which we predict that ' + respnameshort + ' changes for an increase of 1 in ' + varname2short + '. '
regress2prachtml += '</p>'

regress2prachtml += '<p>'
regress2prachtml += 'This time under the Beta column the standardised slope takes value ' + coefbeta2_str[0] + ' which represents the predicted change in ' + respnameshort + ' in standard deviation units for an increase of 1 standard deviation in ' + varname2short + '. '
regress2prachtml += '</p>'

regress2prachtml += '<p>'
regress2prachtml += 'For the slope cofficient on ' + varname2short + ' the t statistic is ' + coeft2_str[1] + ' and this value can be compared with a t distribution to test the null hypothesis that the slope is 0. '
if coefsig2_num[1] > 0.05:
    regress2prachtml += 'The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope is zero. '
else:
    if coefsig2_num[1] < 0.001:
        regress2prachtml += 'The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress2prachtml += 'The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' which is less than 0.05. '
    regress2prachtml += 'We therefore have significant evidence to reject the null hypothesis that the slope is zero. '
regress2prachtml += '</p>'

regress2prachtml += '<p>'
regress2prachtml += 'For the intercept the t statistic is ' + coeft2_str[0] + ' and ' + 'the p value (quoted under Sig.) is ' + coefsig2_str[0]
if coefsig2_num[0] > 0.05:
    regress2prachtml += ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the intercept is zero. '
else:
    if coefsig2_num[0] < 0.001:
        regress2prachtml += ' (reported as p &lt; .001)'
    regress2prachtml += ' which is less than 0.05. '
    regress2prachtml += 'We therefore have significant evidence to reject the null hypothesis that the intercept is zero. '
regress2prachtml += '</p>'

regress2prachtml += '<p>'
regress2prachtml += 'The final two columns give confidence intervals for the coefficients and so a 95 percent confidence interval for the intercept takes values between ' + coeflb2_str[0] + ' and ' + coefub2_str[0] + '. '
regress2prachtml += '</p>'

regress2prachtml += '<p>'
regress2prachtml += 'Similarly a 95 percent confidence interval for the slope for  ' + varname2short + ' takes values between ' + coeflb2_str[1] + ' and ' + coefub2_str[1] + '. '
if coefsig2_num[1] > 0.05:
    regress2prachtml += 'Here we see the confidence interval contains 0 which corresponds to the fact we could not reject the null hypothesis that the slope was 0. '
else:
    regress2prachtml += 'Here we see the confidence interval does not contain 0 which corresponds to the fact we could reject the null hypothesis that the slope was 0. '
regress2prachtml += '</p>'
outputs['regression2.html'] = HTMLOutput(regress2prachtml, "Regression two text")

regress2questhtml = '<p>'
regress2questhtml += 'Use SPSS to fit a regression of ' + respnamelong + ' on ' + varname2long + ' and answer the following questions: '
regress2questhtml += '</p>'
regress2questhtml += '<ul>'
regress2questhtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?</li>'
regress2questhtml += '<li>Question: Looking at the Coefficients table what is the predicted value of ' + respnameshort + ' when ' + varname2short + ' = 0?</li>'
regress2questhtml += '<li>Question: Looking at the Coefficients table how much is a one unit increase of ' + varname2short + ' predicted to change ' + respnameshort + '?</li>'
regress2questhtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regress2questhtml += '<li>Question: Is the slope significant?</li>'
regress2questhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress2questhtml += '<li>Question: What is the 95% CI for the slope?</li>'
regress2questhtml += '</ul>'

outputs['regression2questions.html'] = HTMLOutput(regress2questhtml, "Regression two text questions")

regress2solhtml = '<p>'
regress2solhtml += 'The SPSS instructions for the second simple regression are as follows: '
regress2solhtml += '</p>'
regress2solhtml += regress2instrhtml
regress2solhtml += '<p>'
regress2solhtml += 'Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?'
regress2solhtml += '</p>'
regress2solhtml += '<p>'
regress2solhtml += 'Solution: The output from SPSS is as follows:'
regress2solhtml += '</p>'
regress2solhtml += outputs['Table_Model Summary_1.htm'].content
regress2solhtml += '<p>The R squared provides the proportion of the variance explained by the regression and has the value ' + rsq2_str[0] + '.</p>'
regress2solhtml += '<ul>'
regress2solhtml += '<li>Question: Looking at the Coefficients table what is the predicted value of ' + respnameshort + ' when ' + varname2short + ' = 0?</li>'
regress2solhtml += '<li>Question: Looking at the Coefficients table how much is a one unit increase of ' + varname2short + ' predicted to change ' + respnameshort + '?</li>'
regress2solhtml += '<li>Question: What is the standardised slope coefficient and what does it mean?</li>'
regress2solhtml += '<li>Question: Is the slope significant?</li>'
regress2solhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress2solhtml += '<li>Question: What is the 95% CI for the slope?</li>'
regress2solhtml += '</ul>'
regress2solhtml += '<p>'
regress2solhtml += 'Solutions: The output from SPSS is as follows:'
regress2solhtml += '</p>'
regress2solhtml += outputs['Table_Coefficients_1.htm'].content
regress2solhtml += '<p>'
regress2solhtml += 'The answers are as follows:'
regress2solhtml += '</p>'
regress2solhtml += '<ul>'
regress2solhtml += '<li>The predicted value of ' + respnameshort + ' when ' + varname2short + ' = 0 is ' + coefb2_str[0] + '.</li>'
regress2solhtml += '<li>A one unit increase of ' + varname2short + ' is predicted to change ' + respnameshort + ' by ' + coefb2_str[1] + '.</li>'
regress2solhtml += '<li>The standardised slope coefficient takes value ' + coefbeta2_str[0] + ' which represents the predicted change in ' + respnameshort + ' for an increase of 1 standard deviation in ' + varname2short + '.</li>'
if coefsig2_num[1] > 0.05:
    regress2solhtml += '<li>The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope is zero.</li>'
else:
    regress2solhtml += '<li>'
    if coefsig2_num[1] < 0.001:
        regress2solhtml += 'The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress2solhtml += 'The p value (quoted under Sig.) is ' + coefsig2_str[1] + ' which is less than 0.05. '
    regress2solhtml += 'We therefore have significant evidence to reject the null hypothesis that the slope is zero. '
    regress2solhtml += '</li>'
regress2solhtml += '<li>The intercept confidence interval is ' + coeflb2_str[0] + ' to ' + coefub2_str[0] + '.</li>'
regress2solhtml += '<li>The ' + varname2short + ' slope confidence interval is ' + coeflb2_str[1] + ' to ' + coefub2_str[1] + '.</li>'
regress2solhtml += '</ul>'

outputs['regression2answers.html'] = HTMLOutput(regress2solhtml, "Regression two text answers")

regress3prachtml = '<p>'
regress3prachtml += 'We now need to run the third multiple regression to look at if there are significant (linear) effects of both ' + varname1short + ' and ' + varname2short + ' on ' + respnameshort + '. '
regress3prachtml += 'This is done in SPSS as follows:'
regress3prachtml += '</p>'

regress3instrhtml = '<ul class="instructions">'
regress3instrhtml += '<li>Select <strong>Linear</strong> from the <strong>Regression</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
regress3instrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Independent(s)</strong> box to join ' + varname2long + '.</li>'
regress3instrhtml += '<li>The other options will be remembered from last time.</li>'
regress3instrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
regress3instrhtml += '</ul>'

regress3prachtml += regress3instrhtml

regress3prachtml += '<p>'
regress3prachtml += 'The model and coefficients tables can be seen below:'
regress3prachtml += '</p>'

regress3prachtml += outputs['Table_Model Summary_2.htm'].content

regress3prachtml += '<p>'
regress3prachtml += 'This time we see some fit statistics for the multiple regression with both ' + varname1short + ' and ' + varname2short + '. '
regress3prachtml += 'The statistic R here takes the value ' + r3_str[0] + ' . '
regress3prachtml += 'R squared (' + rsq3_str[0] + ') represents the proportion of variance in the response variable, ' + respnameshort  + ' explained by the multiple regression (both of the predictor variables combined). '
regress3prachtml += 'This time the adjusted R square measure takes value ' + adjrsq3_str[0] + ' which we can compare with ' + adjrsq1_str[0] + ' for just ' + varname1short + ' and ' + adjrsq2_str[0] + ' for just ' + varname2short + '. '
regress3prachtml += 'An increase in the adjusted R square compared to either one of these implies that the second added variable has increased the explained variance in ' + respnameshort + '. '
regress3prachtml += 'We next look at the coefficients table which is shown below:'
regress3prachtml += '</p>'

regress3prachtml += outputs['Table_Coefficients_2.htm'].content

regress3prachtml += '<p>'
regress3prachtml += 'This time the coefficients that form the regression equation are as follows: '
regress3prachtml += 'The regression intercept takes value ' + coefb3_str[0] + ' while the regression slope for ' + varname2short + ' takes value ' + coefb3_str[1] + ' and the slope for ' + varname1short + ' takes value ' + coefb3_str[2] + '. '
regress3prachtml += 'These have changed from ' + coefb2_str[1] + ' and ' + coefb1_str[1] + ' respectively when the variables are fitted individually.'
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'This time there are two standardised slopes with the slope for ' + varname2short + ' taking value ' + coefbeta3_str[0] + ' and the slope for ' + varname1short + ' taking value ' + coefbeta3_str[1] + '. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'For ' + varname2short + ' the slope has t statistic ' + coeft3_str[1] + ' and the p value (quoted under Sig.) is ' + coefsig3_str[1]
if coefsig3_num[1] > 0.05:
    regress3prachtml += ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope on ' + varname2short + ' is zero. '
else:
    if coefsig3_num[1] < 0.001:
        regress3prachtml += ' (reported as p &lt; .001) '
    regress3prachtml += ' which is less than 0.05. '
    regress3prachtml += 'We therefore have significant evidence to reject the null hypothesis that the slope on ' + varname2short + ' is zero. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'For ' + varname1short + ' the slope has t statistic ' + coeft3_str[2] + ' and the p value (quoted under Sig.) is ' + coefsig3_str[2]
if coefsig3_num[2] > 0.05:
    regress3prachtml += ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope on ' + varname1short + ' is zero. '
else:
    if coefsig3_num[2] < 0.001:
        regress3prachtml += ' (reported as p &lt; .001) '
    regress3prachtml += ' which is less than 0.05. '
    regress3prachtml += 'We therefore have significant evidence to reject the null hypothesis that the slope on ' + varname1short + ' is zero. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'For the intercept the t statistic is ' + coeft3_str[0] + ' and the p value (quoted under Sig.) is ' + coefsig3_str[0]
if coefsig3_num[0] > 0.05:
    regress3prachtml += ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the intercept is zero. '
else:
    if coefsig3_num[0] < 0.001:
        regress3prachtml += ' (reported as p &lt; .001) '
    regress3prachtml += ' which is less than 0.05. '
    regress3prachtml += 'We therefore have significant evidence to reject the null hypothesis that the intercept is zero. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'The final two columns give confidence intervals for the coefficients and so a 95 percent confidence interval for the intercept takes values between ' + coeflb3_str[0] + ' and ' + coefub3_str[0] + '. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'Similarly a 95 percent confidence interval for the slope for ' + varname2short + ' takes values between ' + coeflb3_str[1] + ' and ' + coefub3_str[1] + '. '
if coefsig3_num[1] > 0.05:
    regress3prachtml += 'Here we see the confidence interval contains 0 which corresponds to the fact we could not reject the null hypothesis that the slope was 0. '
else:
    regress3prachtml += 'Here we see the confidence interval does not contain 0 which corresponds to the fact we could reject the null hypothesis that the slope was 0. '
regress3prachtml += '</p>'

regress3prachtml += '<p>'
regress3prachtml += 'Finally a 95 percent confidence interval for the slope for  ' + varname1short + ' takes values between ' + coeflb3_str[2] + ' and ' + coefub3_str[2] + '. '
if coefsig3_num[2] > 0.05:
    regress3prachtml += 'Here we see the confidence interval contains 0 which corresponds to the fact we could not reject the null hypothesis that the slope was 0. '
else:
    regress3prachtml += 'Here we see the confidence interval does not contain 0 which corresponds to the fact we could reject the null hypothesis that the slope was 0. '
regress3prachtml += '</p>'
outputs['regression3.html'] = HTMLOutput(regress3prachtml, "Regression three text")

regress3questhtml = '<p>'
regress3questhtml += 'Use SPSS to fit a regression of ' + respnamelong + ' on both ' + varname1long + ' and ' + varname2long  + ' and answer the following questions:'
regress3questhtml += '</p>'
regress3questhtml += '<ul>'
regress3questhtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?</li>'
regress3questhtml += '<li>Question: How does the adjusted R squared compare with the two simpler regressions?</li>'
regress3questhtml += '<li>Question: Looking at the Coefficients table what is the value of the intercept?</li>'
regress3questhtml += '<li>Question: Looking at the Coefficients table what is the value of the slope for ' + varname1short + '?</li>'
regress3questhtml += '<li>Question: Looking at the Coefficients table what is the value of the slope for ' + varname2short + '?</li>'
regress3questhtml += '<li>Question: What is the standardised slope coefficient for ' + varname1short + ' and what does it mean?</li>'
regress3questhtml += '<li>Question: What is the standardised slope coefficient for ' + varname2short + ' and what does it mean?</li>'
regress3questhtml += '<li>Question: Is the slope for ' + varname1short + ' significant?</li>'
regress3questhtml += '<li>Question: Is the slope for ' + varname2short + ' significant?</li>'
regress3questhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress3questhtml += '<li>Question: What is the 95% CI for the slope for ' + varname1short + '?</li>'
regress3questhtml += '<li>Question: What is the 95% CI for the slope for ' + varname2short + '?</li>'
regress3questhtml += '</ul>'

outputs['regression3questions.html'] = HTMLOutput(regress3questhtml, "Regression three text questions")

regress3solhtml = '<p>'
regress3solhtml += 'The SPSS instructions for the multiple regression are as follows: '
regress3solhtml += '</p>'
regress3solhtml += regress3instrhtml
regress3solhtml += '<ul>'
regress3solhtml += '<li>Question: Looking at the model summary table what proportion of the variance in the response is explained by the regression?</li>'
regress3solhtml += '<li>Question: How does the adjusted R squared compare with the two simpler regressions?</li>'
regress3solhtml += '</ul>'
regress3solhtml += '<p>'
regress3solhtml += 'Solutions: The output from SPSS is as follows:'
regress3solhtml += '</p>'
regress3solhtml += outputs['Table_Model Summary_2.htm'].content
regress3solhtml += '<p>'
regress3solhtml += 'The answers are as follows:'
regress3solhtml += '</p>'
regress3solhtml += '<ul>'
regress3solhtml += '<li>The R squared provides the proportion of the variance explained by the regression and has the value ' + rsq3_str[0] + '.</li>'
regress3solhtml += '<li>The adjusted R square measure takes value ' + adjrsq3_str[0] + ' which we can compare with ' + adjrsq1_str[0] + ' for just ' + varname1short + ' and ' + adjrsq2_str[0] + ' for just ' + varname2short + '.</li>'
regress3solhtml += '</ul>'

regress3solhtml += '<ul>'
regress3solhtml += '<li>Question: Looking at the Coefficients table what is the value of the intercept?</li>'
regress3solhtml += '<li>Question: Looking at the Coefficients table what is the value of the slope for ' + varname2short + '?</li>'
regress3solhtml += '<li>Question: Looking at the Coefficients table what is the value of the slope for ' + varname1short + '?</li>'
regress3solhtml += '<li>Question: What is the standardised slope coefficient for ' + varname2short + ' and what does it mean?</li>'
regress3solhtml += '<li>Question: What is the standardised slope coefficient for ' + varname1short + ' and what does it mean?</li>'
regress3solhtml += '<li>Question: Is the slope for ' + varname2short + ' significant?</li>'
regress3solhtml += '<li>Question: Is the slope for ' + varname1short + ' significant?</li>'
regress3solhtml += '<li>Question: What is the 95% CI for the intercept?</li>'
regress3solhtml += '<li>Question: What is the 95% CI for the slope for ' + varname2short + '?</li>'
regress3solhtml += '<li>Question: What is the 95% CI for the slope for ' + varname1short + '?</li>'
regress3solhtml += '</ul>'

regress3solhtml += '<p>'
regress3solhtml += 'Solutions: The output from SPSS is as follows:'
regress3solhtml += '</p>'

regress3solhtml += outputs['Table_Coefficients_2.htm'].content

regress3solhtml += '<p>'
regress3solhtml += 'The answers are as follows:'
regress3solhtml += '</p>'
regress3solhtml += '<ul>'
regress3solhtml += '<li>The regression intercept takes value ' + coefb3_str[0] + '.</li>'
regress3solhtml += '<li>The regression slope for ' + varname2short + ' takes value ' + coefb3_str[1] + '.</li>'
regress3solhtml += '<li>The regression slope for ' + varname1short + ' takes value ' + coefb3_str[2] + '.</li>'
regress3solhtml += '<li>The standardised slope coefficient for ' + varname2short + ' takes value ' + coefbeta3_str[0] + ' which represents the predicted change in ' + respnameshort + ' for an increase of 1 standard deviation in ' + varname2short + '.</li>'
regress3solhtml += '<li>The standardised slope coefficient for ' + varname1short + ' takes value ' + coefbeta3_str[1] + ' which represents the predicted change in ' + respnameshort + ' for an increase of 1 standard deviation in ' + varname1short + '.</li>'
if coefsig3_num[1] > 0.05:
    regress3solhtml += '<li>The p value (quoted under Sig.) is ' + coefsig3_str[1] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope for ' + varname2short + ' is zero.</li>'
else:
    regress3solhtml += '<li>'
    if coefsig3_num[1] < 0.001:
        regress3solhtml += 'The p value (quoted under Sig.) is ' + coefsig3_str[1] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress3solhtml += 'The p value (quoted under Sig.) is ' + coefsig3_str[1] + ' which is less than 0.05. '
    regress3solhtml += 'We therefore have significant evidence to reject the null hypothesis that the slope for ' + varname2short + ' is zero. '
    regress3solhtml += '</li>'
if coefsig3_num[2] > 0.05:
    regress3solhtml += '<li>The p value (quoted under Sig.) is ' + coefsig3_str[2] + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the slope for ' + varname1short + ' is zero.</li>'
else:
    regress3solhtml += '<li>'
    if coefsig3_num[2] < 0.001:
        regress3solhtml += 'The p value (quoted under Sig.) is ' + coefsig3_str[2] + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        regress3solhtml += 'The p value (quoted under Sig.) is ' + coefsig3_str[2] + ' which is less than 0.05. '
    regress3solhtml += 'We therefore have significant evidence to reject the null hypothesis that the slope for ' + varname1short + ' is zero. '
    regress3solhtml += '</li>'
regress3solhtml += '<li>The confidence interval for the intercept is ' + coeflb3_str[0] + ' to ' + coefub3_str[0] + '.</li>'
regress3solhtml += '<li>The confidence interval for the slope for ' + varname2short +  ' is ' + coeflb3_str[1] + ' to ' + coefub3_str[1] + '.</li>'
regress3solhtml += '<li>The confidence interval for the slope for ' + varname1short +  ' is ' +  coeflb3_str[2] + ' to ' + coefub3_str[2] + '.</li>'
regress3solhtml += '</ol>'

outputs['regression3answers.html'] = HTMLOutput(regress3solhtml, "Regression three text answers")

regress4prachtml = '<p>'
regress4prachtml += 'Finally we will show how to run two of the regression models in one go and build up the regression in blocks. '
regress4prachtml += 'This is done in SPSS as follows:'
regress4prachtml += '</p>'

regress4instrhtml = '<ul class="instructions">'
regress4instrhtml += '<li>Select <strong>Linear</strong> from the <strong>Regression</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
regress4instrhtml += '<li>Remove the ' + varname2long + ' variable from the <strong>Independent(s)</strong> box to leave just ' + varname1long + '.</li>'
regress4instrhtml += '<li>Click the <strong>Next</strong> button.</li>'
regress4instrhtml += '<li>Copy the ' + varname2long + ' variable into the now empty <strong>Independent(s)</strong> box.</li>'
regress4instrhtml += '<li>Click on the <strong>Save</strong> button.</li>'
regress4instrhtml += '<li>On the screen appears select the tick for <strong>Standardized</strong> found under <strong>Residuals</strong>.</li>'
regress4instrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
regress4instrhtml += '<li>Click on the <strong>OK</strong> button to run the command.</li>'
regress4instrhtml += '</ul>'

regress4prachtml += regress4instrhtml

regress4prachtml += '<p>'
regress4prachtml += 'The model and coefficients tables can be see below:'
regress4prachtml += '</p>'

regress4prachtml += outputs['Table_Model Summary_3.htm'].content

regress4prachtml += '<p>'
regress4prachtml += 'Here we see the model summaries for the first and third regression models earlier i.e. we fit a model with just ' + varname1short + ' and then a second model where we introduce ' + varname2short + '. '
regress4prachtml += '</p>'

regress4prachtml += outputs['Table_Coefficients_3.htm'].content

regress4prachtml += '<p>'
regress4prachtml += 'Similarly we have the model coefficients for the first and third models from earlier in one combined table. '
regress4prachtml += '</p>'

regress4prachtml += '<p>'
regress4prachtml += 'Having selected standardised residuals we get an additional table, the <strong>Residuals statistics</strong> table. '
regress4prachtml += '</p>'

regress4prachtml += outputs['Table_Residuals Statistics.htm'].content

regress4prachtml += '<p>'
regress4prachtml += 'This table just summarises the predictions and residuals that come out of the final regression and it is perhaps easier to look at these via plots. '
regress4prachtml += '</p>'

outputs['regression4.html'] = HTMLOutput(regress4prachtml, "Regression four text")

regress4questhtml = '<p>'
regress4questhtml += 'Use SPSS to build up the regression models by fitting first a regression of  ' + respnamelong + ' on ' + varname1long + ' and then in block 2 both ' + varname1long + ' and ' + varname2long  + ' and answer the following questions: '
regress4questhtml += 'Make sure you tick the box for Standardized found under Save... -> Residuals.: '
regress4questhtml += '</p>'
regress4questhtml += '<ul>'
regress4questhtml += '<li>Question: What do the three output tables show?</li>'
regress4questhtml += '</ul>'

outputs['regression4questions.html'] = HTMLOutput(regress4questhtml, "Regression four text questions")

regress4solhtml = '<p>'
regress4solhtml += 'The SPSS instructions to build the regression in blocks are as follows: '
regress4solhtml += '</p>'
regress4solhtml += regress4instrhtml
regress4solhtml += '<ul>'
regress4solhtml += '<li>Question: What do the three output tables show?</li>'
regress4solhtml += '</ul>'
regress4solhtml += '<p>'
regress4solhtml += 'Solutions: The output from SPSS is as follows:'
regress4solhtml += '</p>'
regress4solhtml += outputs['Table_Model Summary_3.htm'].content
regress4solhtml += '<p>'
regress4solhtml += 'Here we see the model summaries for the first and third regression models earlier i.e. we fit a model with just ' + varname1short + ' and then a second model where we introduce ' + varname2short + '. '
regress4solhtml += '</p>'
regress4solhtml += outputs['Table_Coefficients_3.htm'].content
regress4solhtml += '<p>'
regress4solhtml += 'Similarly we have the model coefficients for the first and third models from earlier in one combined table. '
regress4solhtml += '</p>'
regress4solhtml += outputs['Table_Residuals Statistics.htm'].content
regress4solhtml += '<p>'
regress4solhtml += 'This table just summarises the predictions and residuals that come out of the final regression and it is perhaps easier to look at these via plots. '
regress4solhtml += '</p>'

outputs['regression4answers.html'] = HTMLOutput(regress4solhtml, "Regression four text answers")

resihistprachtml = '<p>'
resihistprachtml += 'As we requested that standardized residuals were saved this has resulted in an additional variable being stored in the dataset named <strong>ZRE_1</strong> at the end of the existing variables. '
resihistprachtml += 'We can use this variable to create some residuals plot to assess the fit of the model. '
resihistprachtml += 'We will firstly plot a histogram of the residuals to check their normality which can be done in SPSS as follows: '
resihistprachtml += '</p>'

resihistinstrhtml = '<ul class="instructions">'
resihistinstrhtml += '<li>Select <strong>Histogram</strong> from the <strong>Legacy diagnostics</strong> available from the <strong>Graphs</strong> menu.</li>'
resihistinstrhtml += '<li>Copy the <strong>Standardized Residual [ZRE_1]</strong> variable into the <strong>Variable</strong> box.</li>'
resihistinstrhtml += '<li>Click on the <strong>Display normal curve</strong> tick box.</li>'
resihistinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
resihistinstrhtml += '</ul>'

resihistprachtml += resihistinstrhtml

resihistprachtml += '<p>'
resihistprachtml += 'This will produce the graph as shown below:'
resihistprachtml += '</p>'

resihistprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

resihistprachtml += '<p>'
resihistprachtml += 'Here we hope to see the histogram of residuals roughly following the shape of the normal curve that is superimposed over them. '
resihistprachtml += '</p>'
outputs['residualhistogram.html'] = HTMLOutput(resihistprachtml, "Residual histogram text")

resihistquesthtml = '<p>'
resihistquesthtml += 'Use Histogram from the Legacy diagnostics on the residuals from the last model to answer the following question: '
resihistquesthtml += '</p>'
resihistquesthtml += '<ul>'
resihistquesthtml += '<li>Question: Looking at the histogram do the residuals look normally distributed?</li>'
resihistquesthtml += '</ul>'
outputs['residualhistogramquestions.html'] = HTMLOutput(resihistquesthtml, "Residual histogram text questions")

resihistsolhtml = '<p>'
resihistsolhtml += 'The SPSS instructions for the histogram are as follows: '
resihistsolhtml += '</p>'
resihistsolhtml += resihistinstrhtml
resihistsolhtml += '<ul>'
resihistsolhtml += '<li>Question: Looking at the histogram do the residuals look normally distributed?</li>'
resihistsolhtml += '</ul>'
resihistsolhtml += '<p>'
resihistsolhtml += 'Solution: The output from SPSS is as follows:'
resihistsolhtml += '</p>'
resihistsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
resihistsolhtml += '<p>'
resihistsolhtml += 'We hope to see the histogram of residuals roughly following the shape of the normal curve that is superimposed over them.'
resihistsolhtml += '</p>'

outputs['residualhistogramanswers.html'] = HTMLOutput(resihistsolhtml, "Residual histogram text answers")

resiscatter1prachtml = '<p>'
resiscatter1prachtml += 'We can also look at how the distribution of the residuals interacts with the predictor variables in the model to check there is no relationship. '
resiscatter1prachtml += 'We do this via scatterplots which can be produced in SPSS as follows:'
resiscatter1prachtml += '</p>'

resiscatter1instrhtml = '<ul class="instructions">'
resiscatter1instrhtml += '<li>Select <strong>Scatter/Dot</strong> from the <strong>Legacy diagnostics</strong> available from the <strong>Graphs</strong> menu.</li>'
resiscatter1instrhtml += '<li>Select Simple Scatter and click on Define to bring up the Simple Scatterplot window.</li>'
resiscatter1instrhtml += '<li>Copy the <strong>Standardized Residual [ZRE_1]</strong> variable into the <strong>Y Axis</strong> box.</li>'
resiscatter1instrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>X Axis</strong> box.</li>'
resiscatter1instrhtml += '<li>Click on the <strong>OK</strong> button.</li>' 
resiscatter1instrhtml += '</ul>'

resiscatter1prachtml += resiscatter1instrhtml

resiscatter1prachtml += '<p>'
resiscatter1prachtml += 'This will produce the graph as shown below:'
resiscatter1prachtml += '</p>'

resiscatter1prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'

resiscatter1prachtml += '<p>'
resiscatter1prachtml += 'Here we hope not to see any pattern where there was more variability in the residuals for particular values of ' + varname1long + '. '
resiscatter1prachtml += '</p>'

outputs['residualscatter1.html'] = HTMLOutput(resiscatter1prachtml, "Residual scatter one text")

resiscatter1quest = '<p>'
resiscatter1quest += 'Use Scatter from the Legacy diagnostics to look at the relationship between the residuals from the last model and ' + varname1short + '. '
resiscatter1quest += '</p>'
resiscatter1quest += '<ul>'
resiscatter1quest += '<li>Question: Is there any specific pattern in the plot against ' + varname1short + '?</li>'
resiscatter1quest += '</ul>'

outputs['residualscatter1questions.html'] = HTMLOutput(resiscatter1quest, "Residual scatter one text questions")

resiscatter1solhtml = '<p>'
resiscatter1solhtml += 'The SPSS instructions for the first scatter plot are as follows: '
resiscatter1solhtml += '</p>'
resiscatter1solhtml += resiscatter1instrhtml
resiscatter1solhtml += '<ul>'
resiscatter1solhtml += '<li>Question: Is there any specific pattern in the plot against ' + varname1short + '?</li>'
resiscatter1solhtml += '</ul>'
resiscatter1solhtml += '<p>'
resiscatter1solhtml += 'The output from SPSS is as follows:'
resiscatter1solhtml += '</p>'
resiscatter1solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_2_000.png'].content)  + '">'
resiscatter1solhtml += '<p>'
resiscatter1solhtml += 'We hope that the residuals show a random scatter when plotted against the predictor variable and also that their variability is constant across different values of the predictor variable.'
resiscatter1solhtml += '</p>'

outputs['residualscatter1answers.html'] = HTMLOutput(resiscatter1solhtml, "Residual scatter one text answers")

resiscatter2prachtml = '<p>'
resiscatter2prachtml += 'We can repeat this plot for ' + varname2long + ' as follows:'
resiscatter2prachtml += '</p>'

resiscatter2instrhtml = '<ul class="instructions">'
resiscatter2instrhtml += '<li>Select <strong>Scatter/Dot</strong> from the <strong>Legacy diagnostics</strong> available from the <strong>Graphs</strong> menu.</li>'
resiscatter2instrhtml += '<li>Select Simple Scatter and click on Define to bring up the Simple Scatterplot window</li>'
resiscatter2instrhtml += '<li>Remove the ' + varname1long + ' variable from the <strong>X Axis</strong> box.</li>'
resiscatter2instrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>X Axis</strong> box.</li>'
resiscatter2instrhtml += '<li>Click on the <strong>OK</strong> button.</li>' 
resiscatter2instrhtml += '</ul>'

resiscatter2prachtml += resiscatter2instrhtml

resiscatter2prachtml += '<p>'
resiscatter2prachtml += 'This will produce the graph as shown below:'
resiscatter2prachtml += '</p>'

resiscatter2prachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'

resiscatter2prachtml += '<p>'
resiscatter2prachtml += 'Note that again we hope not to see any pattern in the residuals. '
resiscatter2prachtml += '</p>'
outputs['residualscatter2.html'] = HTMLOutput(resiscatter2prachtml, "Residual scatter two text")

resiscatter2questhtml = '<p>'
resiscatter2questhtml += 'Use Scatter from the Legacy diagnostics to look at the relationship between the residuals from the last model and ' + varname2short + '. '
resiscatter2questhtml += '</p>'
resiscatter2questhtml += '<ul>'
resiscatter2questhtml += '<li>Question: Is there any specific pattern in the plot against ' + varname2short + '?</li>'
resiscatter2questhtml += '</ul>'
outputs['residualscatter2questions.html'] = HTMLOutput(resiscatter2questhtml, "Residual scatter two text questions")

resiscatter2solhtml = '<p>'
resiscatter2solhtml += 'The SPSS instructions for the second scatter plot are as follows: '
resiscatter2solhtml += '</p>'
resiscatter2solhtml += resiscatter2instrhtml
resiscatter2solhtml += '<ul>'
resiscatter2solhtml += '<li>Question: Is there any specific pattern in the plot against ' + varname2short + '?</li>'
resiscatter2solhtml += '</ul>'

resiscatter2solhtml += '<p>'
resiscatter2solhtml += 'The output from SPSS is as follows:'
resiscatter2solhtml += '</p>'
resiscatter2solhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_3_000.png'].content)  + '">'
resiscatter2solhtml += '<p>'
resiscatter2solhtml += 'We hope that the residuals show a random scatter when plotted against the predictor variable and also that their variability is constant across different values of the predictor variable.'
resiscatter2solhtml += '</p>'

outputs['residualscatter2answers.html'] = HTMLOutput(resiscatter2solhtml, "Residual scatter two text answers")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['regression1.html'].content
combinedprachtml += outputs['regression2.html'].content
combinedprachtml += outputs['regression3.html'].content
combinedprachtml += outputs['regression4.html'].content
combinedprachtml += outputs['residualhistogram.html'].content
combinedprachtml += outputs['residualscatter1.html'].content
combinedprachtml += outputs['residualscatter2.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['regression1questions.html'].content
combinedquesthtml += outputs['regression2questions.html'].content
combinedquesthtml += outputs['regression3questions.html'].content
combinedquesthtml += outputs['regression4questions.html'].content
combinedquesthtml += outputs['residualhistogramquestions.html'].content
combinedquesthtml += outputs['residualscatter1questions.html'].content
combinedquesthtml += outputs['residualscatter2questions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['regression1answers.html'].content
combinedsolhtml += outputs['regression2answers.html'].content
combinedsolhtml += outputs['regression3answers.html'].content
combinedsolhtml += outputs['regression4answers.html'].content
combinedsolhtml += outputs['residualhistogramanswers.html'].content
combinedsolhtml += outputs['residualscatter1answers.html'].content
combinedsolhtml += outputs['residualscatter2answers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''