# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Analyze->Compare Means->Paired-Samples T Test...'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var1 = DataVector('First variable of pair: ')
var2 = DataVector('Second variable of pair: ')
options = Boolean('Options?', default = False)
if options:
    percentvals = Integer('Confidence Interval Percentile: ', default = 95)
    missingvals = Text('Missing Values', ['Exclude cases analysis by analysis', 'Exclude cases listwise'])
'''

    spssscript = '''
COMPUTE DIFF_${var1}_${var2} = ${var1} - ${var2}.

EXAMINE VARIABLES=DIFF_${var1}_${var2}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /NOTOTAL.

T-TEST PAIRS=${var1} WITH ${var2} (PAIRED)
% if options:
 /CRITERIA=CI(0.${percentvals})
% if missingvals == 'Exclude cases analysis by analysis':
 /MISSING=ANALYSIS
% endif
% if missingvals == 'Exclude cases listwise':
 /MISSING=LISTWISE
% endif
% endif
.
'''


    processoutputs = '''
import EStat
from EStat.Templating import *

import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
varlab = descsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean_str = descsout.xpath(path, namespaces=ns)
print(mean_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses_str = descsout.xpath(path, namespaces=ns)
print(ses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci_str = descsout.xpath(path, namespaces=ns)
print(lci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci_str = descsout.xpath(path, namespaces=ns)
print(uci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms_str = descsout.xpath(path, namespaces=ns)
print(tms_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds_str = descsout.xpath(path, namespaces=ns)
print(meds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis_str = descsout.xpath(path, namespaces=ns)
print(varis_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds_str = descsout.xpath(path, namespaces=ns)
print(sds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins_str = descsout.xpath(path, namespaces=ns)
print(mins_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs_str = descsout.xpath(path, namespaces=ns)
print(maxs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges_str = descsout.xpath(path, namespaces=ns)
print(ranges_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs_str = descsout.xpath(path, namespaces=ns)
print(iqrs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews_str = descsout.xpath(path, namespaces=ns)
print(skews_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses_str = descsout.xpath(path, namespaces=ns)
print(skewses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts_str = descsout.xpath(path, namespaces=ns)
print(kurts_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses_str = descsout.xpath(path, namespaces=ns)
print(kurtses_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats_str = statsout.xpath(path, namespaces=ns)
print(kmstats_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs = int(statsout.xpath(path, namespaces=ns))
print(kmdfs)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs_str = statsout.xpath(path, namespaces=ns)
print(kmsigs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats_str = statsout.xpath(path, namespaces=ns)
print(swstats_str)

if swstats_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs = int(statsout.xpath(path, namespaces=ns))
    print(swdfs)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs_str = statsout.xpath(path, namespaces=ns)
    print(swsigs_str)

corrout = etree.parse(BytesIO(outputs['Table_Paired Samples Correlations.xml'].content))
print(etree.tostring(corrout, pretty_print=True))

statsout = etree.parse(BytesIO(outputs['Table_Paired Samples Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/@label)"
varlab1 = statsout.xpath(path, namespaces=ns)
print(varlab1)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/@label)"
varlab2 = statsout.xpath(path, namespaces=ns)
print(varlab2)

testout = etree.parse(BytesIO(outputs['Table_Paired Samples Test.xml'].content))
print(etree.tostring(testout, pretty_print=True))

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
N = int(corrout.xpath(path, namespaces=ns))
print(N)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation']/x:cell/@number)"
corr_num = str2float(corrout.xpath(path, namespaces=ns))
print(corr_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Correlation']/x:cell/@text)"
corr_str = corrout.xpath(path, namespaces=ns)
print(corr_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
sig_num = str2float(corrout.xpath(path, namespaces=ns))
print(sig_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
sig_str = corrout.xpath(path, namespaces=ns)
print(sig_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@number)"
mean1_num = str2float(statsout.xpath(path, namespaces=ns))
print(mean1_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@text)"
mean1_str = statsout.xpath(path, namespaces=ns)
print(mean1_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@number)"
mean2_num = str2float(statsout.xpath(path, namespaces=ns))
print(mean2_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:cell/@text)"
mean2_str = statsout.xpath(path, namespaces=ns)
print(mean2_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
N1 = int(statsout.xpath(path, namespaces=ns))
print(N1)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
N2 = int(statsout.xpath(path, namespaces=ns))
print(N2)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@number)"
sd1_num = str2float(statsout.xpath(path, namespaces=ns))
print(sd1_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@text)"
sd1_str = statsout.xpath(path, namespaces=ns)
print(sd1_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@number)"
sd2_num = str2float(statsout.xpath(path, namespaces=ns))
print(sd2_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:cell/@text)"
sd2_str = statsout.xpath(path, namespaces=ns)
print(sd2_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@number)"
semean1_num = str2float(statsout.xpath(path, namespaces=ns))
print(semean1_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var1) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@text)"
semean1_str = statsout.xpath(path, namespaces=ns)
print(semean1_str)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@number)"
semean2_num = str2float(statsout.xpath(path, namespaces=ns))
print(semean2_num)

path = "string(//x:group[@text='Pair 1']/x:category[@varName='" + str(var2) + "']/x:dimension[@text='Statistics']/x:category[@text='Std. Error Mean']/x:cell/@text)"
semean2_str = statsout.xpath(path, namespaces=ns)
print(semean2_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Mean']/x:cell/@number)"
mean_num = str2float(testout.xpath(path, namespaces=ns))
print(mean_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Mean']/x:cell/@text)"
mean_str = testout.xpath(path, namespaces=ns)
print(mean_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Std. Deviation']/x:cell/@number)"
sd_num = str2float(testout.xpath(path, namespaces=ns))
print(sd_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Std. Deviation']/x:cell/@text)"
sd_str = testout.xpath(path, namespaces=ns)
print(sd_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Std. Error Mean']/x:cell/@number)"
semean_num = str2float(testout.xpath(path, namespaces=ns))
print(semean_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:category[@text='Std. Error Mean']/x:cell/@text)"
semean_str = testout.xpath(path, namespaces=ns)
print(semean_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:group/x:category[@text='Lower']/x:cell/@number)"
lower_num = str2float(testout.xpath(path, namespaces=ns))
print(lower_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:group/x:category[@text='Lower']/x:cell/@text)"
lower_str = testout.xpath(path, namespaces=ns)
print(lower_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:group/x:category[@text='Upper']/x:cell/@number)"
upper_num = str2float(testout.xpath(path, namespaces=ns))
print(upper_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:group[@text='Paired Differences']/x:group/x:category[@text='Upper']/x:cell/@text)"
upper_str = testout.xpath(path, namespaces=ns)
print(upper_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='t']/x:cell/@number)"
t_num = str2float(testout.xpath(path, namespaces=ns))
print(t_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='t']/x:cell/@text)"
t_str = testout.xpath(path, namespaces=ns)
print(t_str)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
df = int(testout.xpath(path, namespaces=ns))
print(df)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:cell/@number)"
sig2tail_num = str2float(testout.xpath(path, namespaces=ns))
print(sig2tail_num)

path = "string(//x:group[@text='Pair 1']/x:category/x:dimension[@text='Statistics']/x:category[@text='Sig. (2-tailed)']/x:cell/@text)"
sig2tail_str = testout.xpath(path, namespaces=ns)
print(sig2tail_str)

varname1short = str(var1)
varname1long = str(var1)
if varlab1 != '':
    varname1long = varlab1 + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(var2)
varname2long = str(var2)
if varlab2 != '':
    varname2long = varlab2 + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

introhtml = '<h2>Paired t test</h2>'
introhtml += '<p>'
introhtml += 'In this practical we are going to investigate how to perform a paired t-test using SPSS. '
introhtml += 'A paired t-test is used when we have two interval or ratio level variable measured for all observations in a dataset and we want to test if the means of these variables are different. '
introhtml += 'The test assumes that the differences between the two variables are normally distributed. '
introhtml += 'To run a single test in SPSS requires that your dataset has two separate columns containing the two variables to be tested. '
introhtml += 'In this situation we could perform a standard 2 sample t-test by reshaping the two variables into one long variable with an accompanying indicator column that defines which original variable each observation refers to but this would be a less efficient test as it does not take account of the paired nature of the data. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

normtestprachtml = '<p>'
normtestprachtml += 'Before we can perform this test we need to check whether the differences between ' + varname1short + ' and ' + varname2short + ' are normally distributed. '
normtestprachtml += 'First we need to create a difference variable which can be done as follows: '
normtestprachtml += '</p>'

normtestinstrhtml = '<ul class = "instructions">'
normtestinstrhtml += '<li>Select <strong>Compute</strong> from the <strong>Transform</strong> menu.</li>'
normtestinstrhtml += '<li>Type <strong>DIFF_' + varname1short + '_' + varname2short + '</strong> into the <strong>Target Variable</strong> box.</li>'
normtestinstrhtml += '<li>Type <strong>' + varname1short + ' - ' + varname2short + '</strong> into the <strong>Numeric Expression</strong> box.</li>'
normtestinstrhtml += '<li>Click on the <strong>OK</strong> button</li>'
normtestinstrhtml += '</ul>'

normtestinstrhtml += '<p>'
normtestinstrhtml += 'We can now use this new generated variable to perform normality checks. '
normtestinstrhtml += 'Do this as follows: '
normtestinstrhtml += '</p>'

normtestinstrhtml += '<ul class = "instructions">'
normtestinstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
normtestinstrhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
normtestinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
normtestinstrhtml += '<li>Copy the <strong>DIFF_' + varname1short + '_' + varname2short + '</strong> variable into the <strong>Dependent List:</strong> box.</li>'
normtestinstrhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
normtestinstrhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
normtestinstrhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
normtestinstrhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
normtestinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
normtestinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
normtestinstrhtml += '</ul>'

normtestprachtml += normtestinstrhtml

normtestprachtml += '<p>'
normtestprachtml += 'We will first look at a histogram of the variable, <strong>DIFF_' + varname1short + '_' + varname2short + '</strong>. '
normtestprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
normtestprachtml += '</p>'

normtestprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normtestprachtml += '<p>'
normtestprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    normtestprachtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num > 2*skewses_num:
        normtestprachtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestprachtml += 'This distribution appears to be reasonably symmetric. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'We will next look at a statistical test to see if this backs up our visual impressions from the histogram. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a Normal distribution. '
normtestprachtml += '</p>'

normtestprachtml += outputs['Table_Tests of Normality.htm'].content

normtestprachtml += '<p>'
normtestprachtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestprachtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str + '. '
normtestprachtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
if kmsigs_num > 0.05:
    normtestprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        normtestprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    normtestprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'Although the Kolmogorov Smirnov statistic tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestprachtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normtestprachtml += '</p>'

normtestprachtml += '<p>'
normtestprachtml += 'SPSS also supplies QQ plots to assist in looking at normality but for brevity we do not show them here.'
normtestprachtml += '</p>'

outputs['normality.html'] = HTMLOutput(normtestprachtml, "Normality test text")

normtestquesthtml = '<p>'
normtestquesthtml += 'A first step to perform a paired t test in SPSS is to construct a variable that is the difference between the two variables to be compared, in this case ' + varname1long + ' and ' + varname2long + '. '
normtestquesthtml += 'Do this now and then use SPSS to test for normality of this difference variable. '
normtestquesthtml += '</p>' 
normtestquesthtml += '<ul>'
normtestquesthtml += '<li> Question: Do the histogram and normality tests suggest that the difference variable follows a normal distribution?</li>'
normtestquesthtml += '</ul>'

outputs['normalityquestions.html'] = HTMLOutput(normtestquesthtml, "Normality test question text")

normtestsolhtml = '<p>'
normtestsolhtml += 'To construct the difference variable and test it for normality: '
normtestsolhtml += '</p>'
normtestsolhtml += normtestinstrhtml
normtestsolhtml += '<p>'
normtestsolhtml += 'Question: Do the histogram and normality tests suggest that the difference variable follows a normal distribution?'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'Solution: The output from SPSS is as follows:'
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
normtestsolhtml += '<p>'
normtestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    normtestsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num > 2*skewses_num:
        normtestsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestsolhtml += 'This distribution appears to be reasonably symmetric. '
normtestsolhtml += '</p>'

normtestsolhtml += outputs['Table_Tests of Normality.htm'].content

normtestsolhtml += '<p>'
normtestsolhtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestsolhtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str + '. '
normtestsolhtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
if kmsigs_num > 0.05:
    normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov  is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    normtestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Although the Kolmogorov Smirnov statistic tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestsolhtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normtestsolhtml += '</p>'

outputs['normalityanswers.html'] = HTMLOutput(normtestsolhtml, "Normality test answer text")

samplestatsprachtml = '<p>'
samplestatsprachtml += 'We will next move on to the paired t test itself and will test the two variables, ' + varname1short + ' and ' + varname2short + ' for differences. '
samplestatsprachtml += '</p>'

samplestatsprachtml += '<p>'
samplestatsprachtml += 'Below you will see instructions on how to perform the paired t test in SPSS. '
samplestatsprachtml += 'If you follow the instructions you will see the three tabular outputs that are embedded in the explanations below. '
samplestatsprachtml += '</p>'

samplestatsinstrhtml = '<ul class = "instructions">'
samplestatsinstrhtml += '<li>Select <strong>Compare Means</strong> from the <strong>Analyze</strong> menu.</li>'
samplestatsinstrhtml += '<li>Select <strong>Paired-Samples T Test...</strong> from the <strong>Compare Means</strong> sub-menu.</li>'
samplestatsinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
samplestatsinstrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Variable1:</strong> box for Pair 1.</li>'
samplestatsinstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Variable2:</strong> box for Pair 1.</li>'
if options:
    samplestatsinstrhtml += '<li>Click on the <strong>Options</strong> button.</li>'
    samplestatsinstrhtml += '<li>Type <strong>' + str(percentvals) + '</strong> into the <strong>Confidence Interval Percentage</strong> box.</li>'
    samplestatsinstrhtml += '<li>Select <strong>' + str(missingvals) + '</strong> from the <strong>Missing Values</strong> options.</li>'
    samplestatsinstrhtml += '<li>Click on the <strong>Continue</strong> button</li>'
samplestatsinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
samplestatsinstrhtml += '</ul>'

samplestatsprachtml += samplestatsinstrhtml

samplestatsprachtml += '<p>'
samplestatsprachtml += 'The first SPSS output table contains summary statistics for the two variables to be compared and can be seen below:'
samplestatsprachtml += '</p>'

samplestatsprachtml += outputs['Table_Paired Samples Statistics.htm'].content

samplestatsprachtml += '<p>'
samplestatsprachtml += 'The summary statistics table contains 5 columns and 1 row for each of the two variable to be tested. '
samplestatsprachtml += 'After the first column which contains the name of each variable, next we see that the mean of variable ' + varname1short + ' is ' + mean1_str + ' whilst the mean of variable ' + varname2short + ' is ' + mean2_str + '. '
if mean1_num > mean2_num:
    samplestatsprachtml += 'Hence the variable ' + varname1short + ' has the bigger mean and the t test will now establish if this difference is statistically significant. '
if mean2_num > mean1_num:
    samplestatsprachtml += 'Hence the variable ' + varname2short + ' has the bigger mean and the t test will now establish if this difference is statistically significant. '
if mean1_num == mean2_num:
    samplestatsprachtml += 'Very unusually the means are equal. '
samplestatsprachtml += 'We next see the number of valid observations for each variable, i.e. cases with valid values for both ' + varname1short + ' and ' + varname2short + '. '
samplestatsprachtml += 'Here we have ' + str(N1) + ' valid observations for both variables.'
samplestatsprachtml += '</p>'

samplestatsprachtml += '<p>'
samplestatsprachtml += 'In the next column we see the standard deviations for ' + varname1short + ' and ' + varname2short + '. '
samplestatsprachtml += 'In this case the standard deviation of ' + varname1short + ' is ' + sd1_str + ' whilst for ' + varname2short + ' it is ' + sd2_str + '. '
if sd1_num > sd2_num:
    samplestatsprachtml += 'So there is slightly more variability for ' + varname1short + ' than ' + varname2short + '. '
if sd2_num > sd1_num:
    samplestatsprachtml += 'So there is slightly more variability for ' + varname2short + ' than ' + varname1short + '. '
if sd1_num == sd2_num:
    samplestatsprachtml += 'So the variability among ' + varname1short + ' and ' + varname2short + ' is the same. '
samplestatsprachtml += 'In the final column are the standard errors of the means for each group. '
samplestatsprachtml += 'Whilst the standard deviations measure the variability in the data the standard errors of the means measures how confident we are in the estimates of the means. '
samplestatsprachtml += 'As we collect more data the standard error of the mean gets smaller as we get more confident in the mean estimate and in fact the formula for the standard error of the mean = standard deviation / square root of N. '
samplestatsprachtml += 'In this case the standard error of the mean for ' + varname1short + ' is ' + semean1_str + ' whilst for ' + varname2short + ' it is ' + semean2_str + '. '
samplestatsprachtml += '</p>'

outputs['samplestatistics.html'] = HTMLOutput(samplestatsprachtml, "Sample statistics text")

samplestatsquesthtml = '<p>'
samplestatsquesthtml += 'Next perform the paired t test in SPSS to compare differences between ' + varname1long + ' and ' + varname2long + '. '
samplestatsquesthtml += '</p>'
samplestatsquesthtml += '<ul>'
samplestatsquesthtml += '<li>Question: From the Paired sample statistics block which variable has the larger mean?</li>'
samplestatsquesthtml += '<li>Question: Which variable has the largest variability?</li>'
samplestatsquesthtml += '</ul>'

outputs['samplestatisticsquestions.html'] = HTMLOutput(samplestatsquesthtml, "Sample statistics question text")

samplestatssolhtml = '<p>'
samplestatssolhtml += 'Here are the instructions to perform the paired t test in SPSS: '
samplestatssolhtml += '</p>'
samplestatssolhtml += samplestatsinstrhtml
samplestatssolhtml += '<ul>'
samplestatssolhtml += '<li>Question: From the Paired Samples statistics block which variable has the larger mean?</li>'
samplestatssolhtml += '<li>Question: Which variable has the larger variability?</li>'
samplestatssolhtml += '</ul>'

samplestatssolhtml += '<p>'
samplestatssolhtml += 'Solutions: The output from SPSS is as follows:'
samplestatssolhtml += '</p>'
samplestatssolhtml += outputs['Table_Paired Samples Statistics.htm'].content
samplestatssolhtml += '<p>'
samplestatssolhtml += 'The answers are as follows:'
samplestatssolhtml += '</p>'
samplestatssolhtml += '<ul>'
if mean1_num > mean2_num:
    samplestatssolhtml += '<li>The variable ' + varname1short +  ' has the larger mean.</li>'
if mean2_num > mean1_num:
    samplestatssolhtml += '<li>The variable ' + varname2short + ' has the larger mean.</li>'
if mean1_num == mean2_num:
    samplestatssolhtml += '<li>Very unusually the means are equal.</li>'
if sd1_num > sd2_num:
    samplestatssolhtml += '<li>The variable ' + varname1short + ' has the larger variability.</li>'
if sd2_num > sd1_num:
    samplestatssolhtml += '<li>The variable ' + varname2short + ' has the larger variability.</li>'
if sd1_num == sd2_num:
    samplestatssolhtml += '<li>They are the same.</li>'
samplestatssolhtml += '</ul>'
samplestatssolhtml += '</p>'

outputs['samplestatisticsanswers.html'] = HTMLOutput(samplestatssolhtml, "Sample statistics answer text")

corrprachtml = '<p>'
corrprachtml += 'The second SPSS output table contains information on the correlation between the two variables to be compared and can be seen below:'
corrprachtml += '</p>'

corrprachtml += outputs['Table_Paired Samples Correlations.htm'].content

corrprachtml += '<p>'
corrprachtml += 'The correlation between two variables is a single number that describes how related they are to each other. '
corrprachtml += 'It is represented by a correlation coefficient which is a numerical value to describe the correlation. '
corrprachtml += 'Correlations lie between -1 and +1 with a positive value meaning that in general that large values of the first variable are more likely to be observed with large values of the second variable and conversely small values of the first variable are more likely to be observed with small values of the second variable. '
corrprachtml += 'In the case of a negative correlation the opposite is true and large values of the first variable are more likely to be observed with small values of the second variable and conversely small values of the first variable are more likely to be observed with large values of the second variable. '
corrprachtml += 'A correlation of 0 means there is no (linear) relationship between the variables. '
corrprachtml += 'Here SPSS is giving out a form of correlation known as a Pearson correlation and we see that the correlation between ' + varname1short + ' and ' + varname2short + ' is ' + corr_str + '. '
corrprachtml += 'It is helpful to look at the correlation between the two variables here as typically a paired t-test is more useful than a 2-sample t-test when there is a positive correlation between the two variables '
if corr_num > 0:
    corrprachtml += 'as is the case here. '
else:
    corrprachtml += '. '
corrprachtml += 'SPSS also gives out a p value which describes whether the correlation is statistically significantly different from zero. '
if sig_num > 0.05:
    corrprachtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is in fact zero. '
else:
    corrprachtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the correlation is zero. '
corrprachtml += '</p>'
outputs['correlations.html'] = HTMLOutput(corrprachtml, "Text to explain correlations")

corrquesthtml = '<ul>'
corrquesthtml += '<li>Question: Looking at the second SPSS output table what is the correlation between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
corrquesthtml += '</ul>'

outputs['correlationsquestions.html'] = HTMLOutput(corrquesthtml, "Text to explain correlations questions")

corrsolhtml = '<ul>'
corrsolhtml += '<li>Question: Looking at the second SPSS output table what is the correlation between ' + varname1short + ' and ' + varname2short + ' and is it significant?</li>'
corrsolhtml += '</ul>'
corrsolhtml += '<p>'
corrsolhtml += 'Solution: The output from SPSS is as follows:'
corrsolhtml += '</p>'
corrsolhtml += outputs['Table_Paired Samples Correlations.htm'].content
corrsolhtml += '<p>Here we see that the correlation between ' + varname1short + ' and ' + varname2short + ' is ' + corr_str + '. '
corrsolhtml += 'SPSS also gives out a p value which describes whether the correlation is statistically significantly different from zero. '
if sig_num > 0.05:
    corrsolhtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the correlation is in fact zero. '
else:
    corrsolhtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the correlation is zero. '
corrsolhtml += '</p>'
outputs['correlationsanswers.html'] = HTMLOutput(corrsolhtml, "Text to explain correlations answers")

pairedttestprachtml = '<p>'
pairedttestprachtml += 'The third SPSS output table contains details of the t test itself and can be seen below:'
pairedttestprachtml += '</p>'

pairedttestprachtml += outputs['Table_Paired Samples Test.htm'].content

pairedttestprachtml += '<p>'
pairedttestprachtml += 'The above table describes the paired t test. '

pairedttestprachtml += 'If we now look at the row of numbers we will start with the column headed Mean (underneath paired differences). '
pairedttestprachtml += 'Here we see the value ' + mean_str +'. '
pairedttestprachtml += 'If you look back at the summary statistics table this value is calculated by subtracting one mean from the other. '
pairedttestprachtml += 'Next to the mean is the standard deviation (of the differences) which has value ' + sd_str + '. '
pairedttestprachtml += 'If we have two positively correlated variables then this standard deviation will typically be smaller than the standard deviations of the two variables'
if sd_num < sd1_num and sd_num < sd2_num:
    pairedttestprachtml += ' which it is. '
else:
    pairedttestprachtml += '. '
pairedttestprachtml += 'Next up is the standard error of the mean (of the differences). '
pairedttestprachtml += 'This here has the value ' + semean_str + ' and is simply the standard deviation divided by the square root of the sample size. '
pairedttestprachtml += 'Moving forwards two columns, the column entitled t is the statistic used in the t test and t is a standard statistical distribution. '
pairedttestprachtml += 'The t statistic is calculated by dividing the mean difference by its standard error so ' + mean_str + ' / ' + semean_str + ' = ' + t_str + '. '
pairedttestprachtml += 'Next to t is a column labelled df which stands for degrees of freedom and is a parameter used to choose the correct t distribution for the statistic. '
pairedttestprachtml += 'Here the degrees of freedom equal the number of observations - 1 (' + str(df) + ') as we have used 1 degrees of freedom in estimating the mean difference. '
pairedttestprachtml += '</p>'

pairedttestprachtml += '<p>'
pairedttestprachtml += 'The column labelled &quot;Sig (2-tailed)&quot; contains a test of the null hypothesis that the means of the two variables (' + varname1short + ' and ' + varname2short + ') are the same. '
pairedttestprachtml += 'By default, the two-tailed test reported uses a non-directional alternative hypothesis. '
pairedttestprachtml += 'It gives the probability that the data in the sample came from a population in which the variable means are truly equal, when either a positive or a negative difference between sample means is evidence against that null hypothesis. '
pairedttestprachtml += 'To conduct a one-tailed test, in which the alternative hypothesis specifies a particular direction to the difference, we would simply halve the p-value provided by SPSS.'
pairedttestprachtml += '</p>'

pairedttestprachtml += '<p>'
pairedttestprachtml += 'We can reject the null hypothesis if there is sufficient evidence that the mean of ' + varname1short + ' is either higher or lower than the mean of ' + varname2short + '. '
pairedttestprachtml += 'SPSS looks up the t statistic in the appropriate table for the degrees of freedom and in this case the corresponding p value is ' + sig2tail_str + '. '
if sig2tail_num > 0.05:
    pairedttestprachtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two variables have the same means. '
    pairedttestprachtml += 'Finally we can see the 95% confidence interval for the difference which runs from ' + lower_str + ' to ' + upper_str + '. '
    pairedttestprachtml += 'Here we see it contains the value 0 backing up our failure to reject the null hypothesis. '
else:
    pairedttestprachtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two groups have the same means. '
    pairedttestprachtml += 'Finally we can see the 95% confidence interval for the difference which runs from ' + lower_str + ' to ' + upper_str + '. '
    pairedttestprachtml += 'Here we see it does not contain the value 0 backing up our rejection of the null hypothesis. '
pairedttestprachtml += '</p>'

outputs['pairedsampletest.html'] = HTMLOutput(pairedttestprachtml, "Paired sample test text")


pairedttestquesthtml = '<ul>'
pairedttestquesthtml += '<li>Question: Looking at the third SPSS table is there a significant difference between ' + varname1short + ' and ' + varname2short + '?</li>'
pairedttestquesthtml += '</ul>'


outputs['pairedsampletestquestions.html'] = HTMLOutput(pairedttestquesthtml, "Paired sample test question text")

pairedttestsolhtml = '<ul>'
pairedttestsolhtml += '<li>Question: Looking at the third SPSS table is there a significant difference between ' + varname1short + ' and ' + varname2short + '?</li>'
pairedttestsolhtml += '</ul>'
pairedttestsolhtml += '<p>'
pairedttestsolhtml += 'Solution: The output from SPSS is as follows:'
pairedttestsolhtml += '</p>'
pairedttestsolhtml += outputs['Table_Paired Samples Test.htm'].content
pairedttestsolhtml += '<p>'
pairedttestsolhtml += 'Looking up the t statistic in the appropriate table gives the associated p value, in this case ' + sig2tail_str + '. '
if sig2tail_num > 0.05:
    pairedttestsolhtml += 'Here we see that the p value is greater than 0.05 and therefore we cannot reject the null hypothesis that the two variables have the same means. '
    pairedttestsolhtml += 'Finally we can see the 95% confidence interval for the difference which runs from ' + lower_str + ' to ' + upper_str + '. '
    pairedttestsolhtml += 'Here we see it contains the value 0 backing up our failure to reject the null hypothesis. '
else:
    pairedttestsolhtml += 'Here we see that the p value is less than 0.05 and therefore we can reject the null hypothesis that the two groups have the same means. </p>'
outputs['pairedsampletestanswers.html'] = HTMLOutput(pairedttestsolhtml, "Paired sample test answer text")

conchtml = '<p>'
conchtml += 'In conclusion, we could report this to a reader as follows: '
conchtml += 'Mean values were compared for 2 variables with sample size ' + str(N1) + '. '
conchtml += 'The mean was higher for variable '
if mean1_num > mean2_num:
    conchtml += varname1short + ' (M=' + mean1_str + ', SD=' + sd1_str + ') than for variable ' + varname2short + ' (M=' + mean2_str + ', SD=' + sd2_str + '). '
if mean2_num > mean1_num:
    conchtml += varname2short + ' (M=' + mean2_str + ', SD=' + sd2_str + ') than for variable ' + varname1short + ' (M=' + mean1_str + ', SD=' + sd1_str + '). '
if sig2tail_num > 0.05:
    conchtml += 'The difference in means (difference = ' + mean_str + ') was not statistically significant, t(' + str(df) + ') = ' + t_str + ', p=' + sig2tail_str + '. '
else:
    conchtml += 'The difference in means (difference = ' + mean_str + ') was statistically significant, t(' + str(df) + ') = ' + t_str + ', '
    if sig2tail_num == 0:
        conchtml += 'p&lt;.001. '
    else:
        conchtml += 'p=' + sig2tail_str + '. '
conchtml += '</p>'

outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")

concquesthtml = '<ul>'
concquesthtml += '<li>Question: How would you write up these findings?</li>'
concquesthtml += '</ul>'
outputs['conclusionquestions.html'] = HTMLOutput(concquesthtml, "Conclusion question text")

concsolhtml = concquesthtml
concsolhtml += '<p>Solution: </p>'
concsolhtml += conchtml
outputs['conclusionsolutions.html'] = HTMLOutput(concsolhtml, "Conclusion solutions text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['normality.html'].content
combinedprachtml += outputs['samplestatistics.html'].content
combinedprachtml += outputs['correlations.html'].content
combinedprachtml += outputs['pairedsampletest.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['normalityquestions.html'].content
combinedquesthtml += outputs['samplestatisticsquestions.html'].content
combinedquesthtml += outputs['correlationsquestions.html'].content
combinedquesthtml += outputs['pairedsampletestquestions.html'].content
combinedquesthtml += outputs['conclusionquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['normalityanswers.html'].content
combinedsolhtml += outputs['samplestatisticsanswers.html'].content
combinedsolhtml += outputs['correlationsanswers.html'].content
combinedsolhtml += outputs['pairedsampletestanswers.html'].content
combinedsolhtml += outputs['conclusionsolutions.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''

    spssgui = r'''
\begin{aligned}
&\text{Select }\textbf{Compare Means}\text{ from the }\textbf{Analyze}\text{ menu.} \\ 
&\text{Select }\textbf{Paired-Samples T Test...}\text{ from the }\textbf{Compare Means}\text{ sub-menu.} \\ 
&\text{Click on the } \textbf{Reset}\text{ button } \\ 
&\text{Copy the } \textbf{${datafile.variables[var1]['varlabel'] + ' ['+var1+']'}}\text{ variable into the } \textbf{Variable1:}\text{ box for Pair 1. } \\ 
&\text{Copy the } \textbf{${datafile.variables[var2]['varlabel'] + ' ['+var2+']'}}\text{ variable into the } \textbf{Variable2:}\text{ box for Pair 1. } \\ 
% if options:
&\text{Click on the } \textbf{Options}\text{ button } \\
&\text{Type } \textbf{${percentvals}}\text{ into the } \textbf{Confidence Interval Percentage}\text{ box. } \\ 
&\text{Select } \textbf{${missingvals}}\text{ from the } \textbf{Missing Values}\text{ options. } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
% endif
&\text{Click on the } \textbf{OK}\text{ button } \\  
\end{aligned}
'''
