# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression2(Template):
    'Performs non-parametric tests in SPSS (equivalent to Analyze->Nonparametric Tests->Legacy Dialogs->2 Related Samples... in SPSS 23)'

    __version__ = '1.0.0'

    tags = [ 'SPSS']
    engines = ['SPSS_script']

    inputs = '''
var1 = DataVector('First variable: ')
var2 = DataVector('Second variable: ')
'''

    spssscript = '''
COMPUTE DIFF_${var1}_${var2} = ${var1} - ${var2}.

EXAMINE VARIABLES=DIFF_${var1}_${var2}
  /PLOT HISTOGRAM NPPLOT
  /COMPARE GROUPS
  /NOTOTAL.

DESCRIPTIVES VARIABLES=${var1} ${var2}.

NPAR TESTS
 /WILCOXON=${var1} WITH ${var2} (PAIRED)
 /METHOD=EXACT.
'''

    processoutputs = '''
import EStat
from EStat.Templating import *

import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

descsout = etree.parse(BytesIO(outputs['Table_Descriptives.xml'].content))
print(etree.tostring(descsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/@label)"
varlab = descsout.xpath(path, namespaces=ns)
print(varlab)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mean_num = str2float(descsout.xpath(path, namespaces=ns))
print(mean_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mean_str = descsout.xpath(path, namespaces=ns)
print(mean_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
ses_num = str2float(descsout.xpath(path, namespaces=ns))
print(ses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Mean']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
ses_str = descsout.xpath(path, namespaces=ns)
print(ses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
lci_num = str2float(descsout.xpath(path, namespaces=ns))
print(lci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Lower Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
lci_str = descsout.xpath(path, namespaces=ns)
print(lci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
uci_num = str2float(descsout.xpath(path, namespaces=ns))
print(uci_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:group[@text='95% Confidence Interval for Mean']/x:category[@text='Upper Bound']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
uci_str = descsout.xpath(path, namespaces=ns)
print(uci_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
tms_num = str2float(descsout.xpath(path, namespaces=ns))
print(tms_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='5% Trimmed Mean']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
tms_str = descsout.xpath(path, namespaces=ns)
print(tms_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
meds_num = str2float(descsout.xpath(path, namespaces=ns))
print(meds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Median']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
meds_str = descsout.xpath(path, namespaces=ns)
print(meds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
varis_num = str2float(descsout.xpath(path, namespaces=ns))
print(varis_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Variance']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
varis_str = descsout.xpath(path, namespaces=ns)
print(varis_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
sds_num = str2float(descsout.xpath(path, namespaces=ns))
print(sds_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Std. Deviation']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
sds_str = descsout.xpath(path, namespaces=ns)
print(sds_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
mins_num = str2float(descsout.xpath(path, namespaces=ns))
print(mins_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Minimum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
mins_str = descsout.xpath(path, namespaces=ns)
print(mins_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
maxs_num = str2float(descsout.xpath(path, namespaces=ns))
print(maxs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Maximum']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
maxs_str = descsout.xpath(path, namespaces=ns)
print(maxs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
ranges_num = str2float(descsout.xpath(path, namespaces=ns))
print(ranges_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
ranges_str = descsout.xpath(path, namespaces=ns)
print(ranges_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
iqrs_num = str2float(descsout.xpath(path, namespaces=ns))
print(iqrs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Interquartile Range']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
iqrs_str = descsout.xpath(path, namespaces=ns)
print(iqrs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
skews_num = str2float(descsout.xpath(path, namespaces=ns))
print(skews_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
skews_str = descsout.xpath(path, namespaces=ns)
print(skews_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
skewses_num = str2float(descsout.xpath(path, namespaces=ns))
print(skewses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Skewness']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
skewses_str = descsout.xpath(path, namespaces=ns)
print(skewses_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@number)"
kurts_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurts_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Statistic']/x:cell/@text)"
kurts_str = descsout.xpath(path, namespaces=ns)
print(kurts_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@number)"
kurtses_num = str2float(descsout.xpath(path, namespaces=ns))
print(kurtses_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Statistics']/x:category[@text='Kurtosis']/x:dimension[@text='Stat Type']/x:category[@text='Std. Error']/x:cell/@text)"
kurtses_str = descsout.xpath(path, namespaces=ns)
print(kurtses_str)

statsout = etree.parse(BytesIO(outputs['Table_Tests of Normality.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
kmstats_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmstats_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
kmstats_str = statsout.xpath(path, namespaces=ns)
print(kmstats_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@number)"
kmdfs = int(statsout.xpath(path, namespaces=ns))
print(kmdfs)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
kmsigs_num = str2float(statsout.xpath(path, namespaces=ns))
print(kmsigs_num)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Kolmogorov-Smirnov']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
kmsigs_str = statsout.xpath(path, namespaces=ns)
print(kmsigs_str)

path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@text)"
swstats_str = statsout.xpath(path, namespaces=ns)
print(swstats_str)

if swstats_str != '': # For non-weighted data Shapiro-Wilk is only calculated for a sample size less than 5000 (see https://www.ibm.com/support/knowledgecenter/en/SSLVMB_24.0.0/spss/base/syn_examine_plot.html)
    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Statistic']/x:cell/@number)"
    swstats_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swstats_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='df']/x:cell/@text)"
    swdfs = int(statsout.xpath(path, namespaces=ns))
    print(swdfs)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@number)"
    swsigs_num = str2float(statsout.xpath(path, namespaces=ns))
    print(swsigs_num)

    path = "string(//x:dimension[@text='Dependent Variables']/x:category/x:dimension[@text='Test']/x:category[@text='Shapiro-Wilk']/x:dimension[@text='Statistics']/x:category[@text='Sig.']/x:cell/@text)"
    swsigs_str = statsout.xpath(path, namespaces=ns)
    print(swsigs_str)

statsout = etree.parse(BytesIO(outputs['Table_Descriptive Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@text='Variables']/x:category[@varName='" + str(var1) + "']/@label)"
varlab1 = statsout.xpath(path, namespaces=ns)
print(varlab1)

path = "string(//x:dimension[@text='Variables']/x:category[@varName='" + str(var2) + "']/@label)"
varlab2 = statsout.xpath(path, namespaces=ns)
print(varlab2)

ranksout = etree.parse(BytesIO(outputs['Table_Wilcoxon Ranks.xml'].content))
print(etree.tostring(ranksout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Negative Ranks']/x:dimension[@axis='column']/x:category[@text='N']/x:cell/@number)"
Nneg = int(ranksout.xpath(path, namespaces=ns))
print(Nneg)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Negative Ranks']/x:dimension[@axis='column']/x:category[@text='Mean Rank']/x:cell/@number)"
meanneg_num = str2float(ranksout.xpath(path, namespaces=ns))
print(meanneg_num)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Negative Ranks']/x:dimension[@axis='column']/x:category[@text='Mean Rank']/x:cell/@text)"
meanneg_str = ranksout.xpath(path, namespaces=ns)
print(meanneg_str)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Negative Ranks']/x:dimension[@axis='column']/x:category[@text='Sum of Ranks']/x:cell/@number)"
sumneg_num = str2float(ranksout.xpath(path, namespaces=ns))
print(sumneg_num)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Negative Ranks']/x:dimension[@axis='column']/x:category[@text='Sum of Ranks']/x:cell/@text)"
sumneg_str = ranksout.xpath(path, namespaces=ns)
print(sumneg_str)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Positive Ranks']/x:dimension[@axis='column']/x:category[@text='N']/x:cell/@number)"
Npos = int(ranksout.xpath(path, namespaces=ns))
print(Npos)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Positive Ranks']/x:dimension[@axis='column']/x:category[@text='Mean Rank']/x:cell/@number)"
meanpos_num = str2float(ranksout.xpath(path, namespaces=ns))
print(meanpos_num)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Positive Ranks']/x:dimension[@axis='column']/x:category[@text='Mean Rank']/x:cell/@text)"
meanpos_str = ranksout.xpath(path, namespaces=ns)
print(meanpos_str)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Positive Ranks']/x:dimension[@axis='column']/x:category[@text='Sum of Ranks']/x:cell/@number)"
sumpos_num = str2float(ranksout.xpath(path, namespaces=ns))
print(sumpos_num)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Positive Ranks']/x:dimension[@axis='column']/x:category[@text='Sum of Ranks']/x:cell/@text)"
sumpos_str = ranksout.xpath(path, namespaces=ns)
print(sumpos_str)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Ties']/x:dimension[@axis='column']/x:category[@text='N']/x:cell/@number)"
Nties = int(ranksout.xpath(path, namespaces=ns))
print(Nties)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='column']/x:category[@text='N']/x:cell/@number)"
Ntotal = int(ranksout.xpath(path, namespaces=ns))
print(Ntotal)

statsout = etree.parse(BytesIO(outputs['Table_Wilcoxon Test Statistics.xml'].content))
print(etree.tostring(statsout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category[@text='Z']/x:dimension[@axis='column']/x:category/x:cell/@number)"
Z_num = str2float(statsout.xpath(path, namespaces=ns))
print(Z_num)

path = "string(//x:dimension[@axis='row']/x:category[@text='Z']/x:dimension[@axis='column']/x:category/x:cell/@text)"
Z_str = statsout.xpath(path, namespaces=ns)
print(Z_str)

path = "string(//x:dimension[@axis='row']/x:category[@text='Asymp. Sig. (2-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@number)"
asympsig2_num = str2float(statsout.xpath(path, namespaces=ns))
print(asympsig2_num)

path = "string(//x:dimension[@axis='row']/x:category[@text='Asymp. Sig. (2-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@text)"
asympsig2_str = statsout.xpath(path, namespaces=ns)
print(asympsig2_str)

path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (2-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@text)"
exactsig2_str = statsout.xpath(path, namespaces=ns)
print(exactsig2_str)
if exactsig2_str != '':
    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (2-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@number)"
    exactsig2_num = str2float(statsout.xpath(path, namespaces=ns))
    print(exactsig2_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (1-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@number)"
    exactsig1_num = str2float(statsout.xpath(path, namespaces=ns))
    print(exactsig1_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Exact Sig. (1-tailed)']/x:dimension[@axis='column']/x:category/x:cell/@text)"
    exactsig1_str = statsout.xpath(path, namespaces=ns)
    print(exactsig1_str)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Point Probability']/x:dimension[@axis='column']/x:category/x:cell/@number)"
    pointprob_num = str2float(statsout.xpath(path, namespaces=ns))
    print(pointprob_num)

    path = "string(//x:dimension[@axis='row']/x:category[@text='Point Probability']/x:dimension[@axis='column']/x:category/x:cell/@text)"
    pointprob_str = str2float(statsout.xpath(path, namespaces=ns))
    print(pointprob_str)

varname1short = str(var1)
varname1long = str(var1)
if varlab1 != '':
    varname1long = varlab1 + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(var2)
varname2long = str(var2)
if varlab2 != '':
    varname2long = varlab2 + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

introhtml = '<h2>The Wilcoxon Sign Rank Test</h2>'
introhtml += '<p>'
introhtml += 'In this practical we are going to investigate how to perform a Wilcoxon test using SPSS. '
introhtml += 'A Wilcoxon test is used when we have two interval or ratio level variables measured for a set of observations and we want to test if the distribution is different for the two variables but we are unable to assume normality for one or both of the variables. ' 
introhtml += 'It can also be to compare ordered categorical variables. '
introhtml += 'It is the non-parametric equivalent of the paired t-test but unlike the t-test it tests differences in the median rather than the mean. '
introhtml += 'The test does not assume any distribution for the two variables. '
introhtml += 'To run a single test in SPSS requires that your dataset has two column containing the two variables to be compared. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

norminstrhtml = '<p>'
norminstrhtml += 'Before we can perform this test we need to check whether the differences between ' + varname1short + ' and ' + varname2short + ' are normally distributed. '
norminstrhtml += 'First we need to create a difference variable as follows: '
norminstrhtml += '</p>'

norminstrhtml += '<ul class = "instructions">'
norminstrhtml += '<li>Select <strong>Compute</strong> from the <strong>Transform</strong> menu.</li>'
norminstrhtml += '<li>Type <strong>DIFF_' + varname1short + '_' + varname2short + '</strong> into the <strong>Target Variable</strong> box.</li>'
norminstrhtml += '<li>Type <strong>' + varname1short + ' - ' + varname2short + '</strong> into the <strong>Numeric Expression</strong> box.</li>'
norminstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
norminstrhtml += '</ul>'

norminstrhtml += '<p>'
norminstrhtml += 'We can now use this new generated variable to perform normality checks. '
norminstrhtml += 'Do this as follows: '
norminstrhtml += '</p>'

norminstrhtml += '<ul class = "instructions">'
norminstrhtml += '<li>Select <strong>Descriptive Statistics</strong> from the <strong>Analyze</strong> menu.</li>'
norminstrhtml += '<li>Select <strong>Explore</strong> from the <strong>Descriptive Statistics</strong> sub-menu.</li>'
norminstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
norminstrhtml += '<li>Copy the <strong>DIFF_' + varname1short + '_' + varname2short + '</strong> variable into the <strong>Dependent List:</strong> box.</li>'
norminstrhtml += '<li>Click on the <strong>Plots...</strong> button.</li>'
norminstrhtml += '<li>On the screen that appears select the <strong>Histogram</strong> tick box.</li>'
norminstrhtml += '<li>Unselect the <strong>Stem and leaf</strong> button.</li>'
norminstrhtml += '<li>Select the <strong>Normality plots with tests</strong> button.</li>'
norminstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
norminstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
norminstrhtml += '</ul>'

normprachtml = norminstrhtml

normprachtml += '<p>'
normprachtml += 'We will first look at a histogram of the variable, <strong>DIFF_' + varname1short + '_' + varname2short + '</strong>. '
normprachtml += 'This can be found in amongst the set of output objects and looks as follows:'
normprachtml += '</p>'

normprachtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'

normprachtml += '<p>'
normprachtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    normprachtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num > 2*skewses_num:
        normprachtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normprachtml += 'This distribution appears to be reasonably symmetric. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'We will next look at a statistical test to see if this backs up our visual impressions from the histogram. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'The Kolmogorov-Smirnov test is used to test the null hypothesis that a set of data comes from a Normal distribution. '
normprachtml += '</p>'

normprachtml += outputs['Table_Tests of Normality.htm'].content

normprachtml += '<p>'
normprachtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normprachtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str + '. '
normprachtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
normprachtml += '</p>'

normprachtml += '<p>'
if kmsigs_num > 0.05:
    normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov)  is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normprachtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    normprachtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'Although the Kolmogorov Smirnov statistic tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normprachtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normprachtml += '</p>'

normprachtml += '<p>'
normprachtml += 'SPSS also supplies QQ plots to assist in looking at normality but for brevity we do not show them here.'
normprachtml += '</p>'

outputs['normality.html'] = HTMLOutput(normprachtml, "Normality test text")

normtestquesthtml = '<p>'
normtestquesthtml += 'A first step to perform a Wilcoxon test in SPSS is to confirm that one cannot use a paired t test. '
normtestquesthtml += 'To do this we construct a variable that is the difference between the two variables to be compared, in this case ' + varname1long + ' and ' + varname2long + '. '
normtestquesthtml += 'Do this now and then use SPSS to test for normality of this difference variable. '
normtestquesthtml += '</p>' 
normtestquesthtml += '<ul>'
normtestquesthtml += '<li> Question: Do the histogram and normality tests suggest that the difference variable follows a normal distribution?</li>'
normtestquesthtml += '</ul>'

outputs['normalityquestions.html'] = HTMLOutput(normtestquesthtml, "Normality test question text")

normtestsolhtml = '<p>'
normtestsolhtml += 'To construct the difference variable and test it for normality: '
normtestsolhtml += '</p>'
normtestsolhtml += norminstrhtml
normtestsolhtml += '<p>'
normtestsolhtml += 'Question: Do the histogram and normality tests suggest that the difference variable follows a normal distribution?'
normtestsolhtml += '</p>'
normtestsolhtml += '<p>'
normtestsolhtml += 'Solution: The output from SPSS is as follows:'
normtestsolhtml += '</p>'
normtestsolhtml += '<img src="data:image/png;base64,' + base64.b64encode(outputs['Chart_000.png'].content)  + '">'
normtestsolhtml += '<p>'
normtestsolhtml += 'Ideally for a normal distribution this histogram should look symmetric around the mean of the distribution, in this case ' + mean_str + '. '
if skews_num > 2*skewses_num:
    normtestsolhtml += 'This distribution appears to be significantly skewed to the right (positively skewed). '
else:
    if -skews_num > 2*skewses_num:
        normtestsolhtml += 'This distribution appears to be significantly skewed to the left (negatively skewed). '
    else:
        normtestsolhtml += 'This distribution appears to be reasonably symmetric. '
normtestsolhtml += '</p>'

normtestsolhtml += outputs['Table_Tests of Normality.htm'].content

normtestsolhtml += '<p>'
normtestsolhtml += 'The Kolmogorov Smirnov test produces test statistics that are used (along with a degrees of freedom parameter) to test for normality. '
normtestsolhtml += 'Here we see that the Kolmogorov Smirnov statistic takes value ' + kmstats_str + '. '
normtestsolhtml += 'This has degrees of freedom which equals the number of data points, namely ' + str(kmdfs) + '. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
if kmsigs_num > 0.05:
    normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov)  is ' + kmsigs_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the distribution is normal. '
else:
    if kmsigs_num < 0.001:
        normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' (reported as p &lt; .001) which is less than 0.05. '
    else:
        normtestsolhtml += 'Here we see that the p value (quoted under Sig. for Kolmogorov Smirnov) is ' + kmsigs_str + ' which is less than 0.05. '
    normtestsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the variable follows a normal distribution. '
normtestsolhtml += '</p>'

normtestsolhtml += '<p>'
normtestsolhtml += 'Although the Kolmogorov Smirnov statistic tells the researcher whether the distribution followed by a variable is statistically significantly different from a normal distribution one should take care in not overinterpreting such findings. '
normtestsolhtml += 'Significance will be strongly effected by the number of observations and so only a small discrepancy from normality will be deemed significant for very large sample sizes whilst very large discrepancies will be required to reject the null hypothesis for small sample sizes. '
normtestsolhtml += '</p>'

outputs['normalityanswers.html'] = HTMLOutput(normtestsolhtml, "Normality test answers text")

rankprachtml = '<p>'
rankprachtml += 'We will next move on to the Wilcoxon test itself and will test for a difference in distribution between the two variables, ' + varname1short + ' and ' + varname2short + '. '
rankprachtml += '</p>'

rankprachtml += '<p>'
rankprachtml += 'Below you will see instructions on how to perform the Wilcoxon test in SPSS. '
rankprachtml += 'If you follow the instructions you will see the two tabular outputs that are embedded in the explanations below.'
rankprachtml += '</p>'

rankinstrhtml = '<ul class = "instructions">'
rankinstrhtml += '<li>Select <strong>Non Parametric Tests</strong> from the <strong>Analyze</strong> menu.</li>'
rankinstrhtml += '<li>Select <strong>Legacy Dialogs</strong> from the <strong>Non Parametric Tests</strong> sub-menu.</li>'
rankinstrhtml += '<li>Select <strong>2 Related Samples...</strong> from the <strong>Legacy Dialogs</strong> sub-menu.</li>'
rankinstrhtml += '<li>Click on the <strong>Reset</strong> button.</li>'
rankinstrhtml += '<li>Copy the ' + varname1long + ' variable into the <strong>Test Pairs:</strong> box.</li>'
rankinstrhtml += '<li>Copy the ' + varname2long + ' variable into the <strong>Test Pairs:</strong> box.</li>'
rankinstrhtml += '<li>Click on the <strong>Exact...</strong> button.</li>'
rankinstrhtml += '<li>On the screen that appears select the <strong>Exact</strong> button.</li>'
rankinstrhtml += '<li>Click on the <strong>Continue</strong> button.</li>'
rankinstrhtml += '<li>Click on the <strong>OK</strong> button.</li>'
rankinstrhtml += '</ul>'

rankprachtml += rankinstrhtml

rankprachtml += '<p>'
rankprachtml += 'The first SPSS output table contains a summary of the rankings for the difference variable. '
rankprachtml += 'Here observations are split into three types depending on whether the value of ' + varname1short + ' is bigger than ' + varname2short + ' (negative ranks), the value of ' + varname2short + ' is bigger than ' + varname1short + ' (positive ranks), and finally where both variables take the same value (ties). '
rankprachtml += 'These can be seen below:'
rankprachtml += '</p>'

rankprachtml += outputs['Table_Wilcoxon Ranks.htm'].content

rankprachtml += '<p>'
rankprachtml += 'The Wilcoxon test works by firstly assigning a sign (or a tie) to the difference between each pair of observations. '
# Better to have labels here
rankprachtml += 'Here we have worked on ' + varname2short + ' - ' + varname1short + ' so that positive ranks are when ' + varname2short + ' &gt; ' + varname1short + '. '
rankprachtml += 'Here we see that there are ' + str(Nneg) + ' negative ranks, ' + str(Npos) + ' positive ranks and ' + str(Nties) + ' ties. '
rankprachtml += '</p>'

rankprachtml += '<p>'
rankprachtml += 'Having worked out which observed pairs result in which sign for their difference, the magnitude (excluding the sign) of these differences is calculated and these are then ranked in order (excluding ties). '
rankprachtml += 'We now see that the total of the ranks for the negative differences is ' + sumneg_str + ' resulting in a mean rank of ' + meanneg_str + ' whilst the total of the ranks for the positive differences is ' + sumpos_str + ' resulting in a mean rank of ' + meanpos_str + '. '
if meanneg_num > meanpos_num:
    rankprachtml += 'Here the mean of the negative ranks is larger than that for positive ranks suggesting that values for ' + varname1short + ' are generally larger than for ' + varname2short + '. '
else:
    rankprachtml += 'Here the mean of the positive ranks is larger than that for negative ranks suggesting that values for ' + varname2short + ' are generally larger than for ' + varname1short + '. '

rankprachtml += '</p>'
outputs['rank.html'] = HTMLOutput(rankprachtml, "Ranks text")

rankquesthtml = '<p>'
rankquesthtml += 'Next perform the Wilcoxon test in SPSS to compare differences between ' + varname1long + ' and ' + varname2long + '. '
rankquesthtml += '</p>'
rankquesthtml += '<ul>'
rankquesthtml += '<li>Question: Looking at the first SPSS table which variable has the higher mean rank?</li>'
rankquesthtml += '</ul>'

outputs['rankquestions.html'] = HTMLOutput(rankquesthtml, "Ranks questions text")

ranksolhtml = '<p>'
ranksolhtml += 'Here are the instructions to perform the Wilcoxon test in SPSS: '
ranksolhtml += '</p>'
ranksolhtml += rankinstrhtml
ranksolhtml += '<ul>'
ranksolhtml += '<li>Question: Looking at the first SPSS table which variable has the higher mean rank?</li>'
ranksolhtml += '</ul>'
ranksolhtml += '<p>'
ranksolhtml += 'Solution: The output from SPSS is as follows:'
ranksolhtml += '</p>'
ranksolhtml += outputs['Table_Wilcoxon Ranks.htm'].content
ranksolhtml += '<p>'
ranksolhtml += 'Here we have worked on ' + varname2short + ' - ' + varname1short + ' so that positive ranks are when ' + varname2short + ' &gt; ' + varname1short + '. '
ranksolhtml += 'Here we see that there are ' + str(Nneg) + ' negative ranks, ' + str(Npos) + ' positive ranks and ' + str(Nties) + ' ties. '
ranksolhtml += '</p>'

ranksolhtml += '<p>'
ranksolhtml += 'Having worked out which observed pairs result in which sign for their difference, the magnitude (excluding the sign) of these differences is calculated and these are then ranked in order (excluding ties). '
ranksolhtml += 'We now see that the total of the ranks for the negative differences is ' + sumneg_str + ' resulting in a mean rank of ' + meanneg_str + ' while the total of the ranks for the positive differences is ' + sumpos_str + ' resulting in a mean rank of ' + meanpos_str + '. '
if meanneg_num > meanpos_num:
    ranksolhtml += 'Here the mean of the negative ranks is larger than that for positive ranks suggesting that values for ' + varname1short + ' are generally larger than for ' + varname2short + '. '
else:
    ranksolhtml += 'Here the mean of the positive ranks is larger than that for negative ranks suggesting that values for ' + varname2short + ' are generally larger than for ' + varname1short + '. '
ranksolhtml += '</p>'

outputs['rankanswers.html'] = HTMLOutput(ranksolhtml, "Ranks answers text")


testprachtml = '<p>'
testprachtml += 'The Wilcoxon test will now decide whether this difference in mean ranks is significant or not as is illustrated in the second table. '
testprachtml += '</p>'

testprachtml += '<p>'
testprachtml += 'The second SPSS output table contains details of the test itself and can be seen below:'
testprachtml += '</p>'

testprachtml += outputs['Table_Wilcoxon Test Statistics.htm'].content

testprachtml += '<p>'
testprachtml += 'The output here consists of test statistics and their significance as calculated in several ways. '
testprachtml += 'We are considering the Wilcoxon statistic which is calculated from the ranks and is not shown explicitly by SPSS but is used to calculate a Z score. ' 
testprachtml += 'Here we see that Z = ' + Z_str + ' and this can be compared with a standard normal distribution to test whether there are significant differences between the groups. '
testprachtml += '</p>'

testprachtml += '<p>'
if asympsig2_num > 0.05:
    testprachtml += 'Here we see that the p value (quoted next to Asymp. Sig. (2-tailed)) is ' + asympsig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the medians of the two groups are the same. '
else:
    if asympsig2_num < 0.001:
        testprachtml += 'Although SPSS quotes the p value (quoted next to Asymp. Sig. (2-tailed)) as ' + asympsig2_str + ' it is not exactly zero and is in fact simply smaller than 0.001 as SPSS is quoting the first 3 decimal places. '
    else:
        testprachtml += 'Here we see that the p value (quoted next to Asymp. Sig. (2-tailed)) is ' + asympsig2_str + ' which is less than 0.05. '
    testprachtml += 'We therefore have significant evidence to reject the null hypothesis that the two groups have the same medians. '
if exactsig2_str != '':
    testprachtml += 'The normal approximation used above is only an approximation to the p value and it is possible to construct the exact p value. '
    testprachtml += 'This is given in the next row and we see that the exact p value is ' + exactsig2_str + ' whilst the asymptotic p value is ' + asympsig2_str + '. '
    if asympsig2_num > 0.05:
        if exactsig2_num > 0.05:
             testprachtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis cannot be rejected. '
        else:
             testprachtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis can be rejected. '
             testprachtml += 'In this case we would use the exact result. '
    if asympsig2_num < 0.05:
        if exactsig2_num < 0.05:
             testprachtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis can be rejected. '
        else:
             testprachtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis cannot be rejected. '
             testprachtml += 'In this case we would use the exact result. '
    testprachtml += 'For completeness the table also gives a p value for a 1-sided test and a point probability but we will ignore these here. '
else:
    testprachtml += 'The normal approximation used above is only an approximation to the p value however in this case the sample sizes are so large that the exact test was to computational to complete but the asymptotic p value should be accurate enough. '
testprachtml += '</p>'
outputs['test.html'] = HTMLOutput(testprachtml, "Test statistics text")

testquesthtml = '<ul>'
testquesthtml += '<li>Question: Looking at the second SPSS table does this suggest there are significant differences between the variables?</li>'
testquesthtml += '</ul>'

outputs['testquestions.html'] = HTMLOutput(testquesthtml, "Test statistics questions text")

testsolhtml = '<ul>'
testsolhtml += '<li>Question: Looking at the second SPSS table does this suggest there are significant differences between the variables?</li>'
testsolhtml += '</ul>'
testsolhtml += '<p>'
testsolhtml += 'Solution: The output from SPSS is as follows:'
testsolhtml += '</p>'
testsolhtml += outputs['Table_Wilcoxon Test Statistics.htm'].content
testsolhtml += '<p>'
if asympsig2_num > 0.05:
    testsolhtml += 'Here we see that the p value (quoted next to Asymp. Sig. (2-tailed)) is ' + asympsig2_str + ' which is greater than 0.05 and therefore we cannot reject the null hypothesis that the medians of the two groups are the same. '
else:
    if asympsig2_num < 0.001:
        testsolhtml += 'Although SPSS quotes the p value (quoted next to Asymp. Sig. (2-tailed)) as ' + asympsig2_str + ' it is not exactly zero and is in fact simply smaller than 0.001 as SPSS is quoting the first 3 decimal places. '
    else:
        testsolhtml += 'Here we see that the p value (quoted next to Asymp. Sig. (2-tailed)) is ' + asympsig2_str + ' which is less than 0.05. '
    testsolhtml += 'We therefore have significant evidence to reject the null hypothesis that the two groups have the same medians. '
if exactsig2_str != '':
    testprachtml += 'The normal approximation used above is only an approximation to the p value and it is possible to construct the exact p value. '
    testsolhtml += 'This is given in the next row and we see that the exact p value is ' + exactsig2_str + ' whilst the asymptotic p value is ' + asympsig2_str + '. '
    if asympsig2_num > 0.05:
        if exactsig2_num > 0.05:
             testsolhtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis cannot be rejected. '
        else:
             testsolhtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis can be rejected. '
             testsolhtml += 'In this case we would use the exact result. '
    if asympsig2_num < 0.05:
        if exactsig2_num < 0.05:
             testsolhtml += 'The exact p value agrees with the asymptotic p value that the null hypothesis can be rejected. '
        else:
             testsolhtml += 'Unusually the exact p value disagrees with the asymptotic p value and suggests that the null hypothesis cannot be rejected. '
             testsolhtml += 'In this case we would use the exact result. '
else:
    testsolhtml += 'The normal approximation used above is only an approximation to the p value however in this case the sample sizes are so large that the exact test was to computational to complete but the asymptotic p value should be accurate enough. '
testsolhtml += '</p>'

outputs['testanswers.html'] = HTMLOutput(testsolhtml, "Test statistics answers text")

conchtml = '<p>'
conchtml += 'In conclusion, we could report this to a reader as follows:'
conchtml += '</p>'

conchtml += '<p>'
conchtml += 'A comparison of the mean of the distribution of the variables ' + varname1short + ' and ' + varname2short + ' was desired but due to the non-normality of the variables a Wilcoxon signed rank test was carried out. '
if meanneg_num > meanpos_num:
    conchtml += 'The mean of the negative ranks is larger than that for positive ranks suggesting that values for ' + varname1short + ' are generally larger than for ' + varname2short + '. '
if meanpos_num > meanneg_num:
    conchtml += 'The mean of the positive ranks is larger than that for negative ranks suggesting that values for ' + varname2short + ' are generally larger than for ' + varname1short + '. '
if meanpos_num == meanneg_num:
    conchtml += 'Very unusually both negative and positive have identical mean ranks. '
conchtml += 'The Wilcoxon Signed rank test results in a Z statistic of ' + Z_str + ' '
if exactsig2_str != '':
   conchtml += 'which results in an exact p value of '
   if exactsig2_num < 0.001:
       conchtml += 'less than 0.001. '
   else:
       conchtml += exactsig2_str + '. '
   if exactsig2_num < 0.05:
       conchtml += 'This is significant and we can reject the null hypothesis of equal medians for the 2 variables. '
   else:
       conchtml += 'This is not significant and we cannot reject the null hypothesis of equal medians for the 2 variables. '
else:
   conchtml += 'which results in an asypmtotic p value of '
   if asympsig2_num < 0.001:
       conchtml += 'less than 0.001. '
   else:
       conchtml += asympsig2_str + '. '
   if asympsig2_num < 0.05:
       conchtml += 'This is significant and we can reject the null hypothesis of equal medians for the 2 variables. '
   else:
       conchtml += 'This is not significant and we cannot reject the null hypothesis of equal medians for the 2 variables. '
conchtml += '</p>'
    
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")


combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['normality.html'].content
combinedprachtml += outputs['rank.html'].content
combinedprachtml += outputs['test.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['normalityquestions.html'].content
combinedquesthtml += outputs['rankquestions.html'].content
combinedquesthtml += outputs['testquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['normalityanswers.html'].content
combinedsolhtml += outputs['rankanswers.html'].content
combinedsolhtml += outputs['testanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")

'''

    spssgui = r'''
\begin{aligned}
&\text{Select }\textbf{Non Parametric Tests}\text{ from the }\textbf{Analyze}\text{ menu.} \\ 
&\text{Select }\textbf{Legacy Dialogs}\text{ from the }\textbf{Non Parametric Tests}\text{ sub-menu.} \\ 
&\text{Select }\textbf{2 Related Samples...}\text{ from the }\textbf{Legacy Dialogs}\text{ sub-menu.} \\ 
&\text{Click on the } \textbf{Reset}\text{ button } \\ 
&\text{Copy the } \textbf{${datafile.variables[var1]['varlabel'] + ' ['+var1+']'}}\text{ variable into the } \textbf{Test Pairs:}\text{ box. } \\ 
&\text{Copy the } \textbf{${datafile.variables[var2]['varlabel'] + ' ['+var2+']'}}\text{ variable into the } \textbf{Test Pairs:}\text{ box. } \\ 
&\text{Click on the } \textbf{Exact...}\text{ button } \\ 
&\text{On the screen that appears select the } \textbf{Exact}\text{ button } \\ 
&\text{Click on the } \textbf{Continue}\text{ button } \\ 
&\text{Click on the } \textbf{OK}\text{ button } \\  
\end{aligned}
'''